package com.agilex.healthcare.notewriter.datalayer;

import java.util.Calendar;
import java.util.Date;
import java.util.UUID;

import javax.annotation.Resource;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.Count;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.notewriter.translator.ClinicalEventUtilities;
import com.agilex.healthcare.utility.NullChecker;
import com.mongodb.MongoClient;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class ClinicalEventDataLayerMongoTest {

	@Resource
	MongoClient mongoClient;

	@Resource
	ClinicalEventDataLayer dataLayer;

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(ClinicalEventDataLayerMongo.class);

	@Before
	public void setup() {
		ClinicalEventUtilities.removeTestDataFromMongo(mongoClient,LOGGER);
	}

	private ClinicalEvent createAndSaveNewClinicalEvent(ClinicalEventDataLayer dataLayer, Date time) {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(time);
		ClinicalEvent savedEvent = dataLayer.saveClinicalEvent(clinicalEvent);
		return savedEvent;
	}

	@Test
	public void testSavedDataEqualsOriginalDataMinimal() {
		compareSavedDataToOriginalData(ClinicalEventUtilities.createClinicalEventMinimal(Calendar.getInstance().getTime()));
	}

	@Test
	public void testSavedDataEqualsOriginalData() {
		compareSavedDataToOriginalData(ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime()));
	}

	private void compareSavedDataToOriginalData(ClinicalEvent clinicalEvent) {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		ClinicalEvent savedEvent = dataLayer.saveClinicalEvent(clinicalEvent);
		clinicalEvent.setId(savedEvent.getId());
		Assert.assertEquals(clinicalEvent, savedEvent);
	}

	@Test
	public void testSaveClinicalEvent() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();

		Date time = Calendar.getInstance().getTime();
		ClinicalEvent savedEvent = createAndSaveNewClinicalEvent(dataLayer, time);

		ClinicalEventUtilities.assertClinicalEvent(savedEvent, time);
	}

	@Test
	public void testFetchClinicalEvent() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		Calendar cal = Calendar.getInstance();
		Date time = cal.getTime();

		int count = 10;
		for (int i=0; i<count; i++) {
			cal.add(Calendar.MINUTE, -1*(count-i));
			createAndSaveNewClinicalEvent(dataLayer, cal.getTime());
		}

		ClinicalEvent clinicalEvent = createAndSaveNewClinicalEvent(dataLayer, time);

		cal.add(Calendar.DATE, -1);
		count = 7;
		for (int i=0; i<count; i++) {
			cal.add(Calendar.MINUTE, -1*(count-i));
			createAndSaveNewClinicalEvent(dataLayer, cal.getTime());
		}

		ClinicalEvent foundEvent = dataLayer.fetchClinicalEvent(clinicalEvent.getProviderIdentifier().getFacilityCode(),
				clinicalEvent.getProviderIdentifier().getStaffId(),
				clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
				clinicalEvent.getPatientIdentifier().getUniqueId(),
				clinicalEvent.getId());

		ClinicalEventUtilities.assertClinicalEvent(foundEvent, time);
	}

	@Test
	public void testFetchLatestClinicalEvent() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		Calendar cal = Calendar.getInstance();
		Date time = cal.getTime();

		int count = 10;
		for (int i=0; i<count; i++) {
			cal.add(Calendar.MINUTE, -1*(count-i));
			createAndSaveNewClinicalEvent(dataLayer, cal.getTime());
		}

		ClinicalEvent clinicalEvent = createAndSaveNewClinicalEvent(dataLayer, time);

		ClinicalEvent foundEvent = dataLayer.fetchLatestClinicalEvent(clinicalEvent.getProviderIdentifier().getFacilityCode(),
				clinicalEvent.getProviderIdentifier().getStaffId(),
				clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
				clinicalEvent.getPatientIdentifier().getUniqueId());

		ClinicalEventUtilities.assertClinicalEvent(foundEvent, time);
	}

	@Test
	public void testGetClinicalEventCount() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		Date time = Calendar.getInstance().getTime();

		int count = 10;
		for (int i=0; i<count; i++) {
			createAndSaveNewClinicalEvent(dataLayer, time);
		}

		ClinicalEvent clinicalEvent = createAndSaveNewClinicalEvent(dataLayer, time);

		ClinicalEvent foundEvent = dataLayer.fetchLatestClinicalEvent(clinicalEvent.getProviderIdentifier().getFacilityCode(),
				clinicalEvent.getProviderIdentifier().getStaffId(),
				clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
				clinicalEvent.getPatientIdentifier().getUniqueId());

		ClinicalEventUtilities.assertClinicalEvent(foundEvent, time);

		Count clinicalEventCount = dataLayer.getClinicalEventCount(clinicalEvent.getProviderIdentifier().getFacilityCode(),
				clinicalEvent.getProviderIdentifier().getStaffId(),
				clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
				clinicalEvent.getPatientIdentifier().getUniqueId());
		Assert.assertEquals(11, clinicalEventCount.getValue());
	}

	@Test
	public void testUpdateClinicalEvent() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		Calendar cal = Calendar.getInstance();
		Date time = cal.getTime();

		int count = 10;
		for (int i=0; i<count; i++) {
			cal.add(Calendar.MINUTE, -1*(count-i));
			createAndSaveNewClinicalEvent(dataLayer, cal.getTime());
		}

		ClinicalEvent clinicalEvent = createAndSaveNewClinicalEvent(dataLayer, time);

		cal.add(Calendar.DATE, -1);
		count = 7;
		for (int i=0; i<count; i++) {
			cal.add(Calendar.MINUTE, -1*(count-i));
			createAndSaveNewClinicalEvent(dataLayer, cal.getTime());
		}

		assertUpdatedClinicalEvent(dataLayer, clinicalEvent);
	}

	private void assertUpdatedClinicalEvent(ClinicalEventDataLayer dataLayer, ClinicalEvent clinicalEvent) {
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -2);
		Date time = cal.getTime();
		ClinicalEvent changesToEvent = ClinicalEventUtilities.createClinicalEvent(time);

		//backup the original query info
		String orgFacId = changesToEvent.getProviderIdentifier().getFacilityCode();
		String orgStaffId = changesToEvent.getProviderIdentifier().getStaffId();
		String orgAssId = changesToEvent.getPatientIdentifier().getAssigningAuthority();
		String orgUniqueId = changesToEvent.getPatientIdentifier().getUniqueId();

		//Now, replace that query info with one from an existing clinical event
		changesToEvent.setId(clinicalEvent.getId());
		changesToEvent.getProviderIdentifier().setFacilityCode(clinicalEvent.getProviderIdentifier().getFacilityCode());
		changesToEvent.getProviderIdentifier().setStaffId(clinicalEvent.getProviderIdentifier().getStaffId());
		changesToEvent.getPatientIdentifier().setAssigningAuthority(clinicalEvent.getPatientIdentifier().getAssigningAuthority());
		changesToEvent.getPatientIdentifier().setUniqueId(clinicalEvent.getPatientIdentifier().getUniqueId());

		//Update the old event with the new information.
		ClinicalEvent updatedEvent = dataLayer.updateClinicalEvent(changesToEvent);

		//Assert that the query info values of the updatedEvent match those that we set earlier.
		Assert.assertEquals(changesToEvent.getId(), updatedEvent.getId());
		Assert.assertEquals(changesToEvent.getProviderIdentifier().getFacilityCode(), updatedEvent.getProviderIdentifier().getFacilityCode());
		Assert.assertEquals(changesToEvent.getProviderIdentifier().getStaffId(), updatedEvent.getProviderIdentifier().getStaffId());
		Assert.assertEquals(changesToEvent.getPatientIdentifier().getAssigningAuthority(), updatedEvent.getPatientIdentifier().getAssigningAuthority());
		Assert.assertEquals(changesToEvent.getPatientIdentifier().getUniqueId(), updatedEvent.getPatientIdentifier().getUniqueId());

		//Now, we are going to replace the query info values with the original query so we can re-use the assertClinicalEvent
		//method to test that all of the other fields within the updatedEvent got set properly.
		updatedEvent.getProviderIdentifier().setFacilityCode(orgFacId);
		updatedEvent.getProviderIdentifier().setStaffId(orgStaffId);
		updatedEvent.getPatientIdentifier().setAssigningAuthority(orgAssId);
		updatedEvent.getPatientIdentifier().setUniqueId(orgUniqueId);
		ClinicalEventUtilities.assertClinicalEvent(updatedEvent, time);
	}

	private String NOTE_TITLES_FACILITY_CODE = "500";
	private String NOTE_TITLES_STAFF_ID = "staff046";

	@Test
	public void testFetchNoteTitles() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		SearchFilterFactory factory = new SearchFilterFactory();
		SearchFilter filter = factory.createEmptySearchFilter();
		NoteTitles titles = dataLayer.getMongoNoteTitles(NOTE_TITLES_FACILITY_CODE, filter);

		Assert.assertNotNull(titles);
		Assert.assertTrue(titles.getSize() > 0);

		String previousName = "";
		for (NoteTitle title : titles) {
			Assert.assertTrue(NullChecker.isNotNullish(title.getIen()));
			Assert.assertTrue(NullChecker.isNotNullish(title.getName()));

			Assert.assertTrue(previousName.compareTo(title.getName()) <= 0);
			previousName = title.getName();
		}
	}

	@Test
	public void testFetchNoteTitlesWithFilterMaxRecords() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		SearchFilterFactory factory = new SearchFilterFactory();
		SearchFilter filter = factory.createEmptySearchFilter();
		filter.setSearchString("ph");
		filter.setMaxRecords(10);
		NoteTitles titles = dataLayer.getMongoNoteTitles(NOTE_TITLES_FACILITY_CODE, filter);

		Assert.assertNotNull(titles);
		Assert.assertEquals(10, titles.getSize());

		String previousName = "";
		for (NoteTitle title : titles) {
			Assert.assertTrue(NullChecker.isNotNullish(title.getIen()));
			Assert.assertTrue(NullChecker.isNotNullish(title.getName()));

			Assert.assertTrue(previousName.compareTo(title.getName()) <= 0);
			previousName = title.getName();
		}
	}

	@Test
	public void testFetchNoteTitlesWithFilter() {
		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		SearchFilterFactory factory = new SearchFilterFactory();
		SearchFilter filter = factory.createEmptySearchFilter();
		filter.setSearchString("ph");
		NoteTitles titles = dataLayer.getMongoNoteTitles(NOTE_TITLES_FACILITY_CODE, filter);

		Assert.assertNotNull(titles);
		Assert.assertTrue(titles.getSize() > 0);

		String previousName = "";
		for (NoteTitle title : titles) {
			Assert.assertTrue(NullChecker.isNotNullish(title.getIen()));
			Assert.assertTrue(NullChecker.isNotNullish(title.getName()));

			Assert.assertTrue(title.getName().toLowerCase().contains(filter.getSearchString().toLowerCase()));

			Assert.assertTrue(previousName.compareTo(title.getName()) <= 0);
			previousName = title.getName();
		}
	}

	@Test
	public void testCreateAndFetchLastSelectedNoteTitle() {
		String uniqueStaffId = UUID.randomUUID().toString();
		String ien = "123";
		String titleName = "createNoteTitle";
		NoteTitle newNoteTitle = new NoteTitle(ien, titleName);

		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		NoteTitle savedNoteTitle = dataLayer.insertLastSelectedNoteTitle(NOTE_TITLES_FACILITY_CODE, uniqueStaffId, newNoteTitle);
		Assert.assertNotNull(savedNoteTitle);
		Assert.assertEquals(ien, savedNoteTitle.getIen());
		Assert.assertEquals(titleName, savedNoteTitle.getName());

		NoteTitle title = dataLayer.fetchLastSelectedNoteTitle(NOTE_TITLES_FACILITY_CODE, uniqueStaffId);
		Assert.assertNotNull(title);
		Assert.assertEquals(ien, title.getIen());
		Assert.assertEquals(titleName, title.getName());
	}

	@Test
	public void testUpdateAndFetchLastSelectedNoteTitle() {
		String ien = "123";
		String uniqueTitleName = UUID.randomUUID().toString();
		NoteTitle uniqueNoteTitle = new NoteTitle(ien, uniqueTitleName);

		//ClinicalEventDataLayer dataLayer = new ClinicalEventDataLayerMongo();
		NoteTitle title = dataLayer.fetchLastSelectedNoteTitle(NOTE_TITLES_FACILITY_CODE, NOTE_TITLES_STAFF_ID);

		if (title == null) {
			title = dataLayer.insertLastSelectedNoteTitle(NOTE_TITLES_FACILITY_CODE, NOTE_TITLES_STAFF_ID, new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>"));
		}
		Assert.assertNotNull(title);

		NoteTitle updatedNoteTitle = dataLayer.updateLastSelectedNoteTitle(NOTE_TITLES_FACILITY_CODE, NOTE_TITLES_STAFF_ID, uniqueNoteTitle);
		Assert.assertNotNull(updatedNoteTitle);
		Assert.assertEquals(ien, updatedNoteTitle.getIen());
		Assert.assertEquals(uniqueTitleName, updatedNoteTitle.getName());

	}

	private Date getDate(int year, int month, int day) {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.YEAR, year);
		cal.set(Calendar.MONTH, month);
		cal.set(Calendar.DAY_OF_MONTH, day);
		Date time = cal.getTime();
		return time;
	}

	/**
	 *     PV-1872 fetchLatestClinicalEvent does not return the latest clinical event.
	 *     In the example below (before the bug was fixed, Nov 30, 2013 Clinical Event would be returned
	 *     instead of January 31, 2014 as the latest.
	 *     This was due to them saving it as a text string and 12-30-2013 is textually later than 01-31-2013
	 *     since the 1 in 12 comes after the 0 in 01.
	 */
	@Test
	public void testFetchLatestClinicalEventSortsCorrectly() {

		//Save a clinical event in November and December
		Date time2013Nov = getDate(2013, Calendar.NOVEMBER, 30);
		Date time2013Dec = getDate(2013, Calendar.DECEMBER, 31);
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(time2013Dec);
		clinicalEvent.setLastModifiedDate(time2013Nov);
		dataLayer.saveClinicalEvent(clinicalEvent); //Save one for November.
		clinicalEvent.setLastModifiedDate(time2013Dec);
		ClinicalEvent savedClinicalEvent = dataLayer.saveClinicalEvent(clinicalEvent); //Save a new one for December

		//then update it in January of the next year.
		Date time2014Jan = getDate(2014, Calendar.JANUARY, 31);
		savedClinicalEvent.setLastModifiedDate(time2014Jan);
		dataLayer.updateClinicalEvent(savedClinicalEvent);

		//When we fetch the latest clinical event, it should be the one from January (not the one from December).
		ClinicalEvent foundEvent = dataLayer.fetchLatestClinicalEvent(clinicalEvent.getProviderIdentifier().getFacilityCode(),
				clinicalEvent.getProviderIdentifier().getStaffId(),
				clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
				clinicalEvent.getPatientIdentifier().getUniqueId());

		Date expected = time2014Jan;
		Date actual = foundEvent.getLastModifiedDate();
		Assert.assertEquals(expected, actual);
	}
}
