package com.agilex.healthcare.patientsearch.datalayer;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Calendar;
import java.util.Date;

import javax.annotation.Resource;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.domain.DOBDate;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class PatientSearchDataLayerMongoTest {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(PatientSearchDataLayerMongoTest.class);

	@Resource
	PatientSearchDataLayer dataLayer;

	@Before
	public void setup() {
		new PatientSearchUtilities().removeTestDataFromMongo(LOGGER);
	}

	@After
	public void teardown() {
		new PatientSearchUtilities().removeTestDataFromMongo(LOGGER);
	}

	@Test
	public void testAddSelectedPatient() {
		String staffId = "staff046";
		Date searchedTime = getDate(2014, Calendar.JANUARY, 31);
		String dateOfBirth = "MAY 05, 1900";
		SelectedPatient selectedPatient = addPatient(staffId, "EDIPI", "D123401", dateOfBirth, "zztest", "patient01", "999999901", "MALE", searchedTime);

		assertNotNull(selectedPatient);
		assertEquals(searchedTime, selectedPatient.getSearchedTime());

		assertNotNull(selectedPatient.getPatient());
		
		DOBDate dob = new DOBDate(null, dateOfBirth);
		assertEquals(dob, selectedPatient.getPatient().getDateOfBirth());
		assertEquals("999999901", selectedPatient.getPatient().getSsn());
		assertEquals("MALE", selectedPatient.getPatient().getGender().toUpperCase());
		assertEquals("patient01, zztest", selectedPatient.getPatient().getDisplayName());
	}
	
	@Test
	public void testAddMultiplePatients() {
		String staffId = "staff046";
		
		Date searchedTime = getDate(2014, Calendar.JANUARY, 31);
		addPatient(staffId, "EDIPI", "D123401", "MAY 05, 1900", "zztest", "patient01", "999999901", "MALE", searchedTime);
		addPatient(staffId, "EDIPI", "D123402", "MAY 05, 1900", "zztest", "patient02", "999999902", "MALE", searchedTime);
		RecentlySelectedPatients recentlySelectedPatients = dataLayer.fetchRecentlySelectedPatients(PatientSearchUtilities.FACILITY_CODE, staffId);

		assertTrue("There should only be 2 patients returned.", recentlySelectedPatients.getSize() == 2);

		Date searchedTimeNew = getDate(2014, Calendar.FEBRUARY, 1);
		addPatient(staffId, "EDIPI", "D123401", "MAY 05, 1900", "zztest", "patient01", "999999901", "MALE", searchedTimeNew);
		recentlySelectedPatients = dataLayer.fetchRecentlySelectedPatients(PatientSearchUtilities.FACILITY_CODE, staffId);
		assertTrue("Expecting only 2 results since this last addition should have only changed the search time (not add a new patient)", recentlySelectedPatients.getSize() == 2);

		for (SelectedPatient selectedPatient : recentlySelectedPatients) {
			if ("D123402".equals(selectedPatient.getPatient().getPatientIdentifier().getUniqueId())) {
				assertEquals(searchedTime, selectedPatient.getSearchedTime());
				assertEquals("patient02, zztest", selectedPatient.getPatient().getDisplayName());
			} else {
				assertEquals(searchedTimeNew, selectedPatient.getSearchedTime());
				assertEquals("patient01, zztest", selectedPatient.getPatient().getDisplayName());
			}
		}
	}

	@Test
	public void testFetchRecentlySelectedPatients() {
		String staffId = "staff046";
		
		Date searchedTime = getDate(2014, Calendar.JANUARY, 31);
		addPatient(staffId, "EDIPI", "D123401", "MAY 05, 1900", "zztest", "patient01", "999999901", "MALE", searchedTime);
		addPatient(staffId, "EDIPI", "D123402", "MAY 05, 1900", "zztest", "patient02", "999999902", "MALE", searchedTime);
		addPatient(staffId, "EDIPI", "D123403", "MAY 05, 1900", "zztest", "patient03", "999999903", "MALE", searchedTime);
		RecentlySelectedPatients recentlySelectedPatients = dataLayer.fetchRecentlySelectedPatients(PatientSearchUtilities.FACILITY_CODE, staffId);

		assertTrue(recentlySelectedPatients.getSize() == 3);

		assertEquals(PatientSearchUtilities.FACILITY_CODE, recentlySelectedPatients.getProvider().getFacilityCode());
		assertEquals(staffId, recentlySelectedPatients.getProvider().getStaffId());

		for (SelectedPatient selectedPatient : recentlySelectedPatients) {
			assertNotNull(selectedPatient.getSearchedTime());
			assertNotNull(selectedPatient.getPatient());
			assertNotNull(selectedPatient.getPatient().getDisplayName());
		}
	}

	/**
	 *     PV-1872 fetchLatestClinicalEvent does not return the latest clinical event.
	 *     In the example below (before the bug was fixed, Nov 30, 2013 Clinical Event would be returned
	 *     instead of January 31, 2014 as the latest.
	 *     This was due to them saving it as a text string and 12-30-2013 is textually later than 01-31-2013
	 *     since the 1 in 12 comes after the 0 in 01.
	 */
	@Test
	public void testFetchLatestClinicalEventSortsCorrectly() {
		String staffId = "staff046";
		Date searchedTime = getDate(2013, Calendar.NOVEMBER, 30);
		addPatient(staffId, "EDIPI", "TEST DATA 1", "MAY 05, 1900", "TEST DATA", "TEST DATA", "TEST DATA", "MALE", searchedTime);
		searchedTime = getDate(2013, Calendar.DECEMBER, 31);
		addPatient(staffId, "EDIPI", "TEST DATA 2", "MAY 05, 1900", "TEST DATA", "TEST DATA", "TEST DATA", "MALE", searchedTime);

		//then update it in January of the next year.
		searchedTime = getDate(2014, Calendar.JANUARY, 31);
		addPatient(staffId, "EDIPI", "TEST DATA 3", "MAY 05, 1900", "TEST DATA", "TEST DATA", "TEST DATA", "MALE", searchedTime);
		
		//When we fetch the RecentlySelectedPatients, it should be the one from January (not the one from December).
		RecentlySelectedPatients recentlySelectedPatients = dataLayer.fetchRecentlySelectedPatients(PatientSearchUtilities.FACILITY_CODE, staffId);

		Assert.assertEquals(3, recentlySelectedPatients.getSize());
		Date actual = recentlySelectedPatients.get(0).getSearchedTime();
		Assert.assertEquals(searchedTime, actual);
	}

	@Test
	public void testAddMoreThanMaxRecords() {
		//If it's going to be more than 20 then we need to update this test since it changes dates based on
		//only one month.
		Assert.assertEquals(20,  PatientSearchDataLayerMongo.MAXIMUM_PATIENT_RECORDS);
		
		String staffId = "staff046";
		RecentlySelectedPatients expectedPats = new RecentlySelectedPatients();
		
		for (int i=1; i<=PatientSearchDataLayerMongo.MAXIMUM_PATIENT_RECORDS; i++) {
			Date searchedTime = getDate(2013, Calendar.NOVEMBER, i);
			String uniqueId = "TEST DATA " + i;
			String dateOfBirth = String.format("MAY 05, 19%02d", i);
			String firstName = "TEST FIRST " + i;
			String lastName = "TEST LAST " + i;
			String ssn = "TEST SSN " + i;
			SelectedPatient selectedPatient = addPatient(staffId, "EDIPI", uniqueId, dateOfBirth, firstName, lastName, ssn, "MALE", searchedTime);
			expectedPats.add(selectedPatient);
		}
		
		validatePatientsMatchFetch(staffId, expectedPats);
		
		//remove the first 3 of them from our expected patients since they are the 3 oldest.
		expectedPats.remove(0);
		expectedPats.remove(0);
		expectedPats.remove(0);
		
		//Add 3 more.
		int lastIdx = PatientSearchDataLayerMongo.MAXIMUM_PATIENT_RECORDS+1;
		for (int i=lastIdx; i<lastIdx+3; i++) {
			Date searchedTime = getDate(2013, Calendar.NOVEMBER, i);
			String uniqueId = "TEST DATA " + i;
			String dateOfBirth = String.format("MAY 05, 19%02d", i);
			String firstName = "TEST FIRST " + i;
			String lastName = "TEST LAST " + i;
			String ssn = "TEST SSN " + i;
			SelectedPatient selectedPatient = addPatient(staffId, "EDIPI", uniqueId, dateOfBirth, firstName, lastName, ssn, "MALE", searchedTime);
			expectedPats.add(selectedPatient);
		}
		
		validatePatientsMatchFetch(staffId, expectedPats);
	}

	private void validatePatientsMatchFetch(String staffId, RecentlySelectedPatients expectedPats) {
		RecentlySelectedPatients actualPats = dataLayer.fetchRecentlySelectedPatients(PatientSearchUtilities.FACILITY_CODE, staffId);
		
		assertEquals(expectedPats.size(), actualPats.size());
		
		for (int i=0; i<expectedPats.size(); i++) {
			SelectedPatient pat = expectedPats.get(i);
			boolean found = false;
			for (int k=0; k<actualPats.size(); k++) {
				SelectedPatient recentpat = actualPats.get(k);
				String expectedUniqueId = pat.getPatient().getPatientIdentifier().getUniqueId();
				String expectedAssigningAuthority = pat.getPatient().getPatientIdentifier().getAssigningAuthority();
				String actualUniqueId = recentpat.getPatient().getPatientIdentifier().getUniqueId();
				String actualAssigningAuthority = recentpat.getPatient().getPatientIdentifier().getAssigningAuthority();
				if (actualUniqueId.equalsIgnoreCase(expectedUniqueId) && actualAssigningAuthority.equalsIgnoreCase(expectedAssigningAuthority)) {
					found = true;
					break;
				}
			}
			
			if (found == false)
				Assert.fail("No patient was found for " + pat.getPatient().getDisplayName());
		}
	}
	
//----------------------------------------------------------------------------------------------------------------------

	private SelectedPatient addPatient(String staffId, String assigningAuthority, String uniqueId, String dateOfBirth,
										String firstName, String lastName, String ssn, String gender,
										Date searchedTime) {
		DOBDate dob = new DOBDate(null, dateOfBirth);
		Patient patient = new Patient();
		patient.setPatientIdentifier(new PatientIdentifier(assigningAuthority, uniqueId));
		patient.setDateOfBirth(dob);
		patient.setFirstName(firstName);
		patient.setLastName(lastName);
		patient.setSsn(ssn);
		patient.setGender(gender);
		
		dataLayer.addSelectedPatient(searchedTime, PatientSearchUtilities.FACILITY_CODE, staffId, patient);
		
		SelectedPatient selectedPatient = new SelectedPatient();
		selectedPatient.setPatient(patient);
		selectedPatient.setSearchedTime(searchedTime);
		return selectedPatient;
	}
	
	private Date getDate(int year, int month, int day) {
		Calendar cal = Calendar.getInstance();
		cal.set(year, month, day, 0, 0, 0);
		Date time = cal.getTime();
		return time;
	}
}