package com.agilex.healthcare.mobilehealthplatform.datalayer.document;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataEditRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.utility.NullChecker;

public class DocumentService {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(DocumentService.class);

	private Router router;

	public DocumentService() {
		router = new Router();
	}

	public DocumentService(Router router) {
		router = new Router();
	}

	public Documents fetchDocumentsForPatient(PatientIdentifier patientIdentifier, ScopeFilter scopeFilter) {
		return fetchDocumentsForPatient(patientIdentifier, null, scopeFilter);
	}

	public Documents fetchDocumentsForPatient(PatientIdentifier patientIdentifier, DocumentFilter filter, ScopeFilter scopeFilter) {
		logger.debug(String.format("begin fetch documents for patient with filter [p=%s][filter=%s]", patientIdentifier, filter));

		logger.debug(String.format("begin fetch documents for patient [p=%s]", patientIdentifier));

		RequestMessage request = FetchDocumentListRequestBuilder.forRetrieveList().forDocumentFilter(filter).forDomain(Domain.document).forPatientIdentifier(patientIdentifier).forScopeFilter(scopeFilter).build();
		ResponseMessage responseMessage = router.execute(request);
		Documents documents = PatientDataResponseReader.<Documents, Document> fromResponse(responseMessage).getDataListNoNull(Documents.class);

		logger.debug(String.format("complete with raw document fetch [rc=%s]", documents.size()));

		// note: this was added in case the data layer doesn't apply the filter.
		// This could be removed and put into the data layer or handler
//		Documents filteredDocuments = DocumentFilterer.filter(documents, filter);
//		logger.debug(String.format("applied document filter [rc=%s]", filteredDocuments.size()));
		return documents;
	}

	public Documents fetchDocumentsForPatientAssociatedWithPatientData(PatientIdentifier patient, String relatedDataDomain, DataIdentifier relatedDataIdentifier, ScopeFilter scopeFilter) {
		logger.debug(String.format("begin fetch documents for patient [p=%s][domain=%s][dataid=%s]", patient, relatedDataDomain, relatedDataIdentifier));

		DocumentFilter filter = new DocumentFilter();
		filter.setParentItem(new RelatedDataItem(relatedDataDomain, relatedDataIdentifier));

		Documents documents = fetchDocumentsForPatient(patient, filter, scopeFilter);

		logger.debug(String.format("complete fetch documents for patient [p=%s][domain=%s][dataid=%s][rc=%s]", patient, relatedDataDomain, relatedDataIdentifier, documents.size()));
		return documents;
	}

	public Document fetchSingleDocument(PatientIdentifier patientIdentifier, DataIdentifier dataIdentifier) {
		logger.debug(String.format("begin fetch document [p=%s][id=%s]", patientIdentifier, dataIdentifier));

		RequestMessage request = PatientDataFetchRequestBuilder.forRetrieveSingleById().forDomain(Domain.document).forPatientIdentifier(patientIdentifier).forDataIdentifier(dataIdentifier).build();
		ResponseMessage responseMessage = router.execute(request);
		Document document = PatientDataResponseReader.<Documents, Document> fromResponse(responseMessage).getDataItem();

		logger.debug(String.format("complete with fetch document [p=%s][id=%s]", patientIdentifier, dataIdentifier, document));
		return document;
	}

	public DocumentBody fetchSingleDocumentBody(PatientIdentifier patient, DataIdentifier dataIdentifier) {
		Document document = fetchSingleDocument(patient, dataIdentifier);
		DocumentBody documentBody = null;
		if (document != null)
			documentBody = document.getBody();
		return documentBody;
	}

	public Document createDocument(Document document, ScopeFilter scopeFilter) {
		RequestMessage request = PatientDataEditRequestBuilder.forCreate().forDomain(Domain.document).forData(document).forScope(scopeFilter).build();
		ResponseMessage responseMessage = router.execute(request);
		Document savedDocument = PatientDataResponseReader.<Documents, Document> fromResponse(responseMessage).getDataItem();
		return savedDocument;
	}

	public Document updateDocument(Document document) {
		RequestMessage request = PatientDataEditRequestBuilder.forUpdate().forDomain(Domain.document).forData(document).build();
		ResponseMessage responseMessage = router.execute(request);
		Document savedDocument = PatientDataResponseReader.<Documents, Document> fromResponse(responseMessage).getDataItem();
		return savedDocument;
	}

	public void deleteDocument(Document document) {
		RequestMessage request = PatientDataEditRequestBuilder.forDelete().forDomain(Domain.document).forData(document).build();
		ResponseMessage responseMessage = router.execute(request);
		PatientDataResponseReader.<Documents, Document> fromResponse(responseMessage).getDataItem();
	}

	public Document createDocumentAssociatedWithPatientData(Document document, String relatedDataDomain, DataIdentifier relatedDataIdentifier, ScopeFilter scopeFilter) {
		logger.debug(String.format("save document associated with patient data [patient=%s][domain=%s][data.id=%s]", document.getPatientIdentifier(), relatedDataDomain, relatedDataIdentifier));
		RelatedDataItem parent = new RelatedDataItem();
		parent.setDomain(relatedDataDomain);
		parent.setIdentifier(relatedDataIdentifier);
		document.setParentData(parent);

		if (NullChecker.isNullish(document.getDocumentTitle())) {
			document.setDocumentTitle("Document associated with " + relatedDataDomain + ";" + relatedDataIdentifier);
		}

		return createDocument(document, scopeFilter);
	}

	public Document saveDocumentContent(PatientIdentifier patientIdentifier, DataIdentifier documentIdentifier, byte[] documentContent, String contentType) {
		logger.debug(String.format("fetch document [%s][%s]", patientIdentifier, documentIdentifier));
		Document document = fetchSingleDocument(patientIdentifier, documentIdentifier);
		logger.debug(String.format("fetched document [%s]", document));
		logger.debug(String.format("assign content to document [%s]", document));
		document.getBody().setContents(documentContent);
		document.getBody().setMimeType(contentType);
		document = updateDocument(document);
		logger.debug(String.format("saved document [%s]", document));
		return document;
	}

    public ReportDocuments fetchAllReportDocumentsForPatient(PatientIdentifier patientIdentifier, DateFilter dateFilter){
        logger.debug("Fetching All types of documents for a Patient");
        RequestMessage requestMessage = PatientDataFetchRequestBuilder.forPatientAllReports()
                .forPatientIdentifier(patientIdentifier)
                .forDateFilter(dateFilter)
                .forDomain(Domain.patientAllReportDocuments)
                .forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
                .build();
        ResponseMessage responseMessage = router.execute(requestMessage);
        return PatientDataResponseReader.<ReportDocuments, ReportDocument>fromResponse(responseMessage).getDataListNoNull(ReportDocuments.class);
    }

}
