package com.agilex.healthcare.mobilehealthplatform.datalayer.order;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.RequestMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.ResponseMessage;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.Router;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.HospitalLocationsResponseReader;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.OrderPickListItemEntriesResponseReader;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataEditRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataFetchRequestBuilder;
import com.agilex.healthcare.mobilehealthplatform.datalayer.dataretriever.router.patientdata.PatientDataResponseReader;
import com.agilex.healthcare.mobilehealthplatform.domain.Admission;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocation;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocations;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItemEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItemEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyOrder;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationError;
import com.agilex.healthcare.mobilehealthplatform.domain.ValidationResult;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.restservice.exception.InvalidOrderException;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.Domain;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.utility.DateHelper;

public class OrderDataService {

    private Router router = new Router();

    public OrderDataService() {
    }

    public OrderPickListItemEntries fetchCategorySupplements(PatientIdentifier patientIdentifier, String displayGroup, String category) {
        RequestMessage requestMessage = PatientDataFetchRequestBuilder
                .forCategorySupplements()
                .forPatientIdentifier(patientIdentifier)
                .forDisplayGroup(displayGroup)
                .forCategory(category)
                .forDomain(Domain.patientOrders)
                .forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
                .build();
        ResponseMessage responseMessage = router.execute(requestMessage);

        return OrderPickListItemEntriesResponseReader.<OrderPickListItemEntries, OrderPickListItemEntry> fromResponse(responseMessage).getDataListNoNull(OrderPickListItemEntries.class);
    }

    public Order saveRadiologyOrder(PatientIdentifier patientIdentifier, RadiologyOrder radiologyOrder) {
        RequestMessage requestMessage = PatientDataEditRequestBuilder
                .forSaveRadiologyOrder()
                .forData(radiologyOrder)
                .forDomain(Domain.patientOrders)
                .forPatientIdentifier(patientIdentifier)
                .forScope(ScopeFilter.getInstanceForOperationalScope())
                .build();
        ResponseMessage responseMessage = router.execute(requestMessage);
        Order createdRadiologyOrder = PatientDataResponseReader.<Orders, Order> fromResponse(responseMessage).getDataItem();

        return createdRadiologyOrder;
    }

	public Orders fetchOrdersByCustom(PatientIdentifier patientIdentifier, String startDate, String endDate, String statusIEN, String serviceIEN) {
		validate(startDate, endDate, statusIEN, serviceIEN);

		String vistaStartDate = startDate == null || startDate.isEmpty() ? "" : DateHelper.convertDateToVistaFormat(DateHelper.parseDate(startDate));
		String vistaEndDate = endDate == null || endDate.isEmpty() ? "" : DateHelper.convertDateToVistaFormat(DateHelper.parseDate(endDate));

		RequestMessage requestMessage = PatientDataFetchRequestBuilder
				.forOrdersByCustom().forPatientIdentifier(patientIdentifier)
				.forDomain(Domain.patientOrders)
				.forStartDate(vistaStartDate)
				.forEndDate(vistaEndDate)
				.forStatus(statusIEN)
				.forService(serviceIEN)
				.forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
				.build();
		ResponseMessage responseMessage = router.execute(requestMessage);

		Orders orders = PatientDataResponseReader.<Orders, Order> fromResponse(responseMessage).getDataListNoNull(Orders.class);

		return orders;
	}

	private void validate(String startDate, String endDate, String statusIEN, String serviceIEN) {
		ValidationResult<Order> validationResult = new ValidationResult<Order>();

		validateDateFormat(validationResult, "startDate", startDate);
		validateDateFormat(validationResult, "endDate", endDate);
		validateNotNullIEN(validationResult, "status-ien", statusIEN);
		validateNotNullIEN(validationResult, "service-ien", serviceIEN);

		validateEndDateRequiresStartDate(validationResult, "startDate", startDate, endDate);

		if (validationResult.getErrors().size() > 0) {
			throw new InvalidOrderException(validationResult.getErrors());
		}
	}

	private void validateDateFormat(ValidationResult<Order> validationResult, String fieldName, String value) {
		if (value != null && !value.isEmpty()) {
			try {
				DateHelper.parseDate(value);
			} catch (RuntimeException e) {
				ValidationError error = new ValidationError();
				error.setErrorMessage(String.format("%s must be in the format MM/dd/yyyy", value));
				error.setFieldName(fieldName);

				validationResult.addIfNotNull(error);
			}
		}
	}

	private void validateEndDateRequiresStartDate(ValidationResult<Order> validationResult, String fieldName, String startDate, String endDate) {
		if (endDate != null && !endDate.isEmpty() && (startDate == null  || startDate.isEmpty())) {
			ValidationError error = new ValidationError();
			error.setErrorMessage("startDate is required if endDate is provided");
			error.setFieldName(fieldName);

			validationResult.addIfNotNull(error);
		}
	}

	private void validateNotNullIEN(ValidationResult<Order> validationResult, String fieldName, String value) {
		if (value == null || value.isEmpty()) {
			ValidationError error = new ValidationError();
			error.setErrorMessage("Must not be null or empty.");
			error.setFieldName(fieldName);

			validationResult.addIfNotNull(error);
		}

	}

    public JSONArray fetchOrdersJSONByCustom(PatientIdentifier patientIdentifier, String startDate, String endDate, String statusIEN, List<String> serviceList) {
		Orders orders = new Orders();

		for (String serviceIEN : serviceList) {
			orders.addAll(fetchOrdersByCustom(patientIdentifier, startDate, endDate, statusIEN, serviceIEN));
		}

		JSONArray topLevel = new JSONArray();
		JSONObject topLevelGroup = new JSONObject();
		JSONArray groupedOrders = new JSONArray();
		HashMap<String, Orders> typedOrders = new HashMap<String, Orders>();
		int pending = 0, active = 0, total = 0;

		for(Order order : orders) {
			if(typedOrders.containsKey(order.getOrderType())) {
				typedOrders.get(order.getOrderType()).add(order);
			} else {
				Orders newOrders = new Orders();
				newOrders.add(order);
				typedOrders.put(order.getOrderType(), newOrders);
			}
			if("Active".equals(order.getStatus())) {
				active++;
			} else if("Pending".equals(order.getStatus())) {
				pending++;
			}
			total++;
		}

		try {
			for(Map.Entry<String, Orders> typedOrder : typedOrders.entrySet()) {
				JSONObject typeLevelGroup = new JSONObject();
				JSONArray jsonGroupedOrders = new JSONArray();
				typeLevelGroup.put("type", typedOrder.getKey());
				typeLevelGroup.put("size", typedOrder.getValue().size());
				for(Order order : typedOrder.getValue()) {
					jsonGroupedOrders.put(getJSONFromOrder(order));
				}

				typeLevelGroup.put("orders", jsonGroupedOrders);
				groupedOrders.put(typeLevelGroup);

			}

			topLevelGroup.put("id", orders.get(0).getOrderGrouping().name());
			topLevelGroup.put("name", orders.get(0).getOrderGrouping().name());
			topLevelGroup.put("pending", pending);
			topLevelGroup.put("active", active);
			topLevelGroup.put("total", total);
			topLevelGroup.put("groupedOrders", groupedOrders);
			topLevel.put(topLevelGroup);
		} catch (JSONException e) {
			throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
		}

		return topLevel;
    }

    public Admissions getAdmissions(PatientIdentifier patientIdentifier) {
        RequestMessage request = PatientDataFetchRequestBuilder
                .forRetrieveList()
                .forDomain(Domain.admission)
                .forPatientIdentifier(patientIdentifier)
                .forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
                .forDateFilter(DateFilterFactory.createEmptyFilter())
                .includeDetails()
                .build();
        ResponseMessage responseMessage = router.execute(request);
        return PatientDataResponseReader.<Admissions, Admission> fromResponse(responseMessage).getDataListNoNull(Admissions.class);

    }

    public HospitalLocations getClinics(String siteCode) {
        RequestMessage request = PatientDataFetchRequestBuilder
                .forClinics()
                .forDomain(Domain.patientOrdersEmr)
                .forSite(siteCode)
                .forTarget("")
                .forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
                .build();
        ResponseMessage responseMessage = router.execute(request);

        return HospitalLocationsResponseReader.<HospitalLocations, HospitalLocation>fromResponse(responseMessage).getDataListNoNull(HospitalLocations.class);
    }

    public HospitalLocations getLocations(String siteCode) {
        RequestMessage request = PatientDataFetchRequestBuilder
                .forLocations()
                .forDomain(Domain.patientOrdersEmr)
                .forSite(siteCode)
                .forTarget("")
                .forDirection("")
                .forScopeFilter(ScopeFilter.getInstanceForOperationalScope())
                .build();
        ResponseMessage responseMessage = router.execute(request);

        return HospitalLocationsResponseReader.<HospitalLocations, HospitalLocation>fromResponse(responseMessage).getDataListNoNull(HospitalLocations.class);
    }

    /**
     * This method returns a JSONObject object that contains the order in JSON format
     *
     * @param order The order to be coverted to JSON
     *
     * @return A populated JSONObject object.
     */
    private static JSONObject getJSONFromOrder(Order order) {


        JSONObject out = new JSONObject();
        JSONObject provider = new JSONObject();
        JSONObject patient = new JSONObject();
        JSONObject userIdentifier = new JSONObject();

        try {

            userIdentifier.put("uniqueId", nullSafeString(order.getProvider().getUserIdentifier().getUniqueId()));
            userIdentifier.put("assigningAuthority", nullSafeString(order.getProvider().getUserIdentifier().getAssigningAuthority()));

            patient.put("object-type", nullSafeString(order.getProvider().getPatient().getObjectType()));
            PatientIdentifier patientIdentifier = order.getProvider().getPatient().getPatientIdentifier();
            patient.put("patientIdentifier", patientIdentifier == null ? "null" : patientIdentifier);
            patient.put("inpatient", order.getProvider().getPatient().isInpatient());
            patient.put("textMessagingAllowed", order.getProvider().getPatient().isTextMessagingAllowed());
            patient.put("serviceConnected", order.getProvider().getPatient().isServiceConnected());
            patient.put("sensitive", order.getProvider().getPatient().isSensitive());
            patient.put("displayName", nullSafeString(order.getProvider().getPatient().getDisplayName()));

            provider.put("object-type", nullSafeString(order.getProvider().getObjectType()));
            provider.put("lastName", nullSafeString(order.getProvider().getLastName()));
            provider.put("firstName", nullSafeString(order.getProvider().getFirstName()));
            provider.put("displayName", nullSafeString(order.getProvider().getDisplayName()));
            provider.put("userIdentifier", userIdentifier);
            provider.put("patient", patient);
            provider.put("id", nullSafeString(order.getProvider().getId()));

            out.put("orderId", nullSafeString(order.getOrderId()));
            out.put("orderedDate", order.getOrderedDate());
            out.put("startDate", order.getStartDate());
            out.put("stopDate", order.getStopDate());
            out.put("status", nullSafeString(order.getStatus()));
            out.put("dateSigned", order.getDateSigned());
            out.put("verifyingNurse", nullSafeString(order.getVerifyingNurse()));
            out.put("dateVerified", order.getDateVerified());
            out.put("verifyingClerk", nullSafeString(order.getVerifyingClerk()));
            out.put("orderText", nullSafeString(order.getOrderText()));
            out.put("orderDetail", nullSafeString(order.getOrderDetail()));
            out.put("orderTypeId", nullSafeString(order.getOrderTypeId()));
            out.put("orderGrouping", nullSafeString(order.getOrderGrouping().name()));
            out.put("orderType", nullSafeString(order.getOrderType()));
            out.put("orderSubType", nullSafeString(order.getOrderSubType()));
            String orderSignStatus = null != order.getOrderSignStatus() ?nullSafeString(order.getOrderSignStatus().getName()) : "N/A";
            out.put("orderSignStatus", orderSignStatus);
            out.put("flag", order.isFlag());
            out.put("object-type", nullSafeString(order.getObjectType()));
            out.put("provider", provider);

        } catch (JSONException e) {
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
        return out;
    }

    private static String nullSafeString(String string) {
        return null == string ? "null" : string;
    }
}