package com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.UriDtoValidator;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.UriHelper;
import com.agilex.healthcare.utility.ArrayAppender;

import javax.ws.rs.core.UriBuilder;
import java.net.URI;

public class PatientLinkBuilder {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientLinkBuilder.class);

	private static String LONGITUDINAL = "longitudinal";
	private static String OPERATIONAL = "operational";

	private URI baseUri;

	public PatientLinkBuilder(URI baseUri) {
		this.baseUri = baseUri;
	}

	public static URI createPatientSelfUri(PatientIdentifier patientIdentifier, URI baseUri) {
		PatientLinkBuilder linkBuilder = new PatientLinkBuilder(baseUri);
		return linkBuilder.createPatientSelfUri(patientIdentifier);
	}

	public URI createPatientSelfUri(PatientIdentifier patientIdentifier) {
		return UriBuilder.fromUri(this.baseUri).path("patient").path(patientIdentifier.getAssigningAuthority()).path(patientIdentifier.getUniqueId()).build();
	}

	public static URI createPatientSelfUri(Patient patient, URI baseUri) {
		URI uri = null;
		if (patient == null) {
			uri = null;
		} else {
			uri = createPatientSelfUri(patient.getPatientIdentifier(), baseUri);
		}
		return uri;
	}

    public void fillLinks(Patients patients, URI requestUri){
        fillLinks(patients, requestUri, null);
    }

	public void fillLinks(Patients patients, URI requestUri, String siteCode) {
		if (patients != null) {
			UriDtoValidator.validate(patients);

			patients.setSelfUri(requestUri);
			fillCustomRelatedLinks(patients);

			for (Patient dataItem : patients) {
				this.fillLinks(dataItem, null, siteCode);
			}
		}
	}

	private void fillCustomRelatedLinks(Patients patients) {
		// why does this blank method exist?
		// answer: it is here to remain consistent with the pattern in the
		// other link builder implementations
	}

    public void fillLinks(Patient dataItem, URI requestUri) {
        fillLinks(dataItem, requestUri, null);
    }

	public void fillLinks(Patient dataItem, URI requestUri, String siteCode) {
		if (dataItem != null) {
			UriDtoValidator.validate(dataItem);
			UriHelper.setSelfUri(dataItem, requestUri, this.getCannonicalSelfUri(dataItem));
			fillCustomRelatedLinks(dataItem, siteCode);
		}
	}

	private URI getCannonicalSelfUri(Patient dataItem) {
		return createPatientSelfUri(dataItem.getPatientIdentifier(), this.getBaseUri());
	}

	private void fillCustomRelatedLinks(Patient patient, String siteCode) {
		logger.debug("Getting Patient Links from PatientLinkBuilder");

		appendRelatedLink(patient, LinkTitles.PatientIdentifiers, "identifiers");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientDemographics, "demographics");
		appendRelatedLink(patient, LinkTitles.PatientPreference, "preference");
		appendRelatedLink(patient, LinkTitles.PatientImage, "patient-image");

		fillCalendarLinks(patient);

		fillProblemLinks(patient);

		fillAllergyLinks(patient);

		fillRadiologyLinks(patient);

		fillVitalsLinks(patient);

		fillAdmissionLinks(patient);

		fillSurgeryLinks(patient);

		fillConsultLinks(patient);

		fillPatientSecureMessageRelatedLinks(patient);

        fillPatientWaitListRelatedLinks(patient, siteCode);

        fillPatientBookedAppointmentsLinks(patient, siteCode);

		fillPatientMedicationLinks(patient);

        fillPatientReportDocumentLink(patient, siteCode);

		appendRelatedLink(patient, LinkTitles.PatientMedicationRefills, "medication-refills");

		fillPatientLabLinks(patient, siteCode);

		fillPatientNotificationRelatedLinks(patient);

		fillToolTrackingResultLinks(patient);

		fillPatientConsultLinks(patient);

		fillPatientImmunizationLinks(patient, siteCode);

        fillPatientDocumentRelatedLinks(patient, siteCode);

        fillPatientOrdersLinks(patient, siteCode);
        
        fillPatientOrderResultsHistoryLinks(patient, siteCode);
	}

	private void fillPatientSecureMessageRelatedLinks(Patient patient) {
		URI secureMessageUri = UriBuilder.fromUri(this.getBaseUri()).path("securemessage-service").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("secure-message-user").build();
		patient.getLink().addRelatedLink(LinkTitles.SecureMessageUser, secureMessageUri);
	}

    private void fillPatientBookedAppointmentsLinks(Patient patient, String siteCode) {
        //if sitecode not available, try to get it
        if (siteCode == null)
            siteCode = this.retrieveSiteCode();

        if (siteCode != null) {
            URI bookedAppointmentsUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("booked-appointments").build();
            patient.getLink().addRelatedLink(LinkTitles.PatientBookedAppointments, bookedAppointmentsUri);
            URI bookNewAppointmentUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("book-appointment").build();
            patient.getLink().addRelatedLink(LinkTitles.BookNewAppointment, bookNewAppointmentUri);
            URI removeWaitListUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("remove-waitlist-entry").build();
            patient.getLink().addRelatedLink(LinkTitles.removeWaitList, removeWaitListUri);
            URI updateWaitListUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("update-near-list").build();
            patient.getLink().addRelatedLink(LinkTitles.UpdateNearList, updateWaitListUri);
            URI updateEWLUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("update-ewl-entry").build();
            patient.getLink().addRelatedLink(LinkTitles.updateEwlEntry, updateEWLUri);
            URI patientConsultUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("consults").build();
            patient.getLink().addRelatedLink(LinkTitles.PatientConsultsFromVista, patientConsultUri);
        }
    }

    private void fillPatientWaitListRelatedLinks(Patient patient, String siteCode) {
        //if sitecode not available, try to get it
        if (siteCode == null)
            siteCode = this.retrieveSiteCode();

        if (siteCode != null) {
            URI patientWaitListUri = SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(this.getBaseUri()).path("scheduling-service").path("site").path(siteCode).path("waitlist").path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).build();
            patient.getLink().addRelatedLink(LinkTitles.PatientWaitList, patientWaitListUri);
        }
    }

    private String retrieveSiteCode() {
        MhpUser mhpUser = MhpUserFactory.createFromSecurityContext();
        String siteCode = null;
        if(mhpUser!=null)
            siteCode = mhpUser.getVistaLocation();
        return siteCode;
    }

	private void fillPatientNotificationRelatedLinks(Patient patient) {
		URI notificationsUri = UriBuilder.fromUri(this.getBaseUri()).path("notification-service").path("user").path("id").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("notifications").build();
		patient.getLink().addRelatedLink(LinkTitles.PatientNotifications, notificationsUri);

		URI deviceRegistrationUri = UriBuilder.fromUri(this.getBaseUri()).path("notification-service").path("user").path("id").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("device-registrations").build();
		patient.getLink().addRelatedLink(LinkTitles.PatientDevices, deviceRegistrationUri);
	}

    private void fillPatientReportDocumentLink(Patient patient, String siteCode){
        if (siteCode == null)
            siteCode = this.retrieveSiteCode();
        if (siteCode != null) {
            URI reportDocumentUri = UriBuilder.fromUri(this.getBaseUri()).path("jpv").path("site").path(siteCode).path("documents").path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).build();
            patient.getLink().addRelatedLink(LinkTitles.patientReportDocument, reportDocumentUri);
        }
    }

	private void fillPatientImmunizationLinks(Patient patient, String siteCode) {
		URI immunizationCampaignHistoryUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("immunization-history").build();
		URI immunizationCampaignDocumentUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("document-immunization").build();
		URI immunizationCampaignEmailUri =  ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("vaccines").path("VIS").path("mail").build();
		URI immunizationCampaignDeclineUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("decline-immunization").build();
		URI immunizationCampaignDocumentReceivedElsewhereUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority())
																	.path(patient.getPatientIdentifier().getUniqueId()).path("document-immunization-received-elsewhere").build();

		  if(siteCode == null)
	            siteCode = this.retrieveSiteCode();

		if (siteCode != null) {
			URI immunizationClinicsUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("site").path(siteCode).path("immunization-clinics").build();
			URI immunizationNoteTitlesUri = ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(this.getBaseUri()).path("site").path(siteCode).path("immunization-note-titles").build();
			patient.getLink().addRelatedLink(LinkTitles.ImmunizationClinics, immunizationClinicsUri);
			patient.getLink().addRelatedLink(LinkTitles.ImmunizationNoteTitles, immunizationNoteTitlesUri);
		}

		patient.getLink().addRelatedLink(LinkTitles.ImmunizationHistory, immunizationCampaignHistoryUri);
		patient.getLink().addRelatedLink(LinkTitles.DocumentImmunization, immunizationCampaignDocumentUri);
		patient.getLink().addRelatedLink(LinkTitles.emailVIS, immunizationCampaignEmailUri);
		patient.getLink().addRelatedLink(LinkTitles.DeclineImmunization, immunizationCampaignDeclineUri);
		patient.getLink().addRelatedLink(LinkTitles.DocumentImmunizationReceivedElseWhere, immunizationCampaignDocumentReceivedElsewhereUri);
	}

    public void fillPatientDocumentRelatedLinks(Patient patient, String siteCode){
        if(siteCode == null)
            siteCode = this.retrieveSiteCode();

        if(siteCode != null){
            URI patientSurgeryReportUri = UriBuilder.fromUri(this.getBaseUri()).path("surgery-reports").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).build();
            patient.getLink().addRelatedLink(LinkTitles.patientSurgeryReports,patientSurgeryReportUri);

            URI patientDischargeSummariesUri = UriBuilder.fromUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("admissions").path("site").path(siteCode).path("discharge-summaries").build();
            patient.getLink().addRelatedLink(LinkTitles.patientDischargeSummaries, patientDischargeSummariesUri);

            URI patientProgressNotesUri = UriBuilder.fromUri(this.getBaseUri()).path("progress-note").path("site").path(siteCode).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("notes").build();
            patient.getLink().addRelatedLink(LinkTitles.patientProgressNotes, patientProgressNotesUri);
        }

        URI patientSurgicalPathologyReportsUri = UriBuilder.fromUri(this.getBaseUri()).path("patient").path(patient.getPatientIdentifier().getAssigningAuthority()).path(patient.getPatientIdentifier().getUniqueId()).path("lab-pathology").path("pathology-reports").build();
        patient.getLink().addRelatedLink(LinkTitles.patientPathologyReports, patientSurgicalPathologyReportsUri);
    }

    private void fillPatientLabLinks(Patient patient, String siteCode) {
		if(siteCode == null)
			siteCode = this.retrieveSiteCode();

		if(siteCode != null) {
			appendRelatedLink(patient, LinkTitles.PatientChemHemLabTests, "lab-tests", "site", siteCode);
		}

		appendRelatedLinksWithScopes(patient, LinkTitles.PatientMicrobiologyLabResults, "lab-microbiology", "results");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientMicrobiologyLabTestHistory, "lab-microbiology", "tests");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientMicrobiologyLabTestHistoryByGroup, "lab-microbiology", "groups");
	}

	private void fillPatientMedicationLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientMedications, "medications");
		appendRelatedLink(patient, LinkTitles.PatientActiveMedications, "medications", "active");
		appendRelatedLink(patient, LinkTitles.PatientOutpatientMedications, "medications", "outpatient");
		appendRelatedLink(patient, LinkTitles.PatientInpatientMedications, "medications", "inpatient");
		appendRelatedLink(patient, LinkTitles.PatientActiveOutpatientMedications, "medications", "outpatient", "active");
		appendRelatedLink(patient, LinkTitles.PatientActiveInpatientMedications, "medications", "inpatient", "active");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveLongitudinalScope, "medications", "scope", LONGITUDINAL, "active");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveOperationalScope, "medications", "scope", OPERATIONAL, "active");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientInpatientMedications, "medications", "inpatient");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientOutpatientMedications, "medications", "outpatient");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveInpatientLongitudinalScope, "medications", "inpatient", "scope", LONGITUDINAL, "active");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveInpatientOperationalScope, "medications", "inpatient", "scope", OPERATIONAL, "active");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveOutpatientLongitudinalScope, "medications", "inpatient", "scope", LONGITUDINAL, "active");
		appendRelatedLink(patient, LinkTitles.PatientMedicationsActiveOutpatientOperationalScope, "medications", "inpatient", "scope", OPERATIONAL, "active");
	}

	private void fillPatientConsultLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientConsults, "consults");
	}

	private void fillVitalsLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientVitals, "vitals");
		appendRelatedLink(patient, LinkTitles.PatientVitalsGraphLongitudinal, "vitals", "scope", LONGITUDINAL, "graphdata");
		appendRelatedLink(patient, LinkTitles.PatientVitalsGraphOperational, "vitals", "scope", OPERATIONAL, "graphdata");
		appendRelatedLink(patient, LinkTitles.PatientVitalsTableLongitudinal, "vitals", "scope", LONGITUDINAL, "tabledata");
		appendRelatedLink(patient, LinkTitles.PatientVitalsTableOperational, "vitals", "scope", OPERATIONAL, "tabledata");
	}

	private void fillAllergyLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientAllergies, "allergies");
	}

	private void fillAdmissionLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientAdmissions, "admissions");
	}

	private void fillSurgeryLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientSurgeries, "surgeries");
	}

	private void fillConsultLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientConsults, "consults");
	}

	private void fillRadiologyLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientRadiologyTests, "radiology", "tests");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientRadiologyResults, "radiology", "results");
	}

	private void fillCalendarLinks(Patient patient) {
		appendRelatedLink(patient, LinkTitles.PatientCalendarEvents, "calendar", "calendarEvent");
		appendRelatedLink(patient, LinkTitles.PatientCalendarPublicKey, "calendar", "key");
		appendRelatedLink(patient, LinkTitles.PatientPastAppointments, "appointments", "past");
		appendRelatedLink(patient, LinkTitles.PatientFutureAppointments, "appointments", "future");
		appendRelatedLink(patient, LinkTitles.PatientAppointments, "appointments");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientAppointments, "appointments");
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientMentalHealthAppointments, "appointments", "mental-health");
	}

	private void fillProblemLinks(Patient patient) {
		appendRelatedLinksWithScopes(patient, LinkTitles.PatientProblems, "problems");
		appendRelatedLink(patient, LinkTitles.PatientOutpatientProblems, "problems");
		appendRelatedLink(patient, LinkTitles.PatientInpatientProblems, "problems");
	}

	private void fillToolTrackingResultLinks(Patient patient) {
		appendRelatedLink(patient, LinkTitles.ToolTrackingResults, "toolTracking");
	}

	private void fillPatientOrdersLinks(Patient patient, String siteCode) {
		if(siteCode == null)
            siteCode = this.retrieveSiteCode();

		if(siteCode != null) {
			appendRelatedLink(patient, LinkTitles.PatientOrders, "orders", "site", siteCode);
            appendRelatedLink(patient, LinkTitles.PatientOrdersGrouped, "orders", "site", siteCode, "grouped");
            appendRelatedLink(patient, LinkTitles.PatientOrdersByCustom, "orders", "site", siteCode, "custom");
            appendRelatedLink(patient, LinkTitles.PatientOrderImagingTypes, "orders", "site", siteCode, "imaging-types");
            appendRelatedLink(patient, LinkTitles.PatientOrderImagingLists, "orders", "site", siteCode, "imaging-lists");
            appendRelatedLink(patient, LinkTitles.PatientOrderCategorySupplements, "orders", "site", siteCode, "category-supplements");
            appendRelatedLink(patient, LinkTitles.PatientOrderProcedureMessage, "orders", "site", siteCode, "procedure-message");
            appendRelatedLink(patient, LinkTitles.PatientOrderInpatientLocations, "orders", "site", siteCode, "inpatient-locations");
            appendRelatedLink(patient, LinkTitles.PatientOrderSaveRadiologyOrder, "orders", "site", siteCode, "save-patient");
            appendRelatedLink(patient, LinkTitles.PatientOrderAcceptOrder, "orders", "site", siteCode, "accept-order");
            appendRelatedLink(patient, LinkTitles.PatientOrderAdmissions, "orders", "site", siteCode, "admissions");
            appendRelatedLink(patient, LinkTitles.PatientOrderClinics, "orders", "site", siteCode, "clinics");
            appendRelatedLink(patient, LinkTitles.PatientOrderLocations, "orders", "site", siteCode, "locations");
            appendRelatedLink(patient, LinkTitles.PatientOrderCreatinineTest, "orders", "site", siteCode, "creatininetest");
        }
	}
	
	private void fillPatientOrderResultsHistoryLinks(Patient patient, String siteCode) {
		if(siteCode == null)
            siteCode = this.retrieveSiteCode();

		if(siteCode != null) {
			appendRelatedLink(patient, LinkTitles.PatientOrderResultsHistory, "order-results-history", "site", siteCode);
        }
	}

	private AtomLink appendRelatedLink(Patient patient, String uriTitle, URI uri) {
		return patient.getLink().addRelatedLink(uriTitle, uri);
	}

	private void appendRelatedLinksWithScopes(Patient patient, String uriTitle, String... uriPath) {
		appendRelatedLink(patient, uriTitle, ArrayAppender.append(uriPath, "scope", LONGITUDINAL));
		appendRelatedLink(patient, uriTitle + "-longitudinal", ArrayAppender.append(uriPath, "scope", LONGITUDINAL));
		appendRelatedLink(patient, uriTitle + "-operational", ArrayAppender.append(uriPath, "scope", OPERATIONAL));
	}

	private AtomLink appendRelatedLink(Patient patient, String uriTitle, String... uriPath) {
		UriBuilder uriBuilder = UriBuilder.fromUri(patient.getSelfUri());
		for (String singleUriPath : uriPath) {
			uriBuilder.path(singleUriPath);
		}
		URI uri = uriBuilder.build();
		return appendRelatedLink(patient, uriTitle, uri);
	}

	private URI getBaseUri() {
		return this.baseUri;
	}

}