package com.agilex.healthcare.notewriter.datalayer;

import agilex.commons.NullChecker;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.Count;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.agilex.healthcare.notewriter.translator.ClinicalEventTranslator;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class ClinicalEventDataLayerMongo implements ClinicalEventDataLayer {

	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(ClinicalEventDataLayerMongo.class);

	private static DBCollection clinicalEventsCollection;
	private static DBCollection noteTitlesCollection;
	private static DBCollection lastSelectedNoteTitlesCollection;
	private static DB db;

	private static String queryKey = "$query";
	private static String orderByKey = "$orderby";

	public ClinicalEventDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("note-writer");
			clinicalEventsCollection = db.getCollection("clinical-events");
			noteTitlesCollection = db.getCollection("note-titles");
			lastSelectedNoteTitlesCollection = db.getCollection("last-selected-note-titles");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public ClinicalEventDataLayerMongo(String databaseName, String c1, String c2, String c3) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        clinicalEventsCollection = db.getCollection(c1);
        noteTitlesCollection = db.getCollection(c2);
        lastSelectedNoteTitlesCollection = db.getCollection(c3);

        clinicalEventsCollection.dropIndexes();
        noteTitlesCollection.dropIndexes();
        lastSelectedNoteTitlesCollection.dropIndexes();
    }

//-----------------------------------Fetch Methods----------------------------------------------------------------------

	@Override
	public ClinicalEvent fetchLatestClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		ClinicalEvent clinicalEvent = null;

		BasicDBObject query = createQueryAndOrderByDBObject(facilityCode, staffId, assigningAuthority, patientId);

		DBCursor iterator = clinicalEventsCollection.find(query);

		if (iterator.hasNext()) {
			DBObject dbObject = iterator.next();
			clinicalEvent = translator.translateClinicalEvent(dbObject);
		}

		return clinicalEvent;
	}

	@Override
	public ClinicalEvent fetchClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		ClinicalEvent clinicalEvent = null;

		BasicDBObject query = createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId, clinicalEventId);

		DBObject dbObject = clinicalEventsCollection.findOne(query);

		clinicalEvent =  translator.translateClinicalEvent(dbObject);

		return clinicalEvent;
	}

    public DBObject fetchClinicalEventAsDBObject(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId, ClinicalEventState clinicalEventState) {
        BasicDBObject query = createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId, clinicalEventId, clinicalEventState);

        DBObject dbObject = clinicalEventsCollection.findOne(query);

        return dbObject;
    }

//--------------------------------------Update and Save Methods---------------------------------------------------------

	@Override
	public ClinicalEvent updateClinicalEvent(ClinicalEvent clinicalEvent) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();

		BasicDBObject query = createQueryDBObject(clinicalEvent);
		DBObject dbObject = translator.translateClinicalEvent(clinicalEvent);
		clinicalEventsCollection.update(query, dbObject);

		return translator.translateClinicalEvent(dbObject);
	}

	@Override
	public ClinicalEvent saveClinicalEvent(ClinicalEvent clinicalEvent) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();

		DBObject dbObject = translator.translateClinicalEvent(clinicalEvent);
		clinicalEventsCollection.insert(dbObject);

		return translator.translateClinicalEvent(dbObject);
	}

//---------------------------------Count Methods------------------------------------------------------------------------

	@Override
	public Count getClinicalEventCount(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		Count clinicalEventCount = new Count();

		BasicDBObject query = createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId);

		clinicalEventCount.setValue(clinicalEventsCollection.count(query));
		return clinicalEventCount;
	}

//---------------------------------Private Methods for Locating an Object in Mongo--------------------------------------

	private BasicDBObject createQueryDBObject(ClinicalEvent clinicalEvent) {
		return createQueryDBObject(clinicalEvent.getProviderIdentifier().getFacilityCode(),
									clinicalEvent.getProviderIdentifier().getStaffId(),
									clinicalEvent.getPatientIdentifier().getAssigningAuthority(),
									clinicalEvent.getPatientIdentifier().getUniqueId(),
									clinicalEvent.getId());
	}
	private BasicDBObject createQueryDBObject(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		return createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId, null);
	}

	private BasicDBObject createQueryDBObject(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId) {
		return createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId, clinicalEventId, ClinicalEventState.DRAFT);
	}

    private BasicDBObject createQueryDBObject(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId, ClinicalEventState state) {
        BasicDBObject query = new BasicDBObject();
        query.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER + "." + ClinicalEventTranslator.PROVIDER_FACILITY_CODE, facilityCode);
        query.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER + "." + ClinicalEventTranslator.PROVIDER_STAFF_ID, staffId);

        query.put(ClinicalEventTranslator.PATIENT_IDENTIFIER + "." + ClinicalEventTranslator.PATIENT_ASSIGNING_AUTHORITY, assigningAuthority);
        query.put(ClinicalEventTranslator.PATIENT_IDENTIFIER + "." + ClinicalEventTranslator.PATIENT_UNIQUE_ID, patientId);

        query.put(ClinicalEventTranslator.CLINICAL_EVENT_STATE, state.toString());

        if (clinicalEventId != null)
            query.put(ClinicalEventTranslator.ID, clinicalEventId);

        return query;
    }

	private BasicDBObject createOrderByDBObject(int sort) {
		BasicDBObject orderBy = new BasicDBObject();
		orderBy.put(ClinicalEventTranslator.LAST_MODIFIED_DATE, sort);
		return orderBy;
	}

	private BasicDBObject createQueryAndOrderByDBObject(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		BasicDBObject query = createQueryDBObject(facilityCode, staffId, assigningAuthority, patientId);
		BasicDBObject orderBy = createOrderByDBObject(-1);
		BasicDBObject queryAndOrderBy = new BasicDBObject();
		queryAndOrderBy.put(queryKey, query);
		queryAndOrderBy.put(orderByKey, orderBy);

		return queryAndOrderBy;
	}

//--------------------------------------Methods to work with NoteTitles-------------------------------------------------
	@Override
	public NoteTitles getMongoNoteTitles(String facilityCode, SearchFilter filter) {

		ClinicalEventTranslator translator = new ClinicalEventTranslator();
		NoteTitles titles = new NoteTitles();

		BasicDBObject query = new BasicDBObject();
		query.put(ClinicalEventTranslator.NOTE_TITLE_FACILITY, facilityCode);
		if (NullChecker.isNotNullish(filter.getSearchString()))
			query.put(ClinicalEventTranslator.NOTE_TITLE_NAME, java.util.regex.Pattern.compile("(?i:" + filter.getSearchString() + ")"));

		DBCursor iterator = noteTitlesCollection.find(query).sort(new BasicDBObject(ClinicalEventTranslator.NOTE_TITLE_NAME, 1)).limit(filter.getMaxRecords());

		while(iterator.hasNext()) {
			DBObject dbTitle = iterator.next();
			NoteTitle title = translator.translateTitle(dbTitle);
			titles.add(title);
		}

		return titles;
	}

	@Override
	public NoteTitle fetchLastSelectedNoteTitle(String facilityCode, String staffId) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();

		BasicDBObject nestedProvider = new BasicDBObject();
		nestedProvider.put(ClinicalEventTranslator.PROVIDER_FACILITY_CODE, facilityCode);
		nestedProvider.put(ClinicalEventTranslator.PROVIDER_STAFF_ID, staffId);

		BasicDBObject query = new BasicDBObject();
		query.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER, nestedProvider);

		DBObject noteTitle = lastSelectedNoteTitlesCollection.findOne(query);

		return translator.translateTitle(noteTitle);
	}

	@Override
	public NoteTitle updateLastSelectedNoteTitle(String facilityCode, String staffId, NoteTitle noteTitle) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();

		BasicDBObject nestedProvider = new BasicDBObject();
		nestedProvider.put(ClinicalEventTranslator.PROVIDER_FACILITY_CODE, facilityCode);
		nestedProvider.put(ClinicalEventTranslator.PROVIDER_STAFF_ID, staffId);

		BasicDBObject query = new BasicDBObject();
		query.put(ClinicalEventTranslator.PROVIDER_IDENTIFIER, nestedProvider);

		DBObject dbObject = translator.translateLastSelectedTitle(facilityCode, staffId, noteTitle);
		lastSelectedNoteTitlesCollection.update(query, dbObject);

		return translator.translateTitle(dbObject);
	}

	@Override
	public NoteTitle insertLastSelectedNoteTitle(String facilityCode, String staffId, NoteTitle noteTitle) {
		ClinicalEventTranslator translator = new ClinicalEventTranslator();

		DBObject dbObject = translator.translateLastSelectedTitle(facilityCode, staffId, noteTitle);
		lastSelectedNoteTitlesCollection.insert(dbObject);

		return translator.translateTitle(dbObject);
	}

}
