package com.agilex.healthcare.notewriter.datalayer;

import java.util.Date;

import javax.annotation.Resource;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response.Status;

import com.agilex.healthcare.mdws.datalayer.TIUNotesDataLayerMdws;
import org.codehaus.jettison.json.JSONException;

import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalNoteSignature;
import com.agilex.healthcare.mobilehealthplatform.domain.Count;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.MhpObjectFactory;
import com.agilex.healthcare.notewriter.translator.ClinicalNoteTranslator;
import com.agilex.healthcare.notewriter.translator.TaskListTranslator;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.vamf.tasks.datalayer.TaskDataLayer;
import com.agilex.healthcare.vamf.tasks.domain.Task;

public class ClinicalEventDataService {
	private ClinicalEventDataLayer dataLayer = null;
	private NoteTitleDataLayer noteTitleDataLayer = null;
	
	@Resource
	private TaskDataLayer taskDataLayer;
	public ClinicalEventDataService() {
		dataLayer = MhpObjectFactory.getInstance().getClinicalEventDataLayer();
		noteTitleDataLayer = MhpObjectFactory.getInstance().getNoteTitleDataLayer();
	}

	public ClinicalEventDataService(ClinicalEventDataLayer dataLayer, NoteTitleDataLayer noteTitleDataLayer) {
		this.dataLayer = dataLayer;
		this.noteTitleDataLayer = noteTitleDataLayer;
	}

	public ClinicalEvent fetchLatestClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		ClinicalEvent clinicalEvent = dataLayer.fetchLatestClinicalEvent(facilityCode, staffId, assigningAuthority, patientId);

		if (clinicalEvent == null) {
			throw new WebApplicationException(Status.NO_CONTENT);
		}

		return clinicalEvent;
	}

	public ClinicalEvent fetchClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, String noteId) {
		return wrapperToHandleNotFound(dataLayer.fetchClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, noteId));
	}

	private void setIdentifier(ClinicalEvent clinicalEvent, String facilityCode, String staffId, String assigningAuthority, String patientId,
                              String  firstName, String  lastName, String  ssn) {
		ProviderIdentifier providerIdentifier = new ProviderIdentifier(facilityCode, staffId);
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId, firstName, lastName, ssn);
		clinicalEvent.setProviderIdentifier(providerIdentifier);
		clinicalEvent.setPatientIdentifier(patientIdentifier);
		clinicalEvent.setFacilityCode(facilityCode);
	}

	public ClinicalEvent updateClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, ClinicalEvent clinicalEvent) {
		setIdentifier(clinicalEvent, facilityCode, staffId, assigningAuthority, patientId,
                      clinicalEvent.getPatientIdentifier().getFirstName(),
                      clinicalEvent.getPatientIdentifier().getLastName(),
                      clinicalEvent.getPatientIdentifier().getSsn());

		if (!isClinicalEventValid(clinicalEvent, true) || ClinicalEventState.SIGNED.equals(clinicalEvent.getClinicalEventState())) {
			throw new WebApplicationException(Status.BAD_REQUEST);
		}

		createOrUpdateLastSelectedNoteTitle(facilityCode, staffId, clinicalEvent.getNote().getTitle());
		readyClinicalEvent(clinicalEvent, ClinicalEventState.DRAFT);
		ClinicalEvent     updatedClinicalEvent = dataLayer.updateClinicalEvent(clinicalEvent);
        try {
            updateTask(updatedClinicalEvent);
        }
        catch(JSONException e) {
            throw new WebApplicationException(Status.INTERNAL_SERVER_ERROR);
        }

        return wrapperToHandleNotFound(updatedClinicalEvent);
	}

	public ClinicalEvent saveClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, ClinicalEvent clinicalEvent) {
		setIdentifier(clinicalEvent, facilityCode, staffId, assigningAuthority, patientId,
                      clinicalEvent.getPatientIdentifier().getFirstName(),
                      clinicalEvent.getPatientIdentifier().getLastName(),
                      clinicalEvent.getPatientIdentifier().getSsn());
		readyClinicalEvent(clinicalEvent, ClinicalEventState.DRAFT);

		if (!isClinicalEventValid(clinicalEvent, false)) {
			throw new WebApplicationException(Status.BAD_REQUEST);
		}

		createOrUpdateLastSelectedNoteTitle(facilityCode, staffId, clinicalEvent.getNote().getTitle());
		ClinicalEvent     savedClinicalEvent = dataLayer.saveClinicalEvent(clinicalEvent);
        try {
            createNewTask(savedClinicalEvent);
        }
        catch(JSONException e) {
            throw new WebApplicationException(Status.INTERNAL_SERVER_ERROR);
        }

        return savedClinicalEvent;
	}

	public ClinicalEvent signClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId, ClinicalNoteSignature noteSignature) {

		ClinicalEvent clinicalEvent = dataLayer.fetchClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEventId);

		if(!isValidSignableClinicalEvent(clinicalEvent)) {
			throw new WebApplicationException(Status.BAD_REQUEST);
		}
		if (NullChecker.isNullish(clinicalEvent.getFacilityCode())){
			clinicalEvent.setFacilityCode(facilityCode);
		}
		if (NullChecker.isNullish(clinicalEvent.getNote().getSignedDate())){
			clinicalEvent.getNote().setSignedDate(DateHelper.getToday());
		}
		// call mdws
		sign(clinicalEvent, noteSignature.getElectronicSignature());

		// update the state of the note to SIGNED
		clinicalEvent.setClinicalEventState(ClinicalEventState.SIGNED);
		ClinicalEvent     signedClinicalEvent = dataLayer.updateClinicalEvent(clinicalEvent);
        try {
            deleteTask(signedClinicalEvent);
        }
        catch(JSONException e) {
            throw new WebApplicationException(Status.INTERNAL_SERVER_ERROR);
        }

        return signedClinicalEvent;
	}
	
	private void sign(ClinicalEvent clinicalEvent, String esig) {
        TIUNotesDataLayerMdws tiuNotesDataLayer = MhpObjectFactory.getInstance().getMdwsTIUNotesDataLayer();
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		
        if(tiuNotesDataLayer.isValidSignature(note, esig)) {
            tiuNotesDataLayer.signEncounterNote(note, esig);
        }
	}

	public Count getClinicalEventCount(String facilityCode, String staffId, String assigningAuthority, String patientId)
	{
		return dataLayer.getClinicalEventCount(facilityCode, staffId, assigningAuthority, patientId);
	}

	public NoteTitle getLastSelectedNoteTitle(String facilityCode, String staffId) {
		NoteTitle lastSelectedTitle = fetchLastSelectedNoteTitle(facilityCode, staffId);

		if (lastSelectedTitle == null) {
			throw new WebApplicationException(Status.NO_CONTENT);
		}

		return lastSelectedTitle;
	}

	public NoteTitle fetchLastSelectedNoteTitle(String facilityCode, String staffId) {
		return dataLayer.fetchLastSelectedNoteTitle(facilityCode, staffId);
	}

	public NoteTitle createOrUpdateLastSelectedNoteTitle(String facilityCode, String staffId, NoteTitle title) {
		NoteTitle savedTitle = null;
		if (isValidTitle(title)) {
			NoteTitle existingTitle = fetchLastSelectedNoteTitle(facilityCode, staffId);

			if (existingTitle == null) {
				savedTitle = dataLayer.insertLastSelectedNoteTitle(facilityCode, staffId, title);
			} else {
				savedTitle = dataLayer.updateLastSelectedNoteTitle(facilityCode, staffId, title);
			}
		}

		return savedTitle;
	}

	private boolean isValidSignableClinicalEvent(ClinicalEvent clinicalEvent) {
		boolean valid = true;

		valid = isClinicalEventValid(clinicalEvent, true)
				&& isValidTitle(clinicalEvent.getNote().getTitle())
				&& NullChecker.isNotNullish(clinicalEvent.getNote().getBody())
				&& !(ClinicalEventState.SIGNED.equals(clinicalEvent.getClinicalEventState()));

		return valid;
	}

	private boolean isClinicalEventValid(ClinicalEvent clinicalEvent, boolean validateId) {
		boolean valid = true;

		if (clinicalEvent == null
				|| clinicalEvent.getPatientIdentifier() == null
				|| NullChecker.isNullish(clinicalEvent.getPatientIdentifier().getAssigningAuthority())
				|| NullChecker.isNullish(clinicalEvent.getPatientIdentifier().getUniqueId())
				|| NullChecker.isNullish(clinicalEvent.getProviderIdentifier().getFacilityCode())
				|| NullChecker.isNullish(clinicalEvent.getProviderIdentifier().getStaffId())
				|| NullChecker.isNullish(clinicalEvent.getFacilityCode())
				|| (validateId && NullChecker.isNullish(clinicalEvent.getId()))) {
			valid = false;
		}

		return valid;
	}

	private boolean isValidTitle(NoteTitle title) {
		boolean valid = false;

		if (title != null && NullChecker.isNotNullish(title.getIen()) && NullChecker.isNotNullish(title.getName())) {
			valid = true;
		}

		return valid;
	}

	private void readyClinicalEvent(ClinicalEvent clinicalEvent, ClinicalEventState clinicalEventState) {
		clinicalEvent.setClinicalEventState(clinicalEventState);
		clinicalEvent.setLastModifiedDate(new Date());
		if (clinicalEvent.getNote() != null) {
			clinicalEvent.getNote().setSignedDate(null);
		}
	}

	/*
	 * Sign note should send asynchronous message to task service and remove task
	 */
	public void createNewTask(ClinicalEvent clinicalEvent) throws JSONException {
		updateTask(clinicalEvent);
	}

	public void updateTask(ClinicalEvent clinicalEvent) throws JSONException {
		// this might not be necessary, not sure if a task needs updating unless it shows note data in task list
		//TaskDataLayer dataLayer = new TaskDataLayer();
		Task task = TaskListTranslator.translateToTaskListItem(clinicalEvent);
		taskDataLayer.saveTask(task);
	}

	public void deleteTask(ClinicalEvent clinicalEvent) throws JSONException {
		//TaskDataLayer dataLayer = new TaskDataLayer();
		Task task = TaskListTranslator.translateToTaskListItem(clinicalEvent);
		taskDataLayer.removeTask(task);
	}

	private <T> T wrapperToHandleNotFound(T object) {
		if (object == null) {
			throw new WebApplicationException(Status.NOT_FOUND);
		}

		return object;
	}
	
	/**
	 * This version of retrieving NoteTitles pulls them from the Mongo database (which was previously populated by a
	 * TableSync).  Because you are directly accessing a database you are not bound by the constraints or rules of how
	 * MDWS returns data from the RPC - thus you can filter an entire result set (not just the 44 records at a time that
	 * mdws returns you).
	 */
	public NoteTitles getMongoNoteTitles(String facilityCode, String staffId, SearchFilter filter)
	{
		return dataLayer.getMongoNoteTitles(facilityCode, filter);
	}

	/**
	 * Returns the NoteTitles starting at searchIndex.
	 * 
	 * @param searchIndex
	 * The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * If searchIndex is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * <font color="red"><b>Example</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * what you searched for.<br/><br/>
	 */
	
	public NoteTitles getMdwsNoteTitles(String facilityCode, SearchIndex searchIndex) {
		String searchString = "";
		String direction = "";
		
		if (searchIndex != null) {
			if (searchIndex.getSearchString() != null)
				searchString = searchIndex.getSearchString();
			if (searchIndex.getDirection() != null)
				direction = searchIndex.getDirection().toString();
		}
		NoteTitles noteTitles = noteTitleDataLayer.getNoteTitles(searchString, direction);
		return noteTitles;
	}

	public boolean isConsultNote(String noteDefinitionIEN) {
		boolean result = noteTitleDataLayer.isConsultNote(noteDefinitionIEN);
		return result;
	}

	public boolean isPrfNote(String noteDefinitionIEN) {
		boolean result = noteTitleDataLayer.isPrfNote(noteDefinitionIEN);
		return result;
	}
	
	public boolean isOneVisitNote(String patientId, String noteDefinitionIEN, String noteTitle, String locationIen, String vistaTimestamp, String serviceCategory) {
		boolean result = noteTitleDataLayer.isOneVisitNote(patientId, noteDefinitionIEN, noteTitle, locationIen, vistaTimestamp, serviceCategory);
		return result;
	}
	
	public boolean isSurgeryNote(String noteDefinitionIEN) {
		boolean result = noteTitleDataLayer.isSurgeryNote(noteDefinitionIEN);
		return result;
	}
}
