package com.agilex.healthcare.patientsearch.datalayer;

import java.util.Calendar;
import java.util.Date;

import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.mongo.MongoClientFactory;
import com.agilex.healthcare.mongo.SharedMongoClient;
import com.agilex.healthcare.patientsearch.translator.PatientSearchTranslator;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;

public class PatientSearchDataLayerMongo implements PatientSearchDataLayer {
	public static final int MAXIMUM_PATIENT_RECORDS = 20;
	
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(PatientSearchDataLayerMongo.class);

	private static DBCollection collection;
	private static DB db;
	private static String providerKey = "provider";
	private static String facilityCodeKey = "facilityCode";
	private static String staffIdKey = "staffId";
	private static String searchedTimeKey = "searchedTime";

	public PatientSearchDataLayerMongo(MongoClient mongoClient) {
		try {
			db = mongoClient.getDB("patient-search");
			collection = db.getCollection("selected-patients");
		} catch (com.mongodb.MongoException e) {
			LOGGER.error("Unable to connect to Mongo DB - Mongo Exception", e);
			throw new RuntimeException("Unable to connect to Mongo DB - Mongo Exception", e);
		}
	}

    public PatientSearchDataLayerMongo(String databaseName, String collectionName) {
        SharedMongoClient sharedMongoClient = MongoClientFactory.getInstance().getMongoClient();
        db = sharedMongoClient.getDB(databaseName);
        collection = db.getCollection(collectionName);
        collection.dropIndexes();
    }

	@Override
	public RecentlySelectedPatients fetchRecentlySelectedPatients(String facilityCode, String staffId) {
		PatientSearchTranslator translator = new PatientSearchTranslator();
		RecentlySelectedPatients recentlySearchedPatients = new RecentlySelectedPatients();

		BasicDBObject query = createQuery(facilityCode, staffId);

		DBCursor iterator = collection.find(query);

		if (iterator.hasNext()) {
			DBObject dbObject = iterator.next();
			recentlySearchedPatients = translator.translate(dbObject);
		} else {
			ProviderIdentifier selectedPatientProvider = new ProviderIdentifier();
			selectedPatientProvider.setFacilityCode(facilityCode);
			selectedPatientProvider.setStaffId(staffId);
			recentlySearchedPatients.setProvider(selectedPatientProvider);
		}
		
		recentlySearchedPatients.sortDescending(searchedTimeKey);
		
		return recentlySearchedPatients;
	}

	private BasicDBObject createQuery(String facilityCode, String staffId) {
		BasicDBObject nestedProvider = new BasicDBObject();
		nestedProvider.put(facilityCodeKey, facilityCode);
		nestedProvider.put(staffIdKey, staffId);
		BasicDBObject query = new BasicDBObject();
		query.put(providerKey, nestedProvider);
		return query;
	}
	
	private void replaceIfExists(RecentlySelectedPatients recentlySearchedPatients, Patient patient, Date searchedTime) {
		//If we don't have any recentlySearchedPatients, just add it.
		int recentlySearchedSize = recentlySearchedPatients.size();

		//Now iterate through all of the recently searched patients and if we find it, replace it. 
		Date oldestTime = Calendar.getInstance().getTime();
		int oldestTimeIdx = -1;
		for (int i=0; i<recentlySearchedSize; i++) {
			SelectedPatient curPat = recentlySearchedPatients.get(i);
			String expectedUniqueId = patient.getPatientIdentifier().getUniqueId();
			String expectedAssigningAuthority = patient.getPatientIdentifier().getAssigningAuthority();
			String actualUniqueId = curPat.getPatient().getPatientIdentifier().getUniqueId();
			String actualAssigningAuthority = curPat.getPatient().getPatientIdentifier().getAssigningAuthority();
			
			if (actualUniqueId.equalsIgnoreCase(expectedUniqueId) && actualAssigningAuthority.equalsIgnoreCase(expectedAssigningAuthority)) {
				//If we find the patient, update it.  Otherwise, figure out what the last patient was.
				curPat.setSearchedTime(searchedTime);
				curPat.setPatient(patient);
				LOGGER.debug("Found this patient in the recentlySearchedPatients, so we just replaced it");
				return;
			}
			else {
				if (curPat.getSearchedTime().before(oldestTime)) {
					oldestTime = curPat.getSearchedTime();
					oldestTimeIdx = i;
				}
			}
		}
		
		
		//Remove the oldest recently selected patient if there's no room for this one.
		if (recentlySearchedSize >= MAXIMUM_PATIENT_RECORDS) {
			LOGGER.debug("Didn't have any recentlySearchedPatients, so we just removed the oldest one");
			if (oldestTimeIdx == -1)
				throw new IllegalStateException("Unable to determine where to add/replace the recently selected patient");
			recentlySearchedPatients.remove(oldestTimeIdx);
		}
		
		LOGGER.debug("Adding patient to recentlySearchedPatients");
		SelectedPatient pat = new SelectedPatient();
		pat.setSearchedTime(searchedTime);
		pat.setPatient(patient);
		recentlySearchedPatients.add(pat);
	}

	@Override
	public SelectedPatient addSelectedPatient(Date searchedTime, String facilityCode, String staffId, Patient patient) {
		//Grab the collection of recently searched patients for this provider.
		RecentlySelectedPatients recentlySearchedPatients = fetchRecentlySelectedPatients(facilityCode, staffId);
		
		replaceIfExists(recentlySearchedPatients, patient, searchedTime);
		
		PatientSearchTranslator translator = new PatientSearchTranslator();
		DBObject dbObject = translator.translate(facilityCode, staffId, recentlySearchedPatients);
		BasicDBObject query = createQuery(facilityCode, staffId);
		collection.update(query, dbObject, true, false);
		
		SelectedPatient pat = new SelectedPatient();
		translator.setAge(patient);
		pat.setPatient(patient);
		pat.setSearchedTime(searchedTime);
		return pat;
	}
}