package com.agilex.healthcare.notewriter.translator;

import java.util.Calendar;

import junit.framework.Assert;

import org.junit.Test;

import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTo;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;
import com.agilex.healthcare.utility.DateHelper;

public class ClinicalNoteTranslatorTest {
	@Test
	public void translateClinicalEvent() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		validateProcedures(clinicalEvent, note);
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	@Test
	public void translateClinicalEventNoDiagnoses() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		clinicalEvent.getEncounter().setDiagnosisCodes(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		
		validateProcedures(clinicalEvent, note);
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	@Test
	public void translateClinicalEventNoProcedureModifiers() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		for (int i=0; i<clinicalEvent.getEncounter().getProcedureCodes().size(); i++)
			clinicalEvent.getEncounter().getProcedureCodes().get(i).setModifiers(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		validateProcedures(clinicalEvent, note, false);
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	@Test
	public void translateClinicalEventNoProcedures() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		clinicalEvent.getEncounter().setProcedureCodes(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	@Test
	public void translateClinicalEventNoProviders() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		clinicalEvent.getEncounter().setProviders(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		validateProcedures(clinicalEvent, note);
		
		validateVisitRelatedTos(clinicalEvent, note);
	}

	@Test
	public void translateClinicalEventNoVisitRelatedTos() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		clinicalEvent.getEncounter().setVisitRelatedTos(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		validateProcedures(clinicalEvent, note);
		validateProviders(clinicalEvent, note);
		
	}
	
	@Test
	public void translateClinicalEventNoVisitTypes() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		clinicalEvent.getEncounter().setVisitTypes(null);
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		
		validateProcedures(clinicalEvent, note);
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	@Test
	public void translateClinicalEventNoVisitTypeModifiers() {
		ClinicalEvent clinicalEvent = ClinicalEventUtilities.createClinicalEvent(Calendar.getInstance().getTime());
		for (int i=0; i<clinicalEvent.getEncounter().getVisitTypes().size(); i++) {
			Assert.assertNotNull(clinicalEvent.getEncounter().getVisitTypes().get(i).getSections());
			for (int k=0; k<clinicalEvent.getEncounter().getVisitTypes().get(i).getSections().size(); k++) {
				clinicalEvent.getEncounter().getVisitTypes().get(i).getSections().get(k).setModifiers(null);
			}
		}
		EncounterNote note = ClinicalNoteTranslator.translateToEncounterNote(clinicalEvent);
		validateNote(clinicalEvent, note);
		validateDiagnoses(clinicalEvent, note);
		validateProcedures(clinicalEvent, note, false);
		validateProviders(clinicalEvent, note);
		validateVisitRelatedTos(clinicalEvent, note);
	}
	
	private void validateDiagnoses(ClinicalEvent clinicalEvent, EncounterNote note) {
		Assert.assertNotNull(note.getDiagnoses());
		Assert.assertEquals(clinicalEvent.getEncounter().getDiagnosisCodes().size(), note.getDiagnoses().size());
		
		for (int i=0;i<clinicalEvent.getEncounter().getDiagnosisCodes().size(); i++) {
			Assert.assertEquals(clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getCode(), note.getDiagnoses().get(i).getBasicCode().getCode());
			Assert.assertEquals(clinicalEvent.getEncounter().getDiagnosisCodes().get(i).getShortDescription(), note.getDiagnoses().get(i).getBasicCode().getDescription());
		}
	}
	
	private void validateProcedures(ClinicalEvent clinicalEvent, EncounterNote note) {
		validateProcedures(clinicalEvent, note, true);
	}
	private void validateProcedures(ClinicalEvent clinicalEvent, EncounterNote note, boolean bValidateModifiers) {
		EncounterProcedureCPTCodes procedures = clinicalEvent.getEncounter().getProcedureCodes();
		
		if (clinicalEvent.getEncounter().getVisitTypes() != null) {
			for (int i=0; i<clinicalEvent.getEncounter().getVisitTypes().size(); i++) {
				Assert.assertNotNull(clinicalEvent.getEncounter().getVisitTypes().get(i).getSections());
				EncounterProcedureCPTCodes sections = clinicalEvent.getEncounter().getVisitTypes().get(i).getSections();
				if (procedures == null) {
					procedures = new EncounterProcedureCPTCodes();
				}
				procedures.addAll(sections);
			}
		}
		
		validateProceduresInternal(procedures, note, bValidateModifiers);
	}
	private void validateProceduresInternal(EncounterProcedureCPTCodes procedures, EncounterNote note, boolean bValidateModifiers) {
		
		Assert.assertNotNull(note.getProcedures());
		Assert.assertEquals(procedures.size(), note.getProcedures().size());
		
		for (int i=0;i<procedures.size(); i++) {
			Assert.assertEquals(procedures.get(i).getCode(), note.getProcedures().get(i).getBasicCode().getCode());
			Assert.assertEquals(procedures.get(i).getShortDescription(), note.getProcedures().get(i).getBasicCode().getDescription());
			if (bValidateModifiers) {
				Assert.assertNotNull(note.getProcedures().get(i).getModifiers());
				Assert.assertEquals(procedures.get(i).getModifiers().size(), note.getProcedures().get(i).getModifiers().size());
				for (int k=0; k<procedures.get(i).getModifiers().size(); k++) {
					Assert.assertEquals(procedures.get(i).getModifiers().get(k).getCode(), note.getProcedures().get(i).getModifiers().get(k).getCode());
					Assert.assertEquals("CPT", note.getProcedures().get(i).getModifiers().get(k).getCodeType());
					Assert.assertEquals(procedures.get(i).getModifiers().get(k).getName(), note.getProcedures().get(i).getModifiers().get(k).getDescription());
					Assert.assertEquals(procedures.get(i).getModifiers().get(k).getIen(), note.getProcedures().get(i).getModifiers().get(k).getIen());
				}
			}
		}
	}
	
	private void validateProviders(ClinicalEvent clinicalEvent, EncounterNote note) {
		Assert.assertNotNull(clinicalEvent.getEncounter());
		Assert.assertNotNull(clinicalEvent.getEncounter().getProviders());
		String providers = "";
		for (int i=0; i<clinicalEvent.getEncounter().getProviders().size(); i++){
			EncounterProvider ep = clinicalEvent.getEncounter().getProviders().get(i);
			//Only send those providers that have actually been selected.
			if (ep.isSelected() == false)
				continue;
			
			String id = ep.getProvider().getProviderIen();
			if (providers.length() == 0)
				providers += id;
			else if (ep.isPrimary())					//If you use more than one, the first one in
				providers = id + "," + providers;		//the list will be interpreted as the primary provider
			else
				providers += "," + id;
		}
		Assert.assertEquals(providers, note.getProviders());
	}
	
	private void validateVisitRelatedTos(ClinicalEvent clinicalEvent, EncounterNote note) {
		Assert.assertNotNull(clinicalEvent.getEncounter());
		Assert.assertNotNull(clinicalEvent.getEncounter().getVisitRelatedTos());
		String visitRelatedCodes = "";
		for (int i=0; i<clinicalEvent.getEncounter().getVisitRelatedTos().size(); i++){
			VisitRelatedTo vrt = clinicalEvent.getEncounter().getVisitRelatedTos().get(i);
			String code = vrt.getCode();
			if (visitRelatedCodes.length() == 0)
				visitRelatedCodes += code;
			else
				visitRelatedCodes += "," + code;
		}
		Assert.assertEquals(visitRelatedCodes, note.getVisitRelatedCodes());
	}

	private void validateNote(ClinicalEvent clinicalEvent, EncounterNote note) {
		Assert.assertEquals(clinicalEvent.getPatientIdentifier().getUniqueId(), note.getPatientIen());
		Assert.assertEquals(clinicalEvent.getFacilityCode(), note.getFacilityCode());
		Assert.assertEquals(clinicalEvent.getEncounter().getLocationId(), note.getLocationIen());
		Assert.assertEquals(clinicalEvent.getNote().getTitle().getIen(), note.getNoteTitleIen());
		Assert.assertEquals(clinicalEvent.getEncounter().getServiceCategory(), note.getServiceCategory());
		Assert.assertEquals(clinicalEvent.getEncounter().isCurrentAdmission() ? "1" : "0", note.getInpatient());
		Assert.assertEquals(clinicalEvent.getNote().getBody().toString(), note.getNoteBody());
		Assert.assertEquals(DateHelper.format(clinicalEvent.getNote().getSignedDate(), DateHelper.VISTA_DATETIME_FORMAT), note.getDate());
	}
}
