package com.agilex.healthcare.mdws.connection;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.apache.cxf.transports.http.configuration.HTTPClientPolicy;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;

import com.agilex.healthcare.mdws.datalayer.MdwsResponseException;
import com.agilex.healthcare.mdws.datalayer.MdwsResponseException.UserMessages;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.DataSourceArray;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.FindPatientService;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.FindPatientServiceSoap;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.security.AppUser;

public class EncounterMdwsConnection extends MdwsConnection<FindPatientServiceSoap> {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(EncounterMdwsConnection.class);
	private static final long serialVersionUID = -498237725339909479L;
	private FindPatientServiceSoap port;

	/**
	 * Calls the MDWS service LVEncounterSvc.connect if the authenticatedFlag is false - it will usually be true
	 * since most apps will login to a site with AuthorizationServices and that connection is used from that point on.
	 */
	public EncounterMdwsConnection(MdwsConfiguration configuration, String siteCode) {
		super(configuration);
		this.port = this.establishConnection(configuration, siteCode);
	}

	@Override
	public FindPatientServiceSoap getService() {
		return this.port;
	}
	/**
	 * Pings a HTTP URL and returns true if it didn't time out.
	 */
	private static boolean ping(String url, int timeout) {
		url = url.replaceFirst("https", "http"); // Otherwise an exception may be thrown on invalid SSL certificates.

		HttpURLConnection connection = null;
		try {
			connection = (HttpURLConnection) new URL(url).openConnection();
			connection.setConnectTimeout(timeout);
			connection.setReadTimeout(timeout);
			connection.setRequestMethod("HEAD");
			connection.getResponseCode(); //NOTE: We are getting a 405 responseCode from mdws - but that just proves that the server is up (which is what we care about)
			return true;
		} catch (IOException exception) {
			return false;
		} finally {
			if (connection != null) {
				try { connection.disconnect(); } catch (Exception e) { logger.info("Problem disconnecting from ping"); }
			}
		}
	}

	@Override
	protected FindPatientServiceSoap establishConnection(MdwsConfiguration configuration, String siteCode) {
		FindPatientServiceSoap port = obtainConnection(configuration, siteCode);
		setMdwsSessionId(port);

		super.refreshTimestamp();
		return port;
	}

	public static void setPortSessionId(String sessionId, FindPatientServiceSoap port) {
		Client client = ClientProxy.getClient(port);

		HTTPClientPolicy policy = new HTTPClientPolicy();
		HTTPConduit httpConduit = (HTTPConduit)client.getConduit();

		policy.setCookie("ASP.NET_SessionId=" + sessionId);

		httpConduit.setClient(policy);
	}

	public static void setMdwsSessionId(FindPatientServiceSoap port) {
		MhpUser mhpUser = getMhpUser();
		String sessionId = mhpUser.getMdwsInfo().getSessionId();
		if (sessionId == null)
			throw new MdwsResponseException(UserMessages.CONNECTION_ERROR, "sessionId was null for EncounterMdwsConnection.setMdwsSessionId");
		setPortSessionId(sessionId, port);
	}

	public static MhpUser getMhpUser(){
        SecurityContext ctx = SecurityContextHolder.getContext();
        if (ctx == null)
        	throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, "Unable to get Spring Context");
        if (ctx.getAuthentication() == null)
        	throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, "Unable to get Spring Authentication");
        if (ctx.getAuthentication().getPrincipal() == null)
        	throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, "Unable to get Spring Principal");
        if (((AppUser)ctx.getAuthentication().getPrincipal()).getMhpUser() == null)
        	throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, "Unable to get Spring MhpUser");


        AppUser appUser = (AppUser) ctx.getAuthentication().getPrincipal();
        MhpUser mhpUser = appUser.getMhpUser();
        mhpUser.setUserName(appUser.getUsername());

        return mhpUser;
	}

	public static FindPatientServiceSoap obtainConnection(MdwsConfiguration configuration, String siteCode) {
		return obtainConnection(true, configuration, siteCode);
	}

	public static FindPatientServiceSoap obtainConnection(boolean authenticated, MdwsConfiguration configuration, String siteCode) {
		String baseUrl = "";
		if (authenticated) {
			MhpUser mhpUser = getMhpUser();
			baseUrl = mhpUser.getMdwsInfo().getBaseUrl();
			if (baseUrl == null)
				throw new MdwsResponseException(UserMessages.CONNECTION_ERROR, "baseUrl was null for obtainConnection");
		}
		else {
			baseUrl = configuration.getEndPointBaseUrl();
		}

		String url = baseUrl + configuration.getEndPointUrl();

		if (configuration.isPingWsdlBeforeConnect() && ping(url, configuration.getWsConnectTimeout()) == false)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, "Unable to reach the server at " + url);

		URL wsdlLocation = constructWsdlUrl(url);
		FindPatientService mdwsService = null;
		try {
			mdwsService = new FindPatientService(wsdlLocation, new QName("http://mdws.domain/patientFinder/FindPatientService", "FindPatientService"));
		} catch (Throwable t) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, t.getMessage());
		}
		FindPatientServiceSoap port = mdwsService.getFindPatientServiceSoap();

		((BindingProvider) port).getRequestContext().put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
		((BindingProvider) port).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, url);
		((BindingProvider) port).getRequestContext().put("javax.xml.ws.client.connectionTimeout", configuration.getWsConnectTimeout());
		((BindingProvider) port).getRequestContext().put("javax.xml.ws.client.receiveTimeout", configuration.getWsRequestTimeout());

		if (!authenticated) {
			DataSourceArray dsa = null;
			try {
				dsa = port.connect(siteCode);
			} catch (Throwable t) {
				throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, t.getMessage());
			}
			if (dsa.getFault() != null)
				throw new MdwsResponseException(MdwsResponseException.UserMessages.CONNECTION_ERROR, dsa.getFault().getMessage());
		}

		return port;
	}

	private static URL constructWsdlUrl(String endPointUrl){
		String wsdlLocation = endPointUrl + "?WSDL";
		URL wsdlLocationURL = null;
		try {
			wsdlLocationURL = new URL(wsdlLocation);
		} catch (Exception e) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_WSDL, e.getMessage());
		}

		return wsdlLocationURL;
	}

}
