package com.agilex.healthcare.mdws.datalayer;

import com.agilex.healthcare.mdws.connection.EmrMdwsConnection;
import com.agilex.healthcare.mdws.connection.EncounterConnectionDataLayer;
import com.agilex.healthcare.mdws.connection.EncounterMdwsConnection;
import com.agilex.healthcare.mdws.connection.MdwsConfiguration;
import com.agilex.healthcare.mdws.datalayer.util.DateHelper;
import com.agilex.healthcare.mdws.datalayer.util.LexiconCodeLookupUtil;
import com.agilex.healthcare.mdws.datalayer.util.ModifierUtil;
import com.agilex.healthcare.mdws.datalayer.util.ProblemListUtil;
import com.agilex.healthcare.mdws.datalayer.util.ProvidersUtil;
import com.agilex.healthcare.mdws.datalayer.util.SvcConditionRelatedTosUtil;
import com.agilex.healthcare.mdws.datalayer.util.UserMenuOptionsUtil;
import com.agilex.healthcare.mdws.datalayer.util.VisitTypeUtil;
import com.agilex.healthcare.mdws.domain.Codes;
import com.agilex.healthcare.mdws.domain.ProblemLists;
import com.agilex.healthcare.mdws.domain.Providers;
import com.agilex.healthcare.mdws.domain.SvcConditionRelatedTos;
import com.agilex.healthcare.mdws.domain.UserMenuOptions;
import com.agilex.healthcare.mdws.domain.VisitTypes;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.FaultTO;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.TaggedLexiconEntryArray;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.UserMenuOptionArray;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedModifiersByCptArrays;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedPatientProblemListDiagnosesArrays;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedProviderArrays;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedSvcConnectedRelatedToConditionsArrays;
import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.TaggedVisitTypeCptByClinicArrays;

public class EncounterDataLayerMdws extends EncounterConnectionDataLayer implements EncounterDataLayer {

	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(EncounterDataLayerMdws.class);
	private static final String DEFAULT_DATE_FORMAT = "MM/dd/yyyy";
	private static final String ICD_CODE_SYS_FLAG = "1";
	private static final String ICD_CODE_TYPE = "ICD";
	private static final String CPT_CODE_TYPE = "CPT";
//----------------------------------------------------------------------------------------------------------------------

	private void siteCodeCheck(String siteCode) {
		if (siteCode == null || siteCode.isEmpty())
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, "The siteCode was null or empty.");
	}

//----------------------------------------------------------------------------------------------------------------------

	public EncounterDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

//----------------------------------------------------------------------------------------------------------------------
	/**
	 * Calls the MDWS service LVEncounterSvc.getModifiersByCpt for the given patient
	 */
	@Override
	public Codes getModifierCodes(String siteCode, String cptCode) {
		siteCodeCheck(siteCode);
		EncounterMdwsConnection connection = getSiteConnection(siteCode);
		Codes retValue = getModifierCodesFromMdws(connection, cptCode);
		return retValue;
	}

	private Codes getModifierCodesFromMdws(EncounterMdwsConnection connection, String cptCode) {
		if (cptCode == null || cptCode.isEmpty())
			return new Codes();


		TaggedModifiersByCptArrays result = connection.getService().getModifiersByCpt(cptCode);
		MdwsResponseValidator.validateMdwsObject(result);
		Codes retValue = ModifierUtil.translateLVTaggedModifiersByCptArrays(result);
		return retValue;
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getVisitTypeCptByClinic for the given patient
	 */
	@Override
	public VisitTypes getVisitTypeByClinic(String siteCode, String locationIen, String date) {
		siteCodeCheck(siteCode);
		EncounterMdwsConnection connection = getSiteConnection(siteCode);

		VisitTypes retValue = getVisitTypeByClinicFromMdws(connection, locationIen, date);
		return retValue;
	}

	private VisitTypes getVisitTypeByClinicFromMdws(EncounterMdwsConnection connection, String locationIen, String date) {
		if (locationIen == null || locationIen.isEmpty())
			return new VisitTypes();

		TaggedVisitTypeCptByClinicArrays result = connection.getService().getVisitTypeCptByClinic(locationIen, date == null ? "" : date);
		MdwsResponseValidator.validateMdwsObject(result);
		VisitTypes retValue = VisitTypeUtil.translateLVTaggedVisitTypeCptByClinicArrays(result);
		return retValue;
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getLexiconEntries for the given patient (passing in ICD for the codeType).
	 */
	@Override
	public Codes getDiagnosisSubCodes(String siteCode, String icd9Code, String visitDate) {
		String date = visitDate == null ? "" : visitDate;
		return getLexiconCodeLookup(siteCode, ICD_CODE_TYPE, icd9Code, date, new String[]{}, ICD_CODE_SYS_FLAG);
	}

	/**
	 * Calls the MDWS service LVEncounterSvc.getLexiconEntries for the given patient (passing in CPT for the codeType
	 * and filtering out anything which is not CPT-4 in the results). For example HCPCS returned together with CPT-4 
	 * so we want to filter procedures that came from Coding System = HCPCS.
	 * Health Care Common Procedure Coding System (HCPCS)-  This code set, established and maintained by the Centers for 
	 * Medicare & Medicaid Services (CMS), primarily represents items and supplies and non physician services not covered by 
	 * the American Medical Association CPT-4 codes.
	 */
	@Override
	public Codes getProceduresSubCodes(String siteCode, String cptCode, String visitDate) {
		String date = visitDate == null ? "" : visitDate;
		return getLexiconCodeLookup(siteCode, CPT_CODE_TYPE, cptCode, date, new String[] {"CPT-4"}, "");
	}
	
	private Codes getLexiconCodeLookup(String siteCode, String codeType, String code, String visitDate, String[] filter, String codeSystem) {
		siteCodeCheck(siteCode);
		EmrMdwsConnection connection = getEmrSiteConnection(siteCode);

		Codes retValue = getMdwsLexiconCodeLookup(connection, codeType, code, visitDate, filter, codeSystem);
		return retValue;
	}
	
	private Codes getMdwsLexiconCodeLookup(EmrMdwsConnection connection, String codeType, String code, String visitDate, String[] filter, String codeSystem) {
		if (codeType == null || codeType.isEmpty())
			return new Codes();
		if (code == null || code.isEmpty())
			return new Codes();

		try {
			TaggedLexiconEntryArray result = connection.getService().getLexiconEntries(code, codeType, visitDate, codeSystem);
			MdwsResponseValidator.validateMdwsObject(result);
			Codes retValue = LexiconCodeLookupUtil.translateLVTaggedLexiconEntryArray(result, filter);
			return retValue;
		}
		catch (javax.xml.ws.WebServiceException e) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.TIMEOUT, e.getMessage(), e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getPatientProblemListDiagnoses for the given patient
	 */
	@Override
	public ProblemLists getPatientProblemListDiagnoses(String siteCode, String patientIen, String visitDate) {
		siteCodeCheck(siteCode);
		EncounterMdwsConnection connection = getNewSiteConnectionAndSelectPatient(siteCode, patientIen);

		TaggedPatientProblemListDiagnosesArrays result = null;

		try {
			result = connection.getService().getPatientProblemListDiagnoses(visitDate == null ? "" : visitDate);
		}
		catch (Exception e) {
			logger.error("Vista saveNoteAndEncounter Service Exception " + e.getMessage());
			FaultTO faultTO = new FaultTO();
			faultTO.setMessage(e.getMessage());
			throw new MdwsResponseException(faultTO.getMessage());
		}
		MdwsResponseValidator.validateMdwsObject(result);
		ProblemLists retValue = ProblemListUtil.translateLVTaggedPatientProblemListDiagnosesArrays(result);
		return retValue;
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getProvidersBySite for the given patient
	 */
	@Override
	public Providers getProvidersBySite(String siteCode) {
		siteCodeCheck(siteCode);
		EncounterMdwsConnection connection = getSiteConnection(siteCode);

		Providers retValue = getProvidersBySite(connection);
		return retValue;
	}

	private Providers getProvidersBySite(EncounterMdwsConnection connection) {
		TaggedProviderArrays result = connection.getService().getProvidersBySite();
		MdwsResponseValidator.validateMdwsObject(result);
		Providers retValue = ProvidersUtil.translateLVTaggedProviderArrays(result);
		return retValue;
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getSvcConnectedRelatedToConditions for the given patient
	 */
	@Override
	public SvcConditionRelatedTos getSvcConnectedRelatedToConditions(String siteCode, String patientIen, String visitDate, String locationIen) {
		siteCodeCheck(siteCode);
		EncounterMdwsConnection connection = getNewSiteConnectionAndSelectPatient(siteCode, patientIen);

		//the VA's RPC requires the date to be zero if there is no date (unlike our others which can pass an empty string).
		String vistaVisitDate = "";
		if (visitDate == null || visitDate.isEmpty())
			vistaVisitDate = "0";
		else
			vistaVisitDate = DateHelper.parseAndConvertDateToVistaFormat(visitDate, DEFAULT_DATE_FORMAT);

		TaggedSvcConnectedRelatedToConditionsArrays result = null;
		try {
			result = connection.getService().getSvcConnectedRelatedToConditions(locationIen, vistaVisitDate == null ? "" : vistaVisitDate);
		}
		catch (Exception e) {
			logger.error("Vista getSvcConnectedRelatedToConditions Service Exception " + e.getMessage());
			FaultTO faultTO = new FaultTO();
			faultTO.setMessage(e.getMessage());
			throw new MdwsResponseException(faultTO.getMessage());
		}
		MdwsResponseValidator.validateMdwsObject(result);
		SvcConditionRelatedTos retValue = SvcConditionRelatedTosUtil.translateLVTaggedSvcConnectedRelatedToConditionsArrays(result);
		return retValue;
	}

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.getUserMenuOptions for the given patient
	 */
	@Override
	public UserMenuOptions getUserMenuOptions(String siteCode, String userId) {
		siteCodeCheck(siteCode);
		EmrMdwsConnection connection = getEmrSiteConnection(siteCode);

		UserMenuOptionArray result = connection.getService().getUserMenuOptions(userId);
		MdwsResponseValidator.validateMdwsObject(result);
		UserMenuOptions retValue = UserMenuOptionsUtil.translateLVUserMenuOptionArray(result);
		return retValue;
	}

//------------------------------------Utility Methods-------------------------------------------------------------------

	private EncounterMdwsConnection getSiteConnection(String siteCode) {
		EncounterMdwsConnection connection = super.getConnection(siteCode);

		return connection;
	}

	private EmrMdwsConnection getEmrSiteConnection(String siteCode) {
		EmrDataLayerMdws	emrDataLayerMdws = new EmrDataLayerMdws(mdwsConfiguration);
		EmrMdwsConnection connection = emrDataLayerMdws.getAuthenticatedConnection(siteCode);

		return connection;
	}

	private EmrMdwsConnection getEmrNewSiteConnectionAndSelectPatient(String siteCode, String patientIen) {
		EmrDataLayerMdws	emrDataLayerMdws = new EmrDataLayerMdws(mdwsConfiguration);
		EmrMdwsConnection connection = emrDataLayerMdws.getAuthenticatedConnectionWithPatientSelect(siteCode, patientIen);

		return connection;
	}

	private EncounterMdwsConnection getNewSiteConnectionAndSelectPatient(String siteCode, String patientIen) {
		EncounterMdwsConnection connection = super.getConnection(patientIen, siteCode);
		return connection;
	}

}
