package com.agilex.healthcare.mdws.datalayer;

import java.util.UUID;

import com.agilex.healthcare.mdws.generatedwsdl.encounterservice.FaultTO;

public class MdwsResponseException extends RuntimeException {
	private static final long serialVersionUID = 3875667528783433812L;
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(MdwsResponseException.class);

	private String uuid = UUID.randomUUID().toString();
	private String expandedMessage;
	private UserMessages userMessage;

	// ----------------------------------------------------------------------------------------------------------------------

	public enum UserMessages {
		INVALID_WSDL("Invalid Server Configuration for WSDL"),
		INVALID_PATIENT("Invalid Patient"),
		CONNECTION_ERROR("Unable to connect to Vista system"),
		INVALID_FACILITY_CODE("The facility code provided was not valid."),
		INVALID_LOCATION_IEN("The location IEN provided was not valid."),
		INVALID_ACCESS_VERIFY("You have entered an invalid username or password!"),
		INVALID_ESIG("Invalid credentials. Please re-try."),
		NOT_AUTHORIZED("User not authorized to sign note"),
		INVALID_NOTE_TITLE_IEN("An Error Occurred with Note Title IEN"),
		INVALID_CPT_CODE("An Error Occurred with CPT Code"),
		INVALID_ICD_CODE("An Error Occurred with ICD Code"),
		MISSING_CPT_CODE("Missing CPT Code"),
		MISSING_ICD_CODE("Missing ICD Code"),
		INVALID_VISIT_RELATED_TOS("An Error Occurred with Visit Related Tos"),
		INVALID_NOTE_BODY("Must supply note body"),
		INVALID_PROVIDER_IEN(" Must supply at least one providerIEN"),
		TIMEOUT("Service timed out, please re-try."),
		CONNECTIONS_NOT_READY("Connectivity with mdws was lost, please logout and login again"),
		INVALID_DATA_RETURNED("An Error Occurred with Invalid Data Returned"),
		INVALID_RECORD_ID("Invalidly formed record ID"),
		INVALID_INPUT("The input received was not valid."),
		INVALID_SERVICE_CATEGORY("Input string for service category was not in a correct format."),
		INVALID_INPATIENT("Input string for inpatient was not in a correct format."),
		INVALID_DATE("Date cannot be null"),
		INVALID_PROCEDURES("The procedures were not valid"),
		INVALID_COMMENT("Must supply a comment"),
		INVALID_CONSULT_IEN("The consult ien was not valid"),
		INVALID_DIAGNOSES("The diagnoses were not valid"),
		INVALID_HEALTH_FACTORS("The health factors were not valid"),
		GENERIC_ERROR("An Error Occurred");

		private final String value;

		private UserMessages(String value) {
			this.value = value;
		}

		@Override
		public String toString() {
			return this.value;
		}

		public static UserMessages fromValue(String value) {
			if (value == null || value.isEmpty())
				throw new MdwsResponseException(UserMessages.GENERIC_ERROR, "The UserMessages cannot be null");

			for (UserMessages c : UserMessages.values()) {
				if (c.value.equals(value))
					return c;
			}
			throw new MdwsResponseException(UserMessages.GENERIC_ERROR, "An unknown UserMessages was passed into fromValue");
		}
	}

	public enum MdwsResponseExceptionLogType {DEBUG, INFO, WARN, ERROR}
	/**
	 * Depending on the type of error, the message will be written out as debug, info, warn, or error.
	 * The return value indicates what the type of message was.
	 */
	public static MdwsResponseExceptionLogType logException(org.apache.commons.logging.Log exceptionLogger, MdwsResponseException exception) {
		MdwsResponseExceptionLogType messageType = MdwsResponseExceptionLogType.ERROR;
		switch (exception.userMessage) {
			case INVALID_WSDL:
			case CONNECTION_ERROR:
			case GENERIC_ERROR:
			case INVALID_FACILITY_CODE:
			case INVALID_LOCATION_IEN:
			case INVALID_NOTE_TITLE_IEN:
			case INVALID_CPT_CODE:
			case INVALID_ICD_CODE:
			case MISSING_CPT_CODE:
			case MISSING_ICD_CODE:
				exceptionLogger.error(exception.expandedMessage);
				messageType = MdwsResponseExceptionLogType.ERROR;
			break;

			case INVALID_PATIENT:
				exceptionLogger.warn(exception.expandedMessage);
				messageType = MdwsResponseExceptionLogType.WARN;
			break;

			case TIMEOUT:
			case CONNECTIONS_NOT_READY:
			case INVALID_ESIG:
			case INVALID_ACCESS_VERIFY:
			case NOT_AUTHORIZED:
			case INVALID_VISIT_RELATED_TOS:
			case INVALID_DATA_RETURNED:
			case INVALID_NOTE_BODY:
			case INVALID_PROVIDER_IEN:
			case INVALID_RECORD_ID:
			case INVALID_INPUT:
			case INVALID_SERVICE_CATEGORY:
			case INVALID_INPATIENT:
			case INVALID_DATE:
			case INVALID_PROCEDURES:
			case INVALID_COMMENT:
			case INVALID_CONSULT_IEN:
			case INVALID_DIAGNOSES:
			case INVALID_HEALTH_FACTORS:
				exceptionLogger.info(exception.expandedMessage);
				messageType = MdwsResponseExceptionLogType.INFO;
			break;

			default:
				exceptionLogger.error(exception.expandedMessage);
				messageType = MdwsResponseExceptionLogType.ERROR;
			break;
		}
		return messageType;
	}

	// ----------------------------------------------------------------------------------------------------------------------

	/**
	 *
	 * @param message The message to display to an end user
	 * @param expandedMessage A more detailed message that a developer would want to know.
	 */
	public MdwsResponseException(UserMessages message, String expandedMessage) {
		this(message, expandedMessage, null);
	}

	/**
	 *
	 * @param message The message to display to an end user
	 * @param expandedMessage A more detailed message that a developer would want to know.
	 */
	public MdwsResponseException(UserMessages message, String expandedMessage, Throwable cause) {
		super(message == null ? UserMessages.GENERIC_ERROR.toString() : message.toString(), cause);
		this.userMessage = message;
		this.expandedMessage = String.format("UUID: %s%n%s%nAdditional Info: %s", uuid, message == null ? UserMessages.GENERIC_ERROR.toString() : message.toString(), expandedMessage);
		MdwsResponseException.logException(logger, this);
	}

	/**
	 *
	 * @param message The message to display to an end user
	 * @param fault A MDWS FaultTO with the reason for a failure in MDWS code.
	 */
	public MdwsResponseException(UserMessages message, FaultTO fault) {
		super(message == null ? UserMessages.GENERIC_ERROR.toString() : message.toString());
		this.userMessage = message;
		this.expandedMessage = String.format("UUID: %s%n%s%nAdditional Info: %s", uuid, message == null ? UserMessages.GENERIC_ERROR.toString() : message.toString(), fault.getMessage());
		MdwsResponseException.logException(logger, this);
	}

	/**
	 *
	 * @param fault A MDWS FaultTO with the reason for a failure in MDWS code - this is parsed to try creating
	 * a message to show the end user - SEE @link {@link MdwsResponseFaultTOFilter}.
	 */
	public MdwsResponseException(FaultTO fault) {
		super(MdwsResponseFaultTOFilter.filterFault(fault).toString());
		this.userMessage = MdwsResponseFaultTOFilter.filterFault(fault);
		this.expandedMessage = String.format("UUID: %s%n%s%nAdditional Info: %s", uuid, this.getMessage(), fault.getMessage());
		MdwsResponseException.logException(logger, this);
	}

    /**
     *
     * @param fault A MDWS fault message with the reason for a failure in MDWS code - this is parsed to try creating
     * a message to show the end user - SEE @link {@link MdwsResponseFaultTOFilter}.
     */
    public MdwsResponseException(String message) {
            this.userMessage = MdwsResponseFaultTOFilter.getResponseType(message);
            this.expandedMessage = String.format("UUID: %s%n%s%nAdditional Info: %s", uuid, this.getMessage(), message);
            MdwsResponseException.logException(logger, this);
    }

	// ----------------------------------------------------------------------------------------------------------------------

	public String getUuid() {
		return uuid;
	}

	public String getExpandedMessage() {
		return expandedMessage;
	}

	public void setExpandedMessage(String expandedMessage) {
		this.expandedMessage = expandedMessage;
	}

	public UserMessages getUserMessage() {
		return userMessage;
	}

	// ----------------------------------------------------------------------------------------------------------------------

	@Override
	public String toString() {
		return this.expandedMessage;
	}
}
