package com.agilex.healthcare.mdws.datalayer;

import com.agilex.healthcare.mdws.connection.EmrMdwsConnection;
import com.agilex.healthcare.mdws.connection.MdwsConfiguration;
import com.agilex.healthcare.mdws.connection.TIUNotesConnectionDataLayer;
import com.agilex.healthcare.mdws.connection.TIUNotesMdwsConnection;
import com.agilex.healthcare.mdws.datalayer.util.CptCodeUtil;
import com.agilex.healthcare.mdws.datalayer.util.DiagnosesUtil;
import com.agilex.healthcare.mdws.datalayer.util.NoteBodyUtil;
import com.agilex.healthcare.mdws.datalayer.util.SigningUtil;
import com.agilex.healthcare.mdws.domain.EncounterImmunization;
import com.agilex.healthcare.mdws.domain.EncounterNote;
import com.agilex.healthcare.mdws.generatedwsdl.emrservice.FaultTO;

public class TIUNotesDataLayerMdws extends TIUNotesConnectionDataLayer implements TIUNotesDataLayer {

	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(TIUNotesDataLayerMdws.class);
//----------------------------------------------------------------------------------------------------------------------

	private void siteCodeCheck(String siteCode) {
		if (siteCode == null || siteCode.isEmpty())
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_FACILITY_CODE, "The siteCode was null or empty.");
	}

//----------------------------------------------------------------------------------------------------------------------

        public TIUNotesDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
            super(mdwsConfiguration);
        }

//----------------------------------------------------------------------------------------------------------------------

	/**
	 * Calls the MDWS service LVEncounterSvc.saveNoteAndEncounter for the given patient<br/><br/>
	 *
	 * <b>locationIen (REQUIRED)</b><br/>
	 * <b>noteTitleIen (REQUIRED)</b><br/>
	 * <b>noteBody (REQUIRED)</b><br/>
	 * <b>consultIen (REQUIRED only if the noteTitleIen selected is for a consult.)</b><br/>
	 * <b>esig (REQUIRED)</b><br/><br/>
	 *
	 * <b>EncounterNote.serviceCategory</b> (OPTIONAL): 'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical)
	 * (defaults to 'A' if left blank).<br/><br/>
	 *
	 * <b>EncounterNote.inpatient</b> (OPTIONAL): '1' for true or '0' for false (defaults to '0' if left blank or if set to anything other than '1').<br/><br/>
	 *
	 * <b>EncounterNote.date</b> (OPTIONAL): should be formatted yyyymmdd.hhmmss (.hhmmss portion can be left out if desired).
	 * This date will be used as a timestamp for the ENCOUNTER only. The NOTE will always be stamped with the current date/time.
	 * If no date is passed, then the ENCOUNTER will also be stamped with the current date/time.<br/><br/>
	 *
	 * <b>EncounterNote.providers (REQUIRED)</b>: Must be a comma-separated list of providerDUZ values.
	 * Example: '5,119,541'. It is fine to pass only one provider. If you pass more than one, the first one in the list
	 * will be interpreted as the primary provider. The primary provider must have privileges to sign a note in VistA,
	 * or the service will return an error.<br/><br/>
	 *
	 * <b>EncounterNote.procedures</b> (OPTIONAL): Procedures is a delimited string.
	 * Use semicolons to delimit the individual procedures; carets to delimit the fields within each procedure
	 * (each modifier may be treated as a separate field); and pipes to delimit the modifierCode from the modifierIen.
	 * If your encounter has no procedures (or visit types), you can pass an empty string for this parameter.
	 * Note that passing an empty string is only allowed if you are using service category 'E' and/or
	 * inpatient value '1'. Otherwise, passing an empty string for procedures will return an error.<br/>
	 * <code>
	 * 	<b>Format:</b><br/>
	 * 		cpt1Code^cpt1Description^cpt1Comment^cpt1Modifier1Code|cpt1Modifier1Ien^cpt1Modifier2Code|cpt1Modifier2Ien^cpt1Modifier3Code|cpt1Modifier3Ien;cpt2Code^cpt2Description^cpt2Comment;cpt3Code^cpt3Description<br/>
	 * 	<b>Example:</b><br/>
	 * 		90662^Influenza Virus Vaccine, Split Virus, Preservative Free, Enhanced Immunogenicity via Increased Antigen Content, for Intramuscular use (CPT-4 90662)^flu shot^AK|497^EY|421^GR|250^KX|423;93530^Right Heart Catheterization, for Congenital Cardiac Anomalies (CPT-4 93530)^heart catheterization<br/>
	 * </code>
	 * <b>cptCode and cptDescription</b> are required for each procedure. Comment and modifiers may be omitted.
	 * Any modifier that is included must have both a code and an IEN.<br/><br/>
	 *
	 * <b>EncounterNote.diagnoses</b> (OPTIONAL): Diagnoses is a delimited string.
	 * Use semicolons to delimit the individual diagnoses; carets to delimit the fields within each diagnosis.
	 * If your encounter has no diagnoses, you can pass an empty string for this parameter.
	 * <code>
	 * 	<b>Format:</b><br/>
	 * 		diagnosis1Code^diagnosis1Description^diagnosis1IsPrimary^diagnosis1AddToProblemList^diagnosis1Comment;diagnosis2Code^diagnosis2Description^diagnosis2IsPrimary^diagnosis2AddToProblemList;diagnosis3Code^diagnosis3Description<br/>
	 * 	<b>Example:</b><br/>
	 * 		427.5^Cardiac Arrest * (ICD-9-CM 427.5)^1^1^heart arrest;668.11^Cardiac complications of anesthesia or other sedation in labor and delivery, delivered (ICD-9-CM 668.11)^0^1^complications;V12.53^Personal History of Sudden Cardiac Arrest (ICD-9-CM V12.53)^0^0^personal history<br/>
	 * </code>
	 * <b>icd9Code and description</b> are required for each diagnosis.
	 * isPrimary and addToProblemList should be '1' for true and '0' for false, and will be set to '0' if they are
	 * omitted or invalid. Comment is optional.<br/><br/>
	 *
	 * <b>EncounterNote.healthFactors</b> (OPTIONAL): A delimited string. Use semicolons to delimit the individual
	 * health factors; carets to delimit the fields within each health factor. If your encounter has no health factors,
	 * you can pass an empty string for this parameter.
	 * <code>
	 * 	<b>Format:</b><br/>
	 * 		healthFactor1Code^healthFactor1Description^healthFactor1LevelSeverity^healthFactor1Comment;healthFactor2Code^healthFactor2Description^healthFactor2LevelSeverity^healthFactor2Comment<br/>
	 * 	<b>Example:</b><br/>
	 * 		500085^REFUSED INFLUENZA IMMUNIZATION^^patient already had flu shot;47^ACTIVATE CERVIX CANCER SCREEN^M^<br/>
	 * <b>Code and description</b> are required for each health factor. Level/severity is optional; if it is used,
	 * it must be set to 'M' (Minimal), 'MO' (Moderate), or 'H' (Heavy/severe). Comment is optional.<br/><br/>
	 *
	 * <b>EncounterNote.visitRelatedCodes</b> (OPTIONAL): This is an optional comma-separated list. Note: One of the items in the
	 * comma-separated list used to be called "SAC" for "Southwest Asia Conditions", and that one has
	 * been renamed to "EC" for "Environmental Contaminants" (these are two different names for the same thing).
	 * So if you want to use that one, the input should be EC rather than SAC.
	 * Enter visitRelatedCodes in this format: 'SC,CV,AO,IR,EC,SHD,MST,HNC'.
	 * Include those that are checked ('yes') and leave out those that are not ('no'). Codes may be in any order.
	 * 'EC' means Southwest Asia Conditions (Environmental Conditions).<br/><br/>
	 *
	 * <b>(Don't include the quote marks in any of these parameters.)</b>
	 */
	@Override
	public EncounterNote signEncounterNote(EncounterNote note, String esig) {
		siteCodeCheck(note.getFacilityCode());
		//EncounterMdwsConnection connection = super.getConnection(note.getPatientIen(), note.getFacilityCode());
		TIUNotesMdwsConnection connection = super.getTIUNotesConnection(note.getPatientIen(), note.getFacilityCode());

        com.agilex.healthcare.mdws.generatedwsdl.tiunotesservice.TextTO result = null;

		try {
			String procedures = CptCodeUtil.translate(note.getProcedures());
			String diagnoses = DiagnosesUtil.translate(note.getDiagnoses());

			String locationIen = note.getLocationIen();
			String noteTitleIen = note.getNoteTitleIen();
			String serviceCategory = note.getServiceCategory();
			String inpatient = note.getInpatient();
			String date = note.getDate();
			String providers = note.getProviders();
			String healthFactors = note.getHealthFactors();
			String visitRelatedCodes = note.getVisitRelatedCodes();
			String noteBody = NoteBodyUtil.translate(note);
			String consultIen = note.getConsultIen() == null ? "" : note.getConsultIen();

			SigningUtil.validateEncounterNote(
					locationIen, noteTitleIen, consultIen, serviceCategory,
					inpatient,  date, providers, procedures,
					diagnoses, healthFactors, visitRelatedCodes, noteBody, esig);

			result = connection.getService().saveNoteAndEncounter(
					locationIen, noteTitleIen, consultIen, serviceCategory,
					inpatient,  date, providers, procedures,
					diagnoses, healthFactors, visitRelatedCodes, noteBody, esig);
		} catch (MdwsResponseException ex) {
			throw ex;
		} catch (Exception e) {
			logger.error("Vista saveNoteAndEncounterFull Service Exception " + e.getMessage());
			FaultTO faultTO = new FaultTO();
			faultTO.setMessage(e.getMessage());
			throw new MdwsResponseException(faultTO.getMessage());
		}
		MdwsResponseValidator.validateMdwsObject(result);
		return note;
	}

	/**
	 * Calls the MDWS service LVEncounterSvc.saveNoteAndEncounterImmunizations for the given patient<br/><br/>
	 *
	 * EncounterImmunization.locationIen: cannot be null.<br/><br/>
	 *
	 * EncounterImmunization.noteTitleIen: cannot be null.<br/><br/>
	 *
	 * EncounterImmunization.serviceCategory: cannot be null.<br/>
	 * The serviceCategory is 'A' (ambulatory) for vaccine given by VA
	 * and 'E' (event historical) for vaccine not given by VA (defaults to 'A').<br/><br/>
	 *
	 * EncounterImmunization.date: cannot be null.<br/>
	 * The date (optional) should be formatted yyyymmdd.hhmmss (.hhmmss portion can be left out if desired).<br/>
	 * If you choose serviceCategory "A", it means an immunization was given that day at the VA, any date passed in
	 * will be completely ignored. That's by design because the date of the immunization is required to be the current
	 * date/time. If you choose serviceCategory "E", which means an immunization was not given at the VA, an invalid
	 * date should cause it to fail. In that case it tries to use the passed-in date as the date the immunization was
	 * actually received.<br/><br/>
	 *
	 * EncounterImmunization.cptCode: cannot be null.<br/>
	 * If vaccine was given outside VA, use 'E' and a date.<br/>
	 * If no vaccine was given anywhere, use 'E' and no date.<br/>
	 * If vaccine was given by VA, use 'A', a cptCode (or multiple cptCodes separated by commas), and no date.<br/><br/>
	 *
	 * EncounterImmunization.visitRelatedTos: cannot be null.<br/>
	 * Enter visitRelatedTos in this format: 'SC,CV,AO,IR,EC,SHD,MST,HNC'.<br/>
	 * Include those that are checked ('yes') and leave out those that are not ('no').<br/>
	 * 'EC' means Southwest Asia Conditions (Environmental Conditions).<br/><br/>
	 *
	 * EncounterImmunization.comment: cannot be null.<br/><br/>
	 */
	@Override
	public EncounterImmunization signImmunization(EncounterImmunization immunization, String esig) {
		siteCodeCheck(immunization.getFacilityCode());
		TIUNotesMdwsConnection connection = super.getTIUNotesConnection(immunization.getPatientIen(), immunization.getFacilityCode());

        com.agilex.healthcare.mdws.generatedwsdl.tiunotesservice.TextTO result = null;
		try {
			String locationIen = immunization.getLocationIen();
			String noteTitleIen = immunization.getNoteTitleIen();
			String serviceCategory = immunization.getServiceCategory();
			String date = immunization.getDate();
			String cptCode = immunization.getCptCode();
			String visitRelatedTos = immunization.getVisitRelatedTos();
			String comment = immunization.getComment();

			SigningUtil.validateImmunization(
					locationIen, noteTitleIen, serviceCategory,
					date, cptCode, visitRelatedTos, comment, esig);
			
			result = connection.getService().saveNoteAndEncounterImmunizations(
					locationIen,
					noteTitleIen,
					serviceCategory,
					date,
					cptCode,
					visitRelatedTos,
					comment,
					esig);
		} catch (MdwsResponseException ex) {
			throw ex;
		} catch (Exception e) {
			logger.error("Vista saveNoteAndEncounterImmunizations Service Exception " + e.getMessage());
			FaultTO faultTO = new FaultTO();
			faultTO.setMessage(e.getMessage());
			throw new MdwsResponseException(faultTO.getMessage());
		}
		MdwsResponseValidator.validateMdwsObject(result);
		return immunization;
	}
    
    public boolean isValidSignature(EncounterNote note, String esig) {
        EmrDataLayerMdws emrDataLayerMdws = new EmrDataLayerMdws(mdwsConfiguration);
        EmrMdwsConnection connection = emrDataLayerMdws.getAuthenticatedConnection(note.getFacilityCode());
        com.agilex.healthcare.mdws.generatedwsdl.emrservice.TextTO textTO = connection.getService().isValidEsig(esig);
        if(textTO.getText().equalsIgnoreCase("TRUE")) {
            return true;
        }
        throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_ESIG.toString());
    }

//----------------------------------------------------------------------------------------------------------------------


}
