package com.agilex.healthcare.utility;

import org.joda.time.DateTime;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class VistADateParser {
	protected static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(VistADateParser.class);

	private static final String[] VISTA_DATE_FORMAT_ARRAY = {
		// Parse order matters
		"yyyyMMdd.HHmmss",
		"yyyyMMdd.HHmm",
		"yyyyMMdd.HH",
		"yyyyMMdd",
		"MM/dd/yy",
		"MM/dd/yyyy",
		"MMM d, yyyy@HH:mm:ss",
		"MMM d, yyyy@HH:mm",
		"MMM d, yyyy@HH",
		"MMMM dd, yyyy"
	};

	/**
	 * Attempts to parse date string with any vista format
	 * @param date
	 * @return
	 */
	public Date parseAnyVistaDateString(String date){
		if(NullChecker.isNotNullish(date)){
			// Parse order matters
			Date parsedDate = parseSpecificVistaDateFormats(date);

			if (parsedDate == null) {
				parsedDate = parseVistaDateFormatsRelativeToToday(date);
			}

			if (parsedDate == null) {
				parsedDate = parseVistaDateFormatsRelativeToNow(date);
			}

			if (parsedDate == null) {
				throw new RuntimeException(String.format("Failed to parse date [%s] as any vista date format", date));
			}

			return parsedDate;
		}
		return null;
	}

	/**
	 * Attempts to parse date per specified vista date formats
	 * @param date
	 * @return
	 */
	private Date parseSpecificVistaDateFormats(String date) {
		Date parsedDate = null;

		for (String vistaDateFormat : VISTA_DATE_FORMAT_ARRAY) {
			parsedDate = parse(date, vistaDateFormat);

			if (parsedDate != null) {
				break;
			}
		}

		return parsedDate;
	}

	/**
	 * Attempts to parse dates that are relative to now (i.e. NOW, N, N+n, N-n)
	 * @param date
	 * @return
	 */
	private Date parseVistaDateFormatsRelativeToNow(String date) {
		Date parsedDate = null;

		if (isDateRelativeToNow(date)) {
			String[] timeParts = date.split("@");
			String datePart = timeParts[0];
			boolean parseFailed = false;

			Calendar now = Calendar.getInstance();

			int daysToAdd = 0;
			String[] nowParts;

			if (isDatePartNegativelyRelative(datePart)) {
				nowParts = datePart.split("\\-");

				if (nowParts.length == 2) {
					try {
						int parsedDays = Integer.parseInt(nowParts[1]);
						daysToAdd = parsedDays * -1;
					} catch (NumberFormatException e) {
						parseFailed = true;
					}
				}

			} else if (isDatePartPositivelyRelative(datePart)) {
				nowParts = datePart.split("\\+");

				if (nowParts.length == 2) {
					try {
						daysToAdd = Integer.parseInt(nowParts[1]);
					} catch (NumberFormatException e) {
						parseFailed = true;
					}
				}
			} else if (!isDatePartNow(datePart)) {
				parseFailed = true;
			}

			if (!parseFailed) {
				now.add(Calendar.DAY_OF_YEAR, daysToAdd);
				parsedDate = now.getTime();
			}
		}

		return parsedDate;
	}

	/**
	 * Attempts to parse dates that are relative to today (i.e. TODAY, T, T+n, T-n)
	 * @param date
	 * @return
	 */
	private Date parseVistaDateFormatsRelativeToToday(String date) {
		Date parsedDate = null;

		if (isDateRelativeToToday(date)) {
			String[] timeParts = date.split("@");
			String datePart = timeParts[0];
			boolean parseFailed = false;

			Calendar now = Calendar.getInstance();

			Calendar today = now;
			now.set(Calendar.HOUR_OF_DAY, 0);
			now.set(Calendar.MINUTE, 0);
			now.set(Calendar.SECOND, 0);
			now.set(Calendar.MILLISECOND, 0);

			int daysToAdd = 0;
			String[] todayParts;

			if (isDatePartNegativelyRelative(datePart)) {
				todayParts = datePart.split("\\-");

				if (todayParts.length == 2) {
					try {
						int parsedDays = Integer.parseInt(todayParts[1]);
						daysToAdd = parsedDays * -1;
					} catch (NumberFormatException e) {
						parseFailed = true;
					}
				}

			} else if (isDatePartPositivelyRelative(datePart)) {
				todayParts = datePart.split("\\+");

				if (todayParts.length == 2) {
					try {
						daysToAdd = Integer.parseInt(todayParts[1]);
					} catch (NumberFormatException e) {
						parseFailed = true;
					}
				}
			} else if (!isDatePartToday(datePart)) {
				parseFailed = true;
			}

			if (!parseFailed) {
				today.add(Calendar.DAY_OF_YEAR, daysToAdd);
				parsedDate = today.getTime();

				if (timeParts.length > 1 && timeParts[1] != null && !timeParts[1].equals("U")) {
					DateTime dateTime = new DateTime(parsedDate);
					dateTime = dateTime.plusHours(getHours(timeParts[1]));
					dateTime = dateTime.plusMinutes(getMinutes(timeParts[1]));
					dateTime = dateTime.plusSeconds(getSeconds(timeParts[1]));
					parsedDate = dateTime.toDate();
				}
			}
		}

		return parsedDate;
	}

	/**
	 * Parses using SimpleDateFormat, but doesn't throw an exception on a failed parse
	 * @param date
	 * @param format
	 * @return
	 */
	private Date parse(String date, String format) {
		Date parsedDate;
		try {
			SimpleDateFormat df = new SimpleDateFormat(format);
			parsedDate = df.parse(date);
			LOGGER.debug(String.format("Date [%s] matches format [%s]", date, format));
		} catch (ParseException e) {
			LOGGER.debug(String.format("Date [%s] does not match format [%s]", date, format));
			parsedDate = null;
		}
		return parsedDate;
	}

	private boolean isDateRelativeToToday(String date) {
		boolean dateRelativeToToday = false;
		if (date.startsWith("T")) {
			dateRelativeToToday = true;
		}
		return dateRelativeToToday;
	}

	private boolean isDatePartToday(String datePart) {
		boolean datePartToday = false;
		if (datePart.equals("T") || datePart.equals("TODAY")) {
			datePartToday = true;
		}
		return datePartToday;
	}

	private boolean isDateRelativeToNow(String date) {
		boolean dateRelativeToToday = false;
		
		// Dates starting with Nov or November may be included in this match, handle accordingly
		if (date.startsWith("N")) {
			dateRelativeToToday = true;
		}
		return dateRelativeToToday;
	}

	private boolean isDatePartNow(String datePart) {
		boolean datePartToday = false;
		if (datePart.equals("N") || datePart.equals("NOW")) {
			datePartToday = true;
		}
		return datePartToday;
	}

	private boolean isDatePartNegativelyRelative(String datePart) {
		boolean dateNegativelyRelative = false;
		if (datePart.contains("-")) {
			dateNegativelyRelative = true;
		}
		return dateNegativelyRelative;
	}

	private boolean isDatePartPositivelyRelative(String datePart) {
		boolean datePositivelyRelative = false;
		if (datePart.contains("+")) {
			datePositivelyRelative = true;
		}
		return datePositivelyRelative;
	}

	private int getHours(String time){
		if(time.length() == 1){
			return Integer.parseInt(time+"0");
		}
		else if(time.length() >= 2){
			String hours = time.substring(0,2);
			return Integer.parseInt(hours);
		}
		return 0;
	}

	private int getMinutes(String time){
		if(time.length() >= 3){
			String minutes = "0";
			if(time.length() == 3)
				minutes = time.substring(2,3) + "0";
			if(time.length() >= 4)
				minutes = time.substring(2,4);
			return Integer.parseInt(minutes);
		}
		return 0;
	}

	private int getSeconds(String time){
		if(time.length() >= 5){
			String seconds = "0";
			if(time.length() == 5)
				seconds = time.substring(4,5) + "0";
			if(time.length() == 6)
				seconds = time.substring(4,6);
			return Integer.parseInt(seconds);
		}
		return 0;
	}
}
