package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.appointment;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.Calendar;
import java.util.Date;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.utility.DateHelper;

@ContextConfiguration(locations = {"classpath:integrationApplicationContext.xml"})
@RunWith(SpringJUnit4ClassRunner.class)
public class AppointmentDataLayerMdwsVistaTest {

    @Resource
    MdwsConfiguration mdwsVistaConfiguration;

    @BeforeClass
	public static void login() {
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}

	private PatientIdentifier createPatientIdentifier() {
        PatientIdentifier patientIdentifier = new PatientIdentifier();
        patientIdentifier.setAssigningAuthority("dfn-500");
        patientIdentifier.setUniqueId("237");
        return patientIdentifier;
    }

    private MdwsConfiguration getMdwsConfiguration() {
        return mdwsVistaConfiguration;
    }

    @Test
    public void verifyFetchAppointments() {
        AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
        Appointments appointments = dl.fetchAppointmentsByPatient(createPatientIdentifier(), DateFilterFactory.createEmptyFilter());
        assertNotNull(appointments);
        assertTrue(String.format("Size was: %d", appointments.size()), appointments.size() > 50);
    }
    
    @Test
    public void verifyNoResultsFetchAppointments() {
        AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
        PatientIdentifier patientIdentifier = new PatientIdentifier("dfn-500", "156");
        Appointments appointments = dl.fetchAppointmentsByPatient(patientIdentifier, DateFilterFactory.createEmptyFilter());
        assertNotNull(appointments);
        Assert.assertEquals(0, appointments.size());
    }
    
    @Test
    public void verifyGetVisits() {
        AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
        Appointments appointments = dl.getVisits(createPatientIdentifier(), DateFilterFactory.createEmptyFilter());
        assertNotNull(appointments);
        assertTrue(String.format("Size was: %d", appointments.size()), appointments.size() > 30);
        
        Appointment apt = appointments.get(0);
        Date startAndEndDate = DateHelper.parseVistaDefaultDateTIme("20000406.0900");
        Assert.assertNull(apt.getAlarmOffset());
        Assert.assertNull(apt.getAppointmentDuration());
        Assert.assertEquals(startAndEndDate, apt.getAppointmentStartDate());
        Assert.assertEquals(startAndEndDate, apt.getAppointmentStartDate());
        Assert.assertNull(apt.getClinicalNotes());
        Assert.assertEquals("32", apt.getClinicId());
        Assert.assertEquals("PRIMARY CARE", apt.getClinicName());
        Assert.assertNull(apt.getClinicPhone());
        Assert.assertNull(apt.getClinicType());
        Assert.assertNull(apt.getClinicVisitId());
        TestUtil.assertNullDataIdentifier(apt.getDataIdentifier());
        Assert.assertNull(apt.getFacilityName());
        TestUtil.assertNullPatientIdentifier(apt.getPatientIdentifier());
        Assert.assertNull(apt.getPatientName());
        Assert.assertNull(apt.getPatientSSN());
        Assert.assertNull(apt.getProviderId());
        Assert.assertNull(apt.getProviderName());
        Assert.assertNull(apt.getReasonForVisit());
        Assert.assertNull(apt.getRepeatFrequency());
        Assert.assertNull(apt.getSourceSystem());
        Assert.assertEquals("INPATIENT APPOINTMENT", apt.getStatus());
        Assert.assertEquals("A", apt.getStatusCode());
        TestUtil.assertNullPatientIdentifier(apt.getSurrogateIdentifier());
    }
    
    @Test
    public void verifyGetVisitsFilters() {
        AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
        Appointments all = dl.getVisits(createPatientIdentifier(), DateFilterFactory.createEmptyFilter());
        Appointments filter = dl.getVisits(createPatientIdentifier(), DateFilterFactory.createFilterFromDate("01/01/2001", "01/01/2014"));
        assertNotNull(all);
        assertNotNull(filter);
        assertTrue(String.format("Expected all size of %d to be greater than filtered size of %d", all.size(), filter.size()), all.size() > filter.size());
    }
    
    @Test
    public void verifyGetVisitsSortingDescending() {
    	AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
        Appointments appointments = dl.getVisits(createPatientIdentifier(), DateFilterFactory.createEmptyFilter());
        assertNotNull(appointments);
        assertTrue(String.format("Size was: %d", appointments.size()), appointments.size() > 30);
        
        appointments.sortDescending("appointmentStartDate");
        
        for (int i=0; i<appointments.size(); i++) {
        	Appointment appt = appointments.get(i);
        	if (i == appointments.size()-1)
        		continue;
        	
        	for (int k=i+1; k<appointments.size(); k++) {
        		Appointment apptTest = appointments.get(k);
        		long apptDate = appt.getAppointmentStartDate().getTime();
        		long apptTestDate = apptTest.getAppointmentStartDate().getTime();
        		
        		if (apptDate < apptTestDate)
        			Assert.fail("Found an appointment that was not in descending order");
        	}
        }
    }
    
    @Test
    public void verifyGetVisitsOneYear() {
    	AppointmentDataLayerMdws dl = new AppointmentDataLayerMdws(getMdwsConfiguration());
    	DateFilter dateFilter = DateFilterFactory.createEmptyFilter();
    	Date startDate = dateFilter.getStartDate();
		Date endDate = dateFilter.getEndDate();
		if (startDate == null || endDate == null) {
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR_OF_DAY, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			endDate = cal.getTime();
			cal.add(Calendar.YEAR, -1);
			cal.set(Calendar.HOUR_OF_DAY, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			startDate = cal.getTime();
			dateFilter = DateFilterFactory.createFilterFromDate(startDate, endDate);
		}
		
        Appointments appointments = dl.getVisits(createPatientIdentifier(), dateFilter);
        assertNotNull(appointments);
        assertTrue(String.format("Size was: %d", appointments.size()), appointments.size() > 30);
        
        appointments.sortDescending("appointmentStartDate");
        
        //Make sure the appointments appear within the range.
        for (int i=0; i<appointments.size(); i++) {
        	Appointment appt = appointments.get(i);
        	Date apptStartTime = appt.getAppointmentStartDate();
        	Date apptEndTime = appt.getAppointmentEndDate();
        	assertTrue(String.format("Appointments Start Time (%tc) was before interval requested (%tc)", apptStartTime, startDate), startDate.getTime() < apptStartTime.getTime());
        	assertTrue(String.format("Appointments End Time (%tc) was after interval requested (%tc)", apptEndTime, endDate), endDate.getTime() > apptEndTime.getTime());
        }
    }
}
