package com.agilex.healthcare.mobilehealthplatform.datalayer.vista.medication;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationDataLayerMdws;
import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.AuthenticationUtil;
import com.agilex.healthcare.mobilehealthplatform.datalayer.vista.util.TestUtil;
import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;

@ContextConfiguration(locations = { "classpath:integrationApplicationContext.xml" })
@RunWith(SpringJUnit4ClassRunner.class)
public class MedicationDataLayerMdwsVistaTest {

	private static final String TEST_PATIENT_ID = "1";
	
	@Resource
	MdwsConfiguration mdwsVistaConfiguration;
	private MedicationDataLayerMdws dl;

	@BeforeClass
	public static void login(){
		AuthenticationUtil.loginUser("777", "cprs1234", "cprs4321$", "1");
	}
	@AfterClass
	public static void logoff() {
		AuthenticationUtil.logoffUser();
	}
	
	@Before
	public void setup() {
		dl = new MedicationDataLayerMdws(mdwsVistaConfiguration);
	}

	private PatientIdentifier createPid(String patientId) {
		PatientIdentifier pId = new PatientIdentifier();
		pId.setAssigningAuthority("dfn-777");
		pId.setUniqueId(patientId);
		return pId;
	}

	@Test
	public void verifyFetchMedications() {
		PatientIdentifier pid = createPid(TEST_PATIENT_ID);
		Medications medications = dl.getMedications(pid, false);

		assertTrue(String.format("Zero medications were found for patient [id: %s]", TEST_PATIENT_ID), medications.size() > 0);
	}

	@Test
	public void verifyFetchMedicationsByFilter() {
		MedicationFilter filter = new MedicationFilter();
		filter.addStatus("EXPIRED");
		PatientIdentifier pid = createPid(TEST_PATIENT_ID);
		Medications medications = dl.getMedications(pid, filter);

		assertTrue(String.format("Zero medications were found for patient [id: %s]", TEST_PATIENT_ID), medications.size() == 0);
	}

	@Test
	public void testGetMedicationLite() {
		String TEST_MEDICATION_VA_ID = "126P;O";
		String TEST_MEDICATION_UNIQUE_ID = "a127d1ec0767526b1715d7b728272567de12f22f";
		PatientIdentifier pid = createPid(TEST_PATIENT_ID);
		Medication medication = dl.getMedicationLite(pid, TEST_MEDICATION_UNIQUE_ID);
		assertNotNull(medication);
		assertEquals(TEST_MEDICATION_UNIQUE_ID, medication.getId());
		assertEquals(TEST_MEDICATION_VA_ID, medication.getVaMedicationId());
		assertNull(medication.getCalculatedNextFillDate());
		assertNotNull(medication.getDataIdentifier());
		assertEquals(TEST_MEDICATION_UNIQUE_ID, medication.getDataIdentifier().getUniqueId());
		assertEquals("", medication.getDaysSupply());
		assertEquals("SULFACETAMIDE NA 10% OPH SOLN", medication.getDrugName());
		assertNull(medication.getEndDate());
		assertNull(medication.getExpiredDate());
		assertNull(medication.getLastFilledDate());
		assertEquals("SULFACETAMIDE NA 10% OPH SOLN  Qty: 15", medication.getMedicationDetail());
		assertEquals("Outpatient", medication.getMedicationSource());
		assertEquals("10894", medication.getOrderNumber());
		TestUtil.assertNullPatientIdentifier(medication.getPatientIdentifier());
		assertNull(medication.getPharmacy());
		assertNull(medication.getPrescriptionId());
		assertEquals("15", medication.getQuantity());
		assertNull(medication.getRefillAction());
		assertEquals("", medication.getRefills());
		assertNull(medication.getRefillsRemaining());
		assertEquals("\\ Sig:", medication.getSig());
		assertEquals("IAE GOLDDEV", medication.getSourceSystem());
		assertNull(medication.getStartDate());
		assertEquals("PENDING", medication.getStatus());
		TestUtil.assertNullPatientIdentifier(medication.getSurrogateIdentifier());
		assertNull(medication.getTotalRefills());
	}
	
	@Test
	public void getMedicationDetailNoMedId() {
		assertMedicationDetail(TEST_PATIENT_ID, null, MdwsResponseException.UserMessages.INVALID_INPUT,
				"medicationId cannot be null or empty",
				"Expected MdwsResponseException");
	}
	
	@Test
	public void getMedicationDetailMedNotInList() {
		assertMedicationDetail(TEST_PATIENT_ID, "1", MdwsResponseException.UserMessages.INVALID_INPUT,
				"medicationId ('1') was unable to be found in this: ['dadf5984dacfbe1db118b7f81ff1d5e11fa4a93d', '9a7f06ccad4b48c025e55775683b9cdcb34ba621', 'a127d1ec0767526b1715d7b728272567de12f22f']",
				"Expected MdwsResponseException");
	}
	
	@Test
	public void getMedicationDetailNoMeds() {
		assertMedicationDetail("56", "1", MdwsResponseException.UserMessages.INVALID_INPUT,
				"Zero medications were returned so we can't find an id in that",
				"Expected MdwsResponseException");
	}
	
	@Test
	public void getMedicationDetailUnknownPatient() {
		assertMedicationDetail("1000", "1", MdwsResponseException.UserMessages.INVALID_PATIENT,
				"Patient is unknown to CPRS",
				"Expected MdwsResponseException");
	}
	
	private void assertMedicationDetail(String patientId, String medicationId, MdwsResponseException.UserMessages errorType,
		String expandedMessage, String defaultFailure) {
		boolean receivedError = false;
		try {
			PatientIdentifier pid = createPid(patientId);
			dl.getMedicationDetail(pid, medicationId);
		}
		catch (MdwsResponseException e) {
			receivedError = true;
			if (e.getUserMessage().equals(errorType) == false)
				Assert.fail("Expected " + errorType + ", Received " + e.getUserMessage());
			if (e.getExpandedMessage().indexOf(expandedMessage) < 0)
				Assert.fail("Expected " + expandedMessage + ", Received " + e.getExpandedMessage());
		}
		if (receivedError == false)
			Assert.fail(defaultFailure);
	}
}
