package com.agilex.healthcare.mobilehealthplatform.datalayer.appointment;

import java.util.Date;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedAppointmentArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedVisitArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfigurationHelper;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.AppointmentTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.VisitTranslator;

public class AppointmentDataLayerMdws extends MdwsEmrDataLayer implements AppointmentDataLayer, MentalHealthAppointmentDataLayer {

	public AppointmentDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Throws an UnsupportedOperationException
	 * @throws UnsupportedOperationException
	 */
	@Override
	public Appointments getProviderAppointments(String providerId, DateFilter filter) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Throws an UnsupportedOperationException
	 * @throws UnsupportedOperationException
	 */
	@Override
	public Appointments getClinicAppointments(String clinicId, DateFilter filter) {
		throw new UnsupportedOperationException();
	}

	/**
	 * Calls the MDWS service EmrSvc.getAppointments for the given patient
	 */
	@Override
	public Appointments fetchAppointmentsByPatient(PatientIdentifier patientIdentifier, DateFilter filter) {
		return fetchAppointmentsByPatient(patientIdentifier, filter, false);
	}

	/**
	 * Calls the MDWS service EmrSvc.getAppointments for the given patient
	 */
	@Override
	public Appointments fetchAppointmentsByPatient(PatientIdentifier patientIdentifier, DateFilter filter, boolean shouldIncludeDetails) {
		// shouldIncludeDetails should be ignored. Notes are now requested separately via ClinicalNoteDataLayer
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		TaggedAppointmentArrays taggedAppointments = authenticatedConnection.getPort().getAppointments();
		super.disconnectFromCdwIfApplicable(authenticatedConnection);
		MdwsResponseValidator.validateMdwsObject(taggedAppointments);
		Appointments translated = translate(taggedAppointments);

		Appointments filteredAppointments = null;
		filteredAppointments = filter(translated, filter);

		return filteredAppointments;
	}

	/**
	 * Calls the MDWS service EmrSvc.getAppointments for the given patient
	 */
	@Override
	public Appointment fetchAppointment(PatientIdentifier patientIdentifier, String appointmentId) {
		Appointments appointments = fetchAppointmentsByPatient(patientIdentifier, DateFilterFactory.createEmptyFilter());
		for (Appointment appointment : appointments) {
			if (appointmentId != null && appointmentId.equalsIgnoreCase(appointment.getUniqueId())) {
				return appointment;
			}
		}
		return null;

	}

	/**
	 * Calls the MDWS service EmrSvc.getMentalHealthAppointments for the given patient
	 */
	@Override
	public Appointments fetchMentalHealthAppointments(PatientIdentifier patientIdentifier, DateFilter filter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		TaggedAppointmentArrays taggedAppointments = authenticatedConnection.getPort().getMentalHealthAppointments();
		super.disconnectFromCdwIfApplicable(authenticatedConnection);
		MdwsResponseValidator.validateMdwsObject(taggedAppointments);
		
		Appointments translated = translate(taggedAppointments);
		return filter(translated, filter);
	}

	private Appointments translate(TaggedAppointmentArrays arrays) {
		AppointmentTranslator translator = new AppointmentTranslator();
		return translator.translate(arrays);
	}
	
	private Appointments translate(TaggedVisitArray array) {
		VisitTranslator translator = new VisitTranslator();
		return translator.translate(array);
	}

	private Appointments filter(Appointments appointments, DateFilter dateFilter) {
		return DateFilterer.filterByDate(appointments, dateFilter);
	}
	
	/**
	 * Calls the MDWS service EmrSvc.getVisits for the given patient
	 */
	@Override
	public Appointments getVisits(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);
		
		String siteType  =  new MdwsConfigurationHelper().extractSiteTypeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		String startDate = getStartDateAsString(dateFilter.getStartDate(), siteType);
		String endDate = getEndDateAsString(dateFilter.getEndDate(), siteType);
		
		TaggedVisitArray taggedVisits = authenticatedConnection.getPort().getVisits(startDate, endDate);
		super.disconnectFromCdwIfApplicable(authenticatedConnection);
		MdwsResponseValidator.validateMdwsObject(taggedVisits);
		Appointments translated = translate(taggedVisits);

		return translated;
	}
	
	private String getStartDateAsString(Date date, String system) {
		return MdwsDateTranslationUtil.startDateBySystem(date, system);
	}

	private String getEndDateAsString(Date date, String system) {
		return MdwsDateTranslationUtil.endDateBySystem(date, system);
	}
}
