package com.agilex.healthcare.mobilehealthplatform.datalayer.facility;

import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicDataLayer;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedHospitalLocationArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.HospitalLocationTranslator;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserUtil;

public class ClinicDataLayerMdws extends MdwsEmrDataLayer implements ClinicDataLayer {

	public ClinicDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}

	/**
	 * Returns the Clinics starting at searchString.<br/><br/>
	 * 
	 * Calls the MDWS service EmrSvc.getClinics
	 * 
	 * @param searchString
	 * The searchString should be thought of as being identical to looking up a name in a telephone book.
	 * When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * If searchString is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * <font color="red"><b>Example</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * what you searched for.<br/><br/>
	 */
	@Override
	public Clinics fetchClinics(String ignored, String searchString) {
		if (searchString == null)
			searchString = "";
		
		//All the results we've found in mdws so far have been in uppercase.
		searchString = searchString.toUpperCase();
		
		MhpUser mhpUser = MhpUserUtil.getMhpUser();
		String facilityCode = mhpUser.getVistaLocation();
		
		EmrConnection connection = super.getAuthenticatedConnection(facilityCode);
		TaggedHospitalLocationArray tagged = connection.getPort().getClinics(searchString);
		MdwsResponseValidator.validateMdwsObject(tagged);
		HospitalLocationTranslator translator = new HospitalLocationTranslator();
		Clinics clinics = translator.translateClinicsFromTaggedHospitalLocationArray(tagged, facilityCode);
		return clinics;
	}
}
