package com.agilex.healthcare.mobilehealthplatform.datalayer.medication;

import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedMedicationArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TextTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfigurationHelper;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.MedicationTranslator;
import com.agilex.healthcare.utility.NullChecker;

public class MedicationDataLayerMdws extends MdwsEmrDataLayer implements MedicationDataLayer {

	public MedicationDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
        super(mdwsConfiguration);
	}

	/**
	 * Calls the MDWS service EmrSvc.getAllMeds for the given patient
	 */
	@Override
	public Medications getMedications(PatientIdentifier patientId, MedicationFilter medicationFilter) {
		Medications medications = getMedications(patientId, false);
		Medications filteredMedications = MedicationFilterer.filter(medications, medicationFilter);

		return filteredMedications;
	}

	/**
	 * Calls the MDWS service EmrSvc.getAllMeds for the given patient
	 */
	@Override
	public Medications getMedications(PatientIdentifier patientId, boolean activeOnly) {

		EmrConnection authenticatedSite = super.getAuthenticatedConnectionWithPatientSelect(patientId);
		
		TaggedMedicationArrays mdwsTaggedMedicationArrays = authenticatedSite.getPort().getAllMeds();
		super.disconnectFromCdwIfApplicable(authenticatedSite);

		MdwsResponseValidator.validateMdwsObject(mdwsTaggedMedicationArrays);
		
		MedicationTranslator translator = new MedicationTranslator();
		Medications medications = translator.translateFromTaggedMedicationArrays(mdwsTaggedMedicationArrays);

		if (activeOnly)
			medications = this.filterActiveMedications(medications);

		return medications;
	}

	/**
	 * Calls the MDWS service EmrSvc.getAllMeds for the given patient
	 */
	@Override
	public Medication getMedicationLite(PatientIdentifier patientId, String medicationId) {
		Medications medications = this.getMedications(patientId, false);
		Medication medication = this.getSingleMedication(medications, medicationId);

		return medication;
	}

	/**
	 * Calls the MDWS service EmrSvc.getMedicationDetail for the given patient
	 */
	@Override
	public Medication getMedicationDetail(PatientIdentifier patientId, String medicationId) {
		if (NullChecker.isNullish(medicationId))
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "medicationId cannot be null or empty");
		
		String siteCode = new MdwsConfigurationHelper().extractSiteCodeFromAssigningAuthority(patientId.getAssigningAuthority());
		Medications medications = this.getMedications(patientId, false);
		if (medications == null || medications.size() == 0)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "Zero medications were returned so we can't find an id in that.");
		
		Medication medication = this.getSingleMedication(medications, medicationId);
		if (medication == null) {
			String error = formatError(medicationId, medications);
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, error);
		}
		
		if (medication.isNonVA() == false) {
			String vaMedicationId = medication.getVaMedicationId();
			TextTO medicationDetail = new TextTO();
			if (NullChecker.isNotNullish(vaMedicationId)) {
				EmrConnection authenticatedSite = super.getAuthenticatedConnectionWithPatientSelect(patientId);
				medicationDetail = authenticatedSite.getPort().getMedicationDetail(siteCode, vaMedicationId);
				super.disconnectFromCdwIfApplicable(authenticatedSite);
				MdwsResponseValidator.validateMdwsObject(medicationDetail);
			}
			medication.setMedicationDetail(medicationDetail.getText());
		}

		return medication;
	}

	private String formatError(String medicationId, Medications medications) {
		String error = "medicationId ('" + medicationId + "') was unable to be found in this: [";
		Iterator<Medication> it = medications.iterator();
		boolean firstPass = true;
		while (it.hasNext()) {
			Medication med = it.next();
			if (!firstPass)
				error += ", ";
			error += "'" + med.getId() + "'";
			firstPass = false;
		}
		error += "]";
		return error;
	}
	
	public Medication getSingleMedication(PatientIdentifier patientIdentifier, String medicationId) {
		Medications medications = this.getMedications(patientIdentifier, false);
		Medication medication = this.getSingleMedication(medications, medicationId);

		return medication;
	}

	private Medication getSingleMedication(Medications medications, String medicationId) {
		for (Medication medication : medications) {
			if (NullChecker.isNotNullish(medicationId)) {
				if (medicationId.equalsIgnoreCase(medication.getId()) || medicationId.equalsIgnoreCase(medication.getOrderNumber())) {
					return medication;
				}
			}
		}

		return null;
	}

	Medications filterActiveMedications(Medications medications) {
		Medications activeMedications = new Medications();
		for (Medication medication : medications) {
			if (medication.isActive()) {
				activeMedications.add(medication);
			}
		}

		return activeMedications;
	}

}
