package com.agilex.healthcare.mobilehealthplatform.datalayer.patient;

import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientDemographics;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.PatientTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedPatientArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedPatientArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.DemographicTranslator;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.PatientTranslator;
import com.agilex.healthcare.utility.NullChecker;
import com.agilex.healthcare.utility.PatientUtility;

public class PatientDataLayerMdws extends MdwsEmrDataLayer implements PatientDataLayer{

	private final static org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientDataLayerMdws.class);

	public PatientDataLayerMdws(MdwsConfiguration configuration) {
		super(configuration);
	}



	/**
	 * Calls one of the following MDWS services<br/>
	 * EmrSvc.match<br/>
	 * EmrSvc.getPatientsByWard<br/>
	 * EmrSvc.getPatientsByClinic<br/>
	 */
	@Override
	public Patients patientSearch(PatientSearchCriteria criteria) {
		String searchString = criteria.determineSearchString();
		logger.debug("Searching patients with criteria: ");

		String vistaSiteCode = criteria.getVistaSiteCode();
		logger.debug("Searching patients with Vista SiteCode: ");


		if (NullChecker.isNullish(searchString) && NullChecker.isNullish(criteria.getWard()) && NullChecker.isNullish(criteria.getClinic())) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "Insufficient search criteria");			
		}
		if (NullChecker.isNullish(vistaSiteCode)) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_INPUT, "Vista Site Code is unavailabe to perform search against.");
		}

		PatientTranslator translator = new PatientTranslator();
		EmrConnection connection = super.getAuthenticatedConnection(vistaSiteCode);
		Patients translatedPatients = null;
		
		if (NullChecker.isNotNullish(criteria.getWard()) || NullChecker.isNotNullish(criteria.getClinic())) {
			if (NullChecker.isNotNullish(criteria.getWard())) {
				translatedPatients = getPatientsByWard(connection, criteria.getWard(), translator);
			} else {
				translatedPatients = getPatientsByClinic(connection, criteria.getClinic(), translator);
			}
			translatedPatients = PatientUtility.filter(translatedPatients, criteria);
		} else {
			translatedPatients = getPatientsFromMatch(connection, searchString, translator);
		}
		
		if(translatedPatients == null) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_PATIENT, "Translated Patients is null.");
		}

		Patients filteredPatients = applyPostNameFiltering(criteria, translatedPatients);
		return filteredPatients;

	}

	private Patients getPatientsFromMatch(EmrConnection connection, String searchString, PatientTranslator translator){
		TaggedPatientArrays taggedPatientArrays = connection.getPort().match(searchString);

		MdwsResponseValidator.validateMdwsObject(taggedPatientArrays);
		Patients translatedPatients =  translator.translateFromTaggedPatientArrays(taggedPatientArrays);
		return translatedPatients;
	}

	private Patients getPatientsByWard(EmrConnection connection, String wardId, PatientTranslator translator){
		TaggedPatientArray taggedPatientArray = connection.getPort().getPatientsByWard(wardId);

		MdwsResponseValidator.validateMdwsObject(taggedPatientArray);
		Patients translatedPatients =  translator.translateFromTaggedPatientArray(taggedPatientArray);
		return translatedPatients;
	}

	private Patients getPatientsByClinic(EmrConnection connection, String clinicId, PatientTranslator translator){
		TaggedPatientArray taggedPatientArray = connection.getPort().getPatientsByClinic(clinicId);

		MdwsResponseValidator.validateMdwsObject(taggedPatientArray);
		Patients translatedPatients =  translator.translateFromTaggedPatientArray(taggedPatientArray);
		return translatedPatients;
	}

	//TODO: We are trying to make the search work differently than CPRS
	//1. We do a special filter of the 44 patients that were returned from MDWS.
	//2. We change their data so it's formatted the way we want it rather than how they gave it to us.
	//3. We sort those results.
	//4. We remove any results past our max results (which is currently 40).
	//This bug in JIRA has a document which goes into major detail about how search works in CPRS and how we are
	//drastically changing the desired behavior of said search as well as what to do to fix it.
	//https://jira.agilex.com/browse/SRCH-261
	//SRCH-261 - Patient Search for BCMA, ONE does not make sense
	public static Patients applyPostNameFiltering(PatientSearchCriteria criteria, Patients translatedPatients) {
		boolean discardPatient;

		String lastName = "";
		String firstName = "";
		String middleName = "";

		//determine first and last name search criteria
		if (criteria.getPrototype() != null){
			Patient prototype = criteria.getPrototype();

			if (NullChecker.isNotNullish(prototype.getLastName())){
				lastName = criteria.getPrototype().getLastName().toLowerCase();
			}
			if (NullChecker.isNotNullish(prototype.getFirstName())){
				firstName = criteria.getPrototype().getFirstName().toLowerCase();
			}
			if (NullChecker.isNotNullish(prototype.getMiddleName())){
				middleName = criteria.getPrototype().getMiddleName().toLowerCase();
			}
		}

		//should we apply the filters?
		boolean applyLastNameFilter = NullChecker.isNotNullish(lastName);
		boolean applyFirstNameFilter = NullChecker.isNotNullish(firstName);
		boolean applyMiddleNameFilter = NullChecker.isNotNullish(middleName);

		//apply filters if necessary
		Patients filteredPatients = new Patients();
		for (Patient translatedPatient : translatedPatients) {
			discardPatient = false;  //assume patient is good

			if (applyLastNameFilter && !translatedPatient.getLastName().toLowerCase().contains(lastName)){
				discardPatient = true;
			}
			if (applyFirstNameFilter && !translatedPatient.getFirstName().toLowerCase().contains(firstName)){
				discardPatient = true;
			}
			if (applyMiddleNameFilter && translatedPatient.getMiddleName() != null && !translatedPatient.getMiddleName().toLowerCase().contains(middleName)){
				discardPatient = true;
			}

			if (!discardPatient){
				filteredPatients.add(translatedPatient);
			}
		}
		return filteredPatients;
	}



	/**
	 * Calls the MDWS service EmrSvc.getDemographics for the given patient
	 */
	@Override
	public PatientDemographics getDemographics(PatientIdentifier patientIdentifier) {

		EmrConnection authenticatedSite = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

		PatientTO mdwsPatient = authenticatedSite.getPort().getDemographics();

		if (mdwsPatient.getFault() != null) {
			String errorMsg = mdwsPatient.getFault().getMessage();
			logger.error(errorMsg);
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_PATIENT, errorMsg);
		}
		DemographicTranslator translator = new DemographicTranslator();
		PatientDemographics patient = translator.translate(mdwsPatient.getDemographics());
		return patient;
	}

	/**
	 * Calls the MDWS service EmrSvc.select for the given patient
	 */
	@Override
	public Patient getPatient(PatientIdentifier patientIdentifier) {

		PatientTranslator translator = new PatientTranslator();
		EmrConnection authenticatedSite = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);

		PatientTO mdwsPatient = authenticatedSite.getPort().select(patientIdentifier.getUniqueId());

		if (mdwsPatient.getFault() != null) {
			String errorMsg = mdwsPatient.getFault().getMessage();
			logger.error(errorMsg);
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_PATIENT, errorMsg);
		}

		return translator.translateFromPatientTO(mdwsPatient);
	}
}
