package com.agilex.healthcare.mobilehealthplatform.datalayer.radiology;

import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResult;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyResults;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsEmrDataLayer;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseValidator;
import com.agilex.healthcare.mobilehealthplatform.mdws.connection.EmrConnection;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedRadiologyReportArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfiguration;
import com.agilex.healthcare.mobilehealthplatform.mdws.soapconsumer.MdwsConfigurationHelper;
import com.agilex.healthcare.mobilehealthplatform.mdws.translator.RadiologyTranslator;
import com.agilex.healthcare.utility.NullSafeStringComparer;

public class RadiologyDataLayerMdws extends MdwsEmrDataLayer implements RadiologyDataLayer {
    private static final int nrpts = 1000;

	public RadiologyDataLayerMdws(MdwsConfiguration mdwsConfiguration) {
		super(mdwsConfiguration);
	}
	
	/**
	 * Calls the MDWS service EmrSvc.getRadiologyReports for the given patient with an nrpts of 1000
	 */
	@Override
	public RadiologyResults getResults(PatientIdentifier patientIdentifier, DateFilter dateFilter) {
		
		//String siteCode = new MdwsConfigurationHelper().extractSiteCodeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		String siteType  =  new MdwsConfigurationHelper().extractSiteTypeFromAssigningAuthority(patientIdentifier.getAssigningAuthority());
		
		EmrConnection authenticatedConnection = super.getAuthenticatedConnectionWithPatientSelect(patientIdentifier);		
		
		String startDate = MdwsDateTranslationUtil.startDateBySystem(dateFilter.getStartDate(), siteType);
		String endDate = MdwsDateTranslationUtil.endDateBySystem(dateFilter.getEndDate(), siteType);
		
		TaggedRadiologyReportArrays taggedRadiologyReports = authenticatedConnection.getPort().getRadiologyReports(startDate, endDate, nrpts);
		MdwsResponseValidator.validateMdwsObject(taggedRadiologyReports);
		
		RadiologyResults results = translate(taggedRadiologyReports);
		return filter(results, dateFilter);
	}
	
	/**
	 * Calls the MDWS service EmrSvc.getAppointments for the given patient with an nrpts of 1000
	 */
	@Override
	public RadiologyResult fetchResult(PatientIdentifier patientIdentifier, String resultId) {
		RadiologyResults results = getResults(patientIdentifier, DateFilterFactory.createEmptyFilter());
		
		if(results == null)
			return null;
		
		return extract(results, resultId);
	}
	
	private RadiologyResult extract(RadiologyResults results, String resultId) {
		
		RadiologyResult found = null;
		for(RadiologyResult result : results) {
			if(NullSafeStringComparer.areEqualIgnoreCase(result.getUniqueId(), resultId)) {
				found = result;
			}
		}
		
		return found;
	}

	private RadiologyResults translate(TaggedRadiologyReportArrays taggedRadiologyReportArrays) {
		RadiologyTranslator translator = new RadiologyTranslator();
		return translator.translate(taggedRadiologyReportArrays);
	}
	
	private RadiologyResults filter(RadiologyResults results, DateFilter filter) {
		return DateFilterer.filterByDate(results, filter);
	}
}
