package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.*;

import com.agilex.healthcare.mobilehealthplatform.domain.*;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.*;
import com.agilex.healthcare.utility.*;

public class ChemistryLabTranslator {

	public LabResults translate(TaggedChemHemRptArrays arr) {
		LabResults retValue = new LabResults();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedChemHemRptArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedChemHemRptArray() == null)
			return retValue;
		
		Iterator<TaggedChemHemRptArray> itTagged = arr.getArrays().getTaggedChemHemRptArray().iterator();
		while (itTagged.hasNext()) {
			TaggedChemHemRptArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());
			
			if (tagged.getRpts() == null || tagged.getRpts().getChemHemRpt() == null)
				continue;
			
			Iterator<ChemHemRpt> it = tagged.getRpts().getChemHemRpt().iterator();
			while (it.hasNext()) {
				ChemHemRpt chemHemReport = it.next();
				//Perform Error Checking
				if (chemHemReport.getFault() != null)
					throw new MdwsResponseException(chemHemReport.getFault());
				if (chemHemReport.getResults() == null || chemHemReport.getResults().getLabResultTO() == null)
					continue;
				
				Iterator<LabResultTO> itLabResultTo = chemHemReport.getResults().getLabResultTO().iterator();
				while (itLabResultTo.hasNext()) {
					LabResultTO labResultTo = itLabResultTo.next();
					//Perform Error Checking
					if (labResultTo.getFault() != null)
						throw new MdwsResponseException(labResultTo.getFault());
					
					LabResult labResult = createLabResult(chemHemReport, labResultTo);
					retValue.add(labResult);
				}
			}
		}
		
		return retValue;
	}

	private LabResult createLabResult(ChemHemRpt chemHemReport, LabResultTO labResultTo) {
		LabResult labResult = new LabResult();
		String value = labResultTo.getValue();
		String boundaryStatus = labResultTo.getBoundaryStatus();
		TriState determineTriStateValue = determineTriStateValue(boundaryStatus);
		String chemHemReportTimeStamp = chemHemReport.getTimestamp();
		Date resultedDate = MdwsDateTranslationUtil.extractDateOrNull(chemHemReportTimeStamp);
		Author authorData = setAuthorData(labResult, chemHemReport.getAuthor());
		
		setTestData(labResult, labResultTo);
		setSpeciminData(labResult, chemHemReport.getSpecimen());
		labResult.setValue(value);
		labResult.setIndicator(boundaryStatus);
		labResult.setAbnormalIndicatorTriFlag(determineTriStateValue);
		labResult.setResultedDate(resultedDate);
		labResult.setAuthor(authorData);
		return labResult;
	}
	
	void setTestData(LabResult labResult, LabResultTO labResultTo) {
		if (labResultTo == null)
			return;
		//Perform Error Checking
		if (labResultTo.getFault() != null)
			throw new MdwsResponseException(labResultTo.getFault());
		
		LabTestTO test = labResultTo.getTest();
		
		if (test == null)
			return;
		//Perform Error Checking
		if (test.getFault() != null)
			throw new MdwsResponseException(test.getFault());
		
		
		labResult.setUniqueId(test.getId());
		labResult.setTestId(test.getId());
		labResult.setTestname(test.getName());
		labResult.setDisplayName(test.getName());
		labResult.setDisplayDescription(test.getName());
		labResult.setValueUnits(test.getUnits());
		labResult.setLoinc(test.getLoinc());
		
		parseAndSetReferenceRange(labResult, test);
	}
	
	private void parseAndSetReferenceRange(LabResult labResult, LabTestTO mdwsLabTest) {
		String referenceRange = mdwsLabTest.getRefRange();
		if (NullChecker.isNotNullish(referenceRange)) {
			String[] lowAndHigh = referenceRange.split("-");
			if (lowAndHigh != null && lowAndHigh.length == 2) {
				labResult.setReferenceLow(lowAndHigh[0].trim());
				labResult.setReferenceHigh(lowAndHigh[1].trim());
			}
		} else {
			labResult.setReferenceLow(mdwsLabTest.getLowRef());
			labResult.setReferenceHigh(mdwsLabTest.getHiRef());
		}
	}

	void setSpeciminData(LabResult labResult, LabSpecimenTO specimen) {
		if (specimen == null)
			return;
		//Perform Error Checking
		if (specimen.getFault() != null)
			throw new MdwsResponseException(specimen.getFault());
		
		String specimenCollectionDateString = specimen.getCollectionDate();
		Date specimenCollectionDate = MdwsDateTranslationUtil.extractDateOrNull(specimenCollectionDateString);
		
		labResult.setSpecimenId(specimen.getId());
		labResult.setSpecimenName(specimen.getName());
		labResult.setAccessionNumber(specimen.getAccessionNum());
		labResult.setSpecimenCollectionDate(specimenCollectionDate);
	}

	private TriState determineTriStateValue(String boundaryStatus) {
		if (NullChecker.isNullish(boundaryStatus))
			return TriState.FALSE;
		else if (boundaryStatus.contains("H") || boundaryStatus.contains("L"))
			return TriState.TRUE;
		else
			return TriState.UNKNOWN;
	}
	
	private Author setAuthorData(LabResult labResult, AuthorTO authorTo){
		
		Author author = new Author();
		if(authorTo!=null){
			//Perform Error Checking
			if (authorTo.getFault() != null)
				throw new MdwsResponseException(authorTo.getFault());
			author.setId(authorTo.getId());
			author.setName(authorTo.getName());
			author.setSignature(authorTo.getSignature());
		}
		return author;
	}
}
