package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import org.apache.log4j.Logger;

import com.agilex.healthcare.mobilehealthplatform.datalayer.medication.MedicationFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.Medication;
import com.agilex.healthcare.mobilehealthplatform.domain.Medications;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.MedicationTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedMedicationArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedMedicationArrays;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedText;
import com.agilex.healthcare.utility.Hasher;
import com.agilex.healthcare.utility.NullChecker;

public class MedicationTranslator {
	private static final Logger LOGGER = Logger.getLogger(MedicationTranslator.class);
	
	public Medications translateFromTaggedMedicationArrays(TaggedMedicationArrays arr) {
		Medications retValue = new Medications();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedMedicationArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedMedicationArray() == null)
			return retValue;

		Iterator<TaggedMedicationArray> itTagged = arr.getArrays().getTaggedMedicationArray().iterator();
		while (itTagged.hasNext()) {
			TaggedMedicationArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getMeds() == null || tagged.getMeds().getMedicationTO() == null)
				continue;

			Iterator<MedicationTO> it = tagged.getMeds().getMedicationTO().iterator();
			while (it.hasNext()) {
				MedicationTO mdwsMedication = it.next();
				//Perform Error Checking
				if (mdwsMedication.getFault() != null)
					throw new MdwsResponseException(mdwsMedication.getFault());

				Medication medication = createMedication(mdwsMedication);
				retValue.add(medication);
			}
		}

		return retValue;
	}
	
	public Medication createMedication(MedicationTO mdwsMedication) {
		assert mdwsMedication != null;
		
		Medication medication = new Medication();

		setId(mdwsMedication, medication);
		String orderId = mdwsMedication.getOrderId();
		String name = mdwsMedication.getName();
		String rxNum = mdwsMedication.getRxNum();
		String detail = mdwsMedication.getDetail();
		String status = mdwsMedication.getStatus();
		String quantity = mdwsMedication.getQuantity();
		String sig = mdwsMedication.getSig();
		String refills = mdwsMedication.getRefills();
		String daysSupply = mdwsMedication.getDaysSupply();
		String type = mdwsMedication.getType();
		
		Integer totalRefills = parseIntegerOrNull(refills);
		String remaining = mdwsMedication.getRemaining();
		Integer refillsRemaining = parseIntegerOrNull(remaining);
		
		String lastFillDate = mdwsMedication.getLastFillDate();
		Date dtLastFillDate = MdwsDateTranslationUtil.extractDateOrNull(lastFillDate);
		String issueDate = mdwsMedication.getIssueDate();
		Date dtIssueDate = MdwsDateTranslationUtil.extractDateOrNull(issueDate);
		String stopDate = mdwsMedication.getStopDate();
		Date dtStopDate = MdwsDateTranslationUtil.extractDateOrNull(stopDate);
		String expirationDate = mdwsMedication.getExpirationDate();
		Date dtExpirationDate = MdwsDateTranslationUtil.extractDateOrNull(expirationDate);
		
		String comment = mdwsMedication.getComment();
		String documentedDate = mdwsMedication.getStartDate();
		Date dtDocumentedDate = MdwsDateTranslationUtil.extractDateOrNull(documentedDate);
		String startDate = mdwsMedication.getStartDate();
		Date dtStartDate = MdwsDateTranslationUtil.extractDateOrNull(startDate);
		boolean inPatient = mdwsMedication.isIsInpatient();
		boolean nonVA = mdwsMedication.isIsNonVA();
		
		medication.setOrderNumber(orderId);
		medication.setDrugName(name);
		medication.setPrescriptionId(rxNum);
		
		//TODO: This is here to keep compatibility with the front end.  They need to fix their end to check for comment.
		if (detail != null)
			medication.setMedicationDetail(detail);
		else
			medication.setMedicationDetail(comment);
		
		medication.setStatus(status);
		medication.setQuantity(quantity);
		medication.setSig(sig);
		medication.setRefills(refills);
		medication.setDaysSupply(daysSupply);
		medication.setCategory(type);
		
		medication.setTotalRefills(totalRefills);
		medication.setRefillsRemaining(refillsRemaining);
		
		medication.setLastFilledDate(dtLastFillDate);
		
		//TODO: This is here to keep compatibility with the front end that was using issue date as the start date
		//rather than having a field for issue date and a different one for start date.  They need to fix their end
		//and then this can be fixed.
		if (dtIssueDate != null)
			medication.setStartDate(dtIssueDate);
		else
			medication.setStartDate(dtStartDate);
		
		medication.setEndDate(dtStopDate);
		medication.setExpiredDate(dtExpirationDate);
		
		if (mdwsMedication.isIsInpatient()) {
			medication.setMedicationSource(MedicationFilter.INPATIENT);
		} else {
			medication.setMedicationSource(MedicationFilter.OUTPATIENT);
		}

		setSourceSystem(medication, mdwsMedication);
		
		medication.setComment(comment);
		medication.setDocumentedDate(dtDocumentedDate);
		medication.setIssueDate(dtIssueDate);
		medication.setInPatient(inPatient);
		medication.setNonVA(nonVA);
		
		return medication;
	}
	
	private void setSourceSystem(Medication medication, MedicationTO mdwsMedication) {
		TaggedText facility = mdwsMedication.getFacility();
		if(facility == null)
			return;
		//Perform Error Checking
		if (facility.getFault() != null)
			throw new MdwsResponseException(facility.getFault());

		medication.setSourceSystem(facility.getText());
	}

	private void setId(MedicationTO medicationTo, Medication medication) {
		String medicationId = medicationTo.getId();
		if(NullChecker.isNullish(medicationId))
			medicationId = medicationTo.getOrderId();
		if(NullChecker.isNullish(medicationId))
			medicationId = null;
		
		String uniqueId = null;
		if (medicationId != null)
			uniqueId = getVaMedicationId(medicationId);
		else
			uniqueId = getNonVaMedicationId(medicationTo);
		
		medication.setId(uniqueId);
		medication.setUniqueId(uniqueId);
		medication.setVaMedicationId(medicationId);
	}
	
	private String getVaMedicationId(String medicationId) {
		if (medicationId == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "medicationId cannot be null for a VA med");
		try {
			Hasher hasher = new Hasher();
			String retvalue = hasher.hashStringToString(medicationId);
			return retvalue;
		}
		catch (Exception e) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, e.getMessage());
		}
	}
	
	private String getNonVaMedicationId(MedicationTO medicationTo) {
		try {
			String uniqueId = "" + medicationTo.getDateDocumented() +
					":" + medicationTo.getName() +
					":" + (medicationTo.getFacility() == null ? "" : medicationTo.getFacility().getTag()) +
					":" + medicationTo.getSig() +
					":" + medicationTo.getStartDate() +
					":" + medicationTo.getStatus();
			Hasher hasher = new Hasher();
			String retvalue = hasher.hashStringToString(uniqueId);
			return retvalue;
		}
		catch (Exception e) {
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, e.getMessage());
		}
	}
	
	private static Integer parseIntegerOrNull(String value) {
		Integer num = null;

		if (NullChecker.isNotNullish(value)) {
			try {
				num = Integer.parseInt(value);
			} catch (NumberFormatException e) {
				LOGGER.error("Error parsing remaining refills, setting to null", e);
			}
		}
		
		return num;
	}
}
