package com.agilex.healthcare.mobilehealthplatform.mdws.translator;

import java.util.Date;
import java.util.Iterator;

import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResults;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsDateTranslationUtil;
import com.agilex.healthcare.mobilehealthplatform.mdws.MdwsResponseException;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.LabSpecimenTO;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.MicrobiologyRpt;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedMicrobiologyRptArray;
import com.agilex.healthcare.mobilehealthplatform.mdws.generatedwsdl.emrservice.TaggedMicrobiologyRptArrays;
import com.agilex.healthcare.utility.Hasher;
import com.agilex.healthcare.utility.NullChecker;

public class MicrobiologyLabTranslator {

	public LabResults translate(TaggedMicrobiologyRptArrays arr) {
		LabResults retValue = new LabResults();
		if (arr == null)
			throw new MdwsResponseException(MdwsResponseException.UserMessages.INVALID_DATA_RETURNED, "TaggedMicrobiologyRptArrays was null");
		if (arr.getFault() != null)
			throw new MdwsResponseException(arr.getFault());
		if (arr.getArrays() == null || arr.getArrays().getTaggedMicrobiologyRptArray() == null)
			return retValue;

		Iterator<TaggedMicrobiologyRptArray> itTagged = arr.getArrays().getTaggedMicrobiologyRptArray().iterator();
		while (itTagged.hasNext()) {
			TaggedMicrobiologyRptArray tagged = itTagged.next();
			//Perform Error Checking
			if (tagged.getFault() != null)
				throw new MdwsResponseException(tagged.getFault());

			if (tagged.getRpts() == null || tagged.getRpts().getMicrobiologyRpt() == null)
				continue;

			Iterator<MicrobiologyRpt> it = tagged.getRpts().getMicrobiologyRpt().iterator();
			while (it.hasNext()) {
				MicrobiologyRpt mdwsMicrobiologyRpt = it.next();
				//Perform Error Checking
				if (mdwsMicrobiologyRpt.getFault() != null)
					throw new MdwsResponseException(mdwsMicrobiologyRpt.getFault());

				LabResult labResult = createLabResult(mdwsMicrobiologyRpt);
				retValue.add(labResult);
			}
		}

		return retValue;
	}

	private LabResult createLabResult(MicrobiologyRpt microbiologyReport) {
		LabResult labResult = new LabResult();
		String timestamp = microbiologyReport.getTimestamp();
		Date resultedDate = MdwsDateTranslationUtil.extractDateOrNull(timestamp);
		String title = microbiologyReport.getTitle();
		String sample = microbiologyReport.getSample();
		String text = microbiologyReport.getText();
		
		// Handle scenarios where microbiology reports don't have identifiers
		String id = microbiologyReport.getId();
		if (NullChecker.isNullish(id))
			id = createRepeatableUniqueId(microbiologyReport);
		labResult.setUniqueId(id);

		labResult.setTestname(title);
		labResult.setLabType(sample);
		labResult.setDisplayName(title);
		labResult.setDisplayDescription(title);
		labResult.setValue(text);

		setSpeciminData(labResult, microbiologyReport);

		labResult.setComment(microbiologyReport.getComment());
		labResult.setResultedDate(resultedDate);

		return labResult;
	}

	private String createRepeatableUniqueId(MicrobiologyRpt rpt) {
		//We want to create something unique yet at the same time we need to be able to refer back to it in case they
		//ask for a single Vital.  So, we will create a unique id by taking all of the possible data that we know about
		//to create a String that contains everything and creating a hash from that String.
		
		String uniqueId = "" +
				rpt.getComment() +
				":" + rpt.getId() +
				":" + rpt.getSample() +
				":" + rpt.getTimestamp() +
				":" + rpt.getTitle() +
				":" + (rpt.getFacility() == null ? "" : rpt.getFacility().getTag()) +
				":" + (rpt.getFacility() == null ? "" : rpt.getFacility().getText()) +
				":" + (rpt.getAuthor() == null ? "" : rpt.getAuthor().getId()) +
				":" + rpt.getText();
		Hasher hasher = new Hasher();
		String retvalue = hasher.hashStringToString(uniqueId);
		return retvalue;
	}

	private void setSpeciminData(LabResult labResult, MicrobiologyRpt microbiologyReport) {
		LabSpecimenTO specimen = microbiologyReport.getSpecimen();
		if (specimen == null)
			return;
		//Perform Error Checking
		if (specimen.getFault() != null)
			throw new MdwsResponseException(specimen.getFault());
		
		String id = specimen.getId();
		String name = specimen.getName();
		String accessionNum = specimen.getAccessionNum();
		
		labResult.setSpecimenId(id);
		labResult.setSpecimenName(name);
		labResult.setAccessionNumber(accessionNum);
		labResult.setOrderId(accessionNum);
	}
}
