package com.agilex.healthcare.facilitydefinition.restservice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicDataService;
import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicFilter;
import com.agilex.healthcare.facilitydefinition.datalayer.clinic.ClinicFilterFactory;
import com.agilex.healthcare.facilitydefinition.datalayer.specialty.SpecialtyDataService;
import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardDataService;
import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardFilter;
import com.agilex.healthcare.facilitydefinition.datalayer.ward.WardFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.Clinics;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialties;
import com.agilex.healthcare.mobilehealthplatform.domain.Specialty;
import com.agilex.healthcare.mobilehealthplatform.domain.Wards;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndexFactory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.LocationServiceResourceDirectoryBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ClinicLinkBuilder;

/**
 * REST Services for working with Location
 */
@Path("/location-service/")
@Component
@Scope("request")
public class LocationServiceResource {

	@Resource
	LocationServiceResourceDirectoryBuilder locationServiceResourceDirectoryBuilder;
	
	/**
	 * Retrieves the ResourceDirectory for working with Location
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @return JSON containing the ResourceDirectory
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/resource-directory")
	public ResourceDirectory fetchResourceDirectory(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		ResourceDirectory resourceDirectory = locationServiceResourceDirectoryBuilder.getResourceDirectory(uriInfo.getBaseUri(), facilityCode);
		return resourceDirectory;
	}
	
	/**
	 * Returns the Wards starting at searchIndex using MDWS services and returning data following the same behavior as CPRS.<br/>
	 * Calls the MDWS service EmrSvc.getWards<br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>
	 * 			The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * 			When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * 			from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * 			a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * 			you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * 			anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * 			If searchIndex.getSearchString() is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * 			When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * 			exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * 			last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * 			telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * 			<font color="red"><b>Example</b></font><br/>
	 * 			Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * 			what you searched for.<br/><br/>
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.  <font color="red">NOTE: This is no longer used as we can get the information
	 * from the logged on user.</font>
	 * @return JSON containing the Wards (up to 44 at a time) 
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/mdws-wards")
	public Wards fetchMdwsWards(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		WardDataService dataService = new WardDataService();
		return dataService.fetchWards();
	}
	
	/**
	 * Retrieves all of the Wards for the facility that contain (or partially contain) the parameter searchstring.<br/>
	 * Calls the MDWS service EmrSvc.getWards (if configured to use mdws instead of Mongo)<br/><br/>
	 * 
	 * <font color="red">NOTE: This currently accesses the mongo database to retrieve Wards and performs
	 * filtering based on that.  The functionality differs from the way CPRS behaves.  It also means the TableSync
	 * must be run to populate the Wards in the mongo database.  Because you are accessing a real database, you can
	 * perform filtering and other database operations.  You are not limited to the interface or the way that MDWS
	 * returns data (indexed like a phone book).  However, it also means you don't have the most current data.</font>
	 * <br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>the name (or partial name) of the results that you want returned.<br/>
	 * 			If searchstring is not supplied, then all results will be returned.
	 * 		</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>maxrecords</td>
	 * 		<td>the maximum number of records you want returned by the search after filtering.
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @return JSON containing a collection of all of the Wards which match or partially match your searchstring as
	 * contained in the Mongo database.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/wards")
	public Wards fetchWards(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		WardFilterFactory factory = new WardFilterFactory();
		WardFilter filter = factory.createFromUri(uriInfo.getRequestUri());
		
		WardDataService dataService = new WardDataService();
		return dataService.fetchWards(facilityCode, filter);
	}

	
	/**
	 * Returns the Clinics starting at searchIndex using MDWS services and returning data following the same behavior as CPRS.<br/>
	 * Calls the MDWS service EmrSvc.getClinics<br/><br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>
	 * 			The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * 			When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * 			from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * 			a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * 			you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * 			anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * 			If searchIndex.getSearchString() is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * 			When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * 			exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * 			last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * 			telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * 			<font color="red"><b>Example</b></font><br/>
	 * 			Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * 			what you searched for.<br/><br/>
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.  <font color="red">NOTE: This is no longer used as we can get the information
	 * from the logged on user.</font>
	 * @return JSON containing the Clinics (up to 44 at a time)
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/mdws-clinics")
	public Clinics fetchMdwsClinics(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		SearchIndexFactory factory = new SearchIndexFactory();
		SearchIndex searchIndex = factory.createFromUriDecoded(uriInfo.getRequestUri());
		
		ClinicDataService dataService = new ClinicDataService();
		Clinics retvalue = dataService.fetchClinics(facilityCode, searchIndex);
		return retvalue;
	}
	
	/**
	 * Retrieves all of the Clinics for the facility that contain (or partially contain) the parameter searchstring.<br/>
	 * Calls the MDWS service EmrSvc.getClinics (if configured to use mdws instead of Mongo)<br/><br/>
	 * 
	 * <font color="red">NOTE: This currently accesses the mongo database to retrieve Clinics and performs
	 * filtering based on that.  The functionality differs from the way CPRS behaves.  It also means the TableSync
	 * must be run to populate the Clinics in the mongo database.  Because you are accessing a real database, you can
	 * perform filtering and other database operations.  You are not limited to the interface or the way that MDWS
	 * returns data (indexed like a phone book).  However, it also means you don't have the most current data.</font>
	 * <br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>the name (or partial name) of the results that you want returned.<br/>
	 * 			If searchstring is not supplied, then all results will be returned.
	 * 		</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>maxrecords</td>
	 * 		<td>the maximum number of records you want returned by the search after filtering.
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @return JSON containing a collection of all of the Clinics which match or partially match your searchstring as
	 * contained in the Mongo database.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/clinics")
	public Clinics fetchClinics(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		ClinicFilterFactory factory = new ClinicFilterFactory();
		ClinicFilter filter = factory.createFromUri(uriInfo.getRequestUri());
		
		ClinicDataService dataService = new ClinicDataService();
		Clinics retvalue = dataService.fetchClinics(facilityCode, filter);
		return retvalue;
	}

	/**
	 * Retrieves specialties for that Facility.<br/>
	 * Either calls CDW or Mongo (depending on configuration)
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @return JSON containing Specialties for that facility
	 */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("site/{facility-code}/specialties")
    public Specialties fetchSpecialties (@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
        SpecialtyDataService dataService = new SpecialtyDataService();
        Specialties specialties = dataService.getFacilitySpecialties(facilityCode);
        buildClinicsAtomLinksForAllSpecialties(uriInfo,facilityCode,specialties);
        return specialties;
    }
    
    /**
     * Retrieves specialties for that Facility.<br/>
     * Either calls CDW or Mongo (depending on configuration)
     * 
     * @param facilityCode The siteId for the facility.
     * @param system
     * @return JSON containing Specialties for that facility
     */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("site/{facility-code}/specialties/system/{system}")
    public Specialties fetchSpecialtiesBySystem (@PathParam("facility-code") String facilityCode,
    											 @PathParam("system") String system,
     											 @Context UriInfo uriInfo, 
    											 @Context HttpServletRequest request, 
    											 @Context HttpHeaders headers) {
        SpecialtyDataService dataService = new SpecialtyDataService();
        Specialties specialties = dataService.fetchSpecialties(facilityCode, system);
        return specialties;
    }
    
    /**
     * Retrieves a Specialty for that facility that matches the specialtyName<br/>
     * Either calls CDW or Mongo (depending on configuration)
     * 
     * @param facilityCode The siteId for the facility.
     * @param specialtyName
     * @return JSON containing a Specialty for that facility that matches the specialtyName
     */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("site/{facility-code}/specialty")
    public Specialty fetchSpecialty (@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @QueryParam("name") String specialtyName) {
        SpecialtyDataService dataService = new SpecialtyDataService();
        Specialty specialty = dataService.getSpecialty(facilityCode, specialtyName);
        buildClinicAtomLinks(uriInfo, facilityCode,specialty.getSpecialtyClinics());
        return specialty;
    }

    private void buildClinicsAtomLinksForAllSpecialties(UriInfo uriInfo, String facilityCode, Specialties specialties) {
        for(Specialty specialty: specialties){
            buildClinicAtomLinks(uriInfo, facilityCode,specialty.getSpecialtyClinics());
        }
    }

    private void buildClinicAtomLinks(UriInfo uriInfo, String facilityCode, Clinics clinics) {
        if(uriInfo!=null && clinics!=null){
            ClinicLinkBuilder clinicLinkBuilder = new ClinicLinkBuilder(uriInfo.getBaseUri(), facilityCode);
            clinicLinkBuilder.fillLinks(clinics);
        }
    }
}
