package com.agilex.healthcare.mobilehealthplatform.restservice;

import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import agilex.commons.pagination.PaginationDataLayer;

import com.agilex.healthcare.mobilehealthplatform.datalayer.document.DocumentService;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ReportDocuments;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.translator.ReportDocumentMongoTranslator;
import com.mongodb.DBObject;

/**
 * Documents rest service to retrieve discharge summaries, pathology reports, progress notes, radiology and surgery reports. 
 * 
 */
@Component
@Scope
@Path("/jpv/site/{site-id}/documents")
public class JPVDocumentResource {

    public static final String      LIST_NAME = "PatientDocuments";
    public static final String      DATE = "date";
    public static final String      TYPE = "type";

    @Resource(name = "pagination")
    PaginationDataLayer pagination;

    /**
     * Calls the following MDWS services for the given patient<br/>
	 * EmrSvc.getDischargeSummaries<br/>
	 * EmrSvc.getSurgicalPathologyReports<br/>
	 * EmrSvc.getCytologyReports<br/>
	 * EmrSvc.getNotesWithText<br/>
	 * EmrSvc.getRadiologyReports<br/>
	 * EmrSvc.getSurgeryReportsWithText<br/>
	 * 
     * @param siteId -- provider logged in site
     * @param assigningAuthority -- patient assigning authority like dfn
     * @param patientId -- patient unique identifier based on assigning authority
     * @param queryType -- Used for sorting if the value is "date"
     * @param refresh -- Flag to refresh the cache
     * @param maxPerPageStr -- Max page size
     * @param goToPageStr 
     * @param sortOnString
     * @param filter
     * @param documentType
     * @return ReportDocuments
     */
    @GET
    @Produces({ "application/xml", "application/json"})
    @Path("patient/{assigning-authority}/{patient-id}/{query-type}")
    public ReportDocuments getAllReportsForPatient(@PathParam("site-id") String siteId,
                                                   @PathParam("assigning-authority") String assigningAuthority,
                                                   @PathParam("patient-id") String patientId,
                                                   @PathParam("query-type") String queryType,
                                                   @QueryParam("refresh") String refresh,
                                                   @QueryParam("max-per-page") String maxPerPageStr,
                                                   @QueryParam("go-to-page") String goToPageStr,
                                                   @QueryParam("sort-on") String sortOnString,
                                                   @QueryParam("filter") String filter,
                                                   @QueryParam("document-type") String documentType,
                                                   @Context UriInfo       uriInfo,
                                                   @Context HttpHeaders headers){
        List<DBObject>                    returnedDbObjects;
        ReportDocuments                   reportDocuments;
        int                               gotoPage;
        int                               maxPerPage;
        HashMap<String, Boolean>          sortFields = null;
        DocumentService                   dataService;
        DateFilter                        dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);
        PatientIdentifier                 patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
        ReportDocumentMongoTranslator     translator = new ReportDocumentMongoTranslator();
        String                            recordCountFilter;
        StringBuffer                      buf;


        gotoPage = pagination.parseGoToPage(goToPageStr);
        maxPerPage = pagination.parseMaxPerPage(maxPerPageStr);

        if(queryType.equalsIgnoreCase(DATE) || sortOnString==null) {
            sortFields = pagination.parseSortDataMap("timestamp,false");
        }
        else { 	
            sortFields = pagination.parseSortDataMap(sortOnString);
        }

        //
        // Check pagination cache to see if the data needs to be refreshed.
        //
        if(pagination.listDataIsStale(LIST_NAME, assigningAuthority, null, patientId, null)) {
            dataService = new DocumentService();
            reportDocuments = dataService.fetchAllReportDocumentsForPatient(patientIdentifier, dateFilter);
            pagination.saveList(translator.translateReportDocumentListToArrayList(reportDocuments),
                                LIST_NAME, assigningAuthority, null, patientId, null, false);
        }


        if(queryType.equalsIgnoreCase(DATE)) {
            buf = new StringBuffer();
            if(filter != null && filter.trim().length() > 0) {
                buf.append("x,timestamp," + filter + ",o;");
                buf.append("x,resultDate," + filter + ",o;");
                buf.append("x,entryDate," + filter + ",o;");
                buf.append("x,collectionDate," + filter + ",o;");
                buf.append("x,documentType," + filter + ",o;");
                buf.append("x,localTitle," + filter + ",o;");
                buf.append("x,title," + filter + ",o;");
                buf.append("x,testName," + filter + ",o;");
                buf.append("x,author.author_name," + filter + ",o;");
                buf.append("x,text," + filter + ",o;");
                buf.append("x,report," + filter + ",o;");
                buf.append("x,exam," + filter + ",o;");
                buf.append("x,facility," + filter + ",o;");
                buf.append("x,facility.name," + filter + ",o;");
                buf.append("x,facilityName," + filter + ",o;");
                buf.append("x,location," + filter + ",o;");
            }
            returnedDbObjects = pagination.getList(LIST_NAME, assigningAuthority, null, patientId, null, sortFields,
                                                   pagination.getStartRecord(gotoPage, maxPerPage),
                                                   pagination.getStopRecord(gotoPage, maxPerPage),
                                                   (buf.length() > 0 ? buf.toString() : null));
        }
        else {
            buf = new StringBuffer("q,document_type," + documentType + ",a;");
            if(filter != null && filter.trim().length() > 0) {
                buf.append("x,timestamp," + filter + ",o;");
                buf.append("x,resultDate," + filter + ",o;");
                buf.append("x,entryDate," + filter + ",o;");
                buf.append("x,collectionDate," + filter + ",o;");
                buf.append("x,documentType," + filter + ",o;");
                buf.append("x,localTitle," + filter + ",o;");
                buf.append("x,title," + filter + ",o;");
                buf.append("x,testName," + filter + ",o;");
                buf.append("x,author.author_name," + filter + ",o;");
                buf.append("x,text," + filter + ",o;");
                buf.append("x,report," + filter + ",o;");
                buf.append("x,exam," + filter + ",o;");
                buf.append("x,facility," + filter + ",o;");
                buf.append("x,facility.name," + filter + ",o;");
                buf.append("x,facilityName," + filter + ",o;");
                buf.append("x,location," + filter + ",o;");
            }
            returnedDbObjects = pagination.getList(LIST_NAME, assigningAuthority, null, patientId, null, sortFields,
                                                   pagination.getStartRecord(gotoPage, maxPerPage),
                                                   pagination.getStopRecord(gotoPage, maxPerPage), buf.toString());
        }
        reportDocuments = translator.translateArrayListToReportDocumentList(returnedDbObjects);

                //
        // Adjust go to page back to a non zero based number.
        //
        gotoPage += 1;

        if(buf.length() == 0) {
            recordCountFilter = null;
        }
        else {
            recordCountFilter = buf.toString();
        }

        reportDocuments.setRecordCount(pagination.getRecordCount(LIST_NAME, assigningAuthority, null, patientId,
                                                                 null, recordCountFilter));
        int  pgCount;
        if(maxPerPage > 0) {
            pgCount = (int)reportDocuments.getRecordCount()/maxPerPage;
            if((int)reportDocuments.getRecordCount() % maxPerPage != 0) {
                pgCount++;
            }
        }
        else {
            pgCount = 0;
        }

        reportDocuments.setPageCount(pgCount);
        if(gotoPage > reportDocuments.getPageCount()) {
            reportDocuments.setCurrentPage(reportDocuments.getPageCount());
        }
        else {
            reportDocuments.setCurrentPage(gotoPage);
        }
        reportDocuments.setStartRecord((maxPerPage * (gotoPage - 1)) + 1);
        reportDocuments.setStopRecord(maxPerPage * (gotoPage));
        if(reportDocuments.getStopRecord() > reportDocuments.getRecordCount()){
            reportDocuments.setStopRecord(reportDocuments.getRecordCount());
        }


        return reportDocuments;
    }
}
