package com.agilex.healthcare.mobilehealthplatform.restservice;

import java.net.URI;
import java.net.URISyntaxException;

import javax.annotation.Resource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import agilex.commons.helper.PropertyHelper;

import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.LinkTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.atom.rel;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.UriHelper;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ImmunizationLinkBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.PatientLinkBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.SchedulingLinkBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.VeteranAppointmentRequestLinkBuilder;
import com.agilex.healthcare.utility.NullChecker;
import com.google.common.base.Joiner;

/**
 *
 * Rest Service to get the current logged in user resource with applicable atom links
 *
 */
@Path("/mhpuser")
@Component
@Scope("request")
public class MhpUserResource {
	private static final String PORTAL_SERVICE_BASEURI = "portal_service_baseuri";

    @Resource
    private PropertyHelper propertyHelper;

    /**
     * This endpoint returns mhpuser
     *
     * @return {@link MhpUser}
     * @throws URISyntaxException
     */
    @GET
	@Produces({ "application/xml", "application/json" })
	public MhpUser getMhpUserWithRightOfAccessFlag(@Context UriInfo uriInfo) throws URISyntaxException {
		MhpUser mhpUser = MhpUserFactory.createFromSecurityContext();
		MhpUser mergedUser = mergeWithRightOfAccessInfo(mhpUser);

 		setUpLinks(uriInfo, mergedUser);

		return mergedUser;
	}

	private void setUpLinks(UriInfo uriInfo, MhpUser mhpUser) throws URISyntaxException {
        boolean includePatientLinks = !MhpUserFactory.doesUserHasStaffRole();
        boolean includeStaffLinks =  NullChecker.isNotNullish(mhpUser.getVistaLocation());
		mhpUser.setDisplayName(calculateDisplayName(mhpUser.getLastName(), mhpUser.getFirstName()));
        mhpUser.getPatient().setDisplayName(calculateDisplayName(mhpUser.getPatientLastName(), mhpUser.getPatientFirstName()));

        if (includePatientLinks) {
            mhpUser.setPatientUri(createPatientLink(mhpUser, uriInfo));
        }

        URI baseUri = uriInfo.getBaseUri();
        URI mhpUri = new URI(uriInfo.getBaseUri().toString().replace(baseUri.getPath(), "") + "/MobileHealthPlatformWeb/rest/");

		addRelatedLink(mhpUser, LinkTitles.StaffUserDisclaimer, UriHelper.mergeUri(mhpUri, "mhpuser/staff-user-disclaimer"));
        addRelatedLink(mhpUser, LinkTitles.StaffUserDisclaimerContent, UriHelper.mergeUri(mhpUri, "mhpuser/staff-user-disclaimer/content"));
        addRelatedLink(mhpUser, LinkTitles.RightOfAccess, UriHelper.mergeUri(mhpUri, "mhpuser/right-of-access"));
        addRelatedLink(mhpUser, LinkTitles.RightOfAccessPDF, UriHelper.mergeUri(mhpUri, "mhpuser/right-of-access/pdf"));
        addRelatedLink(mhpUser, LinkTitles.RightOfAccessImage, UriHelper.mergeUri(mhpUri, "mhpuser/right-of-access/image"));

        if (includeStaffLinks) {
	        addRelatedLink(mhpUser, LinkTitles.PatientsOperationalScope, UriBuilder.fromUri(baseUri).path("patients").path("site").path(mhpUser.getVistaLocation()).build());
	        addRelatedLink(mhpUser, LinkTitles.LocationServiceResourceDirectory, UriBuilder.fromUri(baseUri).path("location-service").path("site").path(mhpUser.getVistaLocation()).path("resource-directory").build());
            addRelatedLink(mhpUser, LinkTitles.SchedulingResourceDirectory, SchedulingLinkBuilder.getSchedulingManagerServiceBaseUri(baseUri).path("scheduling-service").path("site").path(mhpUser.getVistaLocation()).path("resource-directory").build());
	        addRelatedLink(mhpUser, LinkTitles.PatientSearchResourceDirectory, UriBuilder.fromUri(baseUri).path("patient-search").path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path("resource-directory").build());
	        addRelatedLink(mhpUser, LinkTitles.NoteWriterResourceDirectory, UriBuilder.fromUri(baseUri).path("note-writer").path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path("resource-directory").build());
	        addRelatedLink(mhpUser, LinkTitles.ClinicalEventResourceDirectory, UriBuilder.fromUri(baseUri).path("note-writer").path("clinical-event-service").path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path("resource-directory").build());
	        addRelatedLink(mhpUser, LinkTitles.CurrentProviderConsults, UriBuilder.fromUri(baseUri).path("provider").path(mhpUser.getVistaLocation()).path(mhpUser.getId()).path("consults").path("system").path("cdw").build());
	        addRelatedLink(mhpUser, LinkTitles.ConsultsBySpecialties, UriBuilder.fromUri(baseUri).path("provider").path(mhpUser.getVistaLocation()).path(mhpUser.getId()).path(LinkTitles.ConsultsBySpecialties).path("system").path("cdw").build());
	        addRelatedLink(mhpUser, LinkTitles.MVisumUrl, UriBuilder.fromUri(baseUri).path("mvisum").path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path(LinkTitles.MVisumUrl).build());
	        addRelatedLink(mhpUser, LinkTitles.ImmunizationResourceDirectory, ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(baseUri).path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path("resource-directory").build());
	        addRelatedLink(mhpUser, LinkTitles.ImmunizationPatientResourceDirectory, ImmunizationLinkBuilder.getImmunizationCampaignBaseUri(baseUri).path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getId()).path("patient-resource-directory").build());

	        addRelatedLink(mhpUser, LinkTitles.OrderServices, UriBuilder.fromUri(baseUri).path(LinkTitles.OrderServices).path("site").path(mhpUser.getVistaLocation()).build());
	        addRelatedLink(mhpUser, LinkTitles.OrderStatuses, UriBuilder.fromUri(baseUri).path(LinkTitles.OrderStatuses).path("site").path(mhpUser.getVistaLocation()).build());

	        String providerAssigningAuthority = mhpUser.getVistaLocation();
            String userId = mhpUser.getUserIdentifier().getUniqueId();

            //use configured value if it exists
            String configuredUserContextUrl = propertyHelper.getProperty("userContextUrl");
            String baseUserContextUrl;
            if (NullChecker.isNotNullish(configuredUserContextUrl)){
                baseUserContextUrl = configuredUserContextUrl;
            }
            else{
                baseUserContextUrl = getUserContextBaseUri(uriInfo);
            }
            addRelatedLink(mhpUser, LinkTitles.UserContextLink, UriBuilder.fromUri(baseUserContextUrl).path("context").path("user").path("system").path(providerAssigningAuthority).path("id").path(userId).build());

            String taskResourcesBaseUrl = propertyHelper.getProperty("taskResourcesUrl");
            addRelatedLink(mhpUser, LinkTitles.TaskResourceDirectory, UriBuilder.fromUri(taskResourcesBaseUrl).path("tasks").path("site").path(providerAssigningAuthority).path("staff").path("id").path(providerAssigningAuthority).path(userId).path("resource-directory").build());
            addPortalFavoritesRelatedLink(mhpUser);
        }

        addRelatedLink(mhpUser, LinkTitles.StaffDirectory, UriHelper.mergeUri(baseUri, "staff-directory"));

        if (includePatientLinks) {
            final String PATIENT_STRING = "patient";
            String assigningAuthority = mhpUser.getUserIdentifier().getAssigningAuthority();
            addRelatedLink(mhpUser, LinkTitles.SecureMessageUser, UriBuilder.fromUri(baseUri).path("securemessage-service").path(assigningAuthority).path(mhpUser.getPatientId()).path("secure-message-user").build());
            addRelatedLink(mhpUser, LinkTitles.Appointments, UriBuilder.fromUri(baseUri).path("appointment-service").path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).path("appointments").build());
            addRelatedLink(mhpUser, LinkTitles.AppointmentsVar, VeteranAppointmentRequestLinkBuilder.getVeteranAppointmentRequestServiceBaseUri(baseUri).path("appointment-service").path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).path("appointments").build());
            addRelatedLink(mhpUser, LinkTitles.PatientInternal, UriBuilder.fromUri(baseUri).path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).build());
            addRelatedLink(mhpUser, LinkTitles.PrimaryCareProviders, UriBuilder.fromUri(baseUri).path("provider-service").path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).path("providers").path("primarycare").build());
            addRelatedLink(mhpUser, LinkTitles.MentalHealthProviders, UriBuilder.fromUri(baseUri).path("provider-service").path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).path("providers").path("mentalhealth").build());
            addRelatedLink(mhpUser, LinkTitles.MostRecentFacilities, UriBuilder.fromUri(baseUri).path("facility-service").path(PATIENT_STRING).path(assigningAuthority).path(mhpUser.getPatientId()).path("facilities").build());
        }


    }

    private void addPortalFavoritesRelatedLink(MhpUser mhpUser) {
    	String portalServiceBaseUri = propertyHelper.getProperty(PORTAL_SERVICE_BASEURI);
    	addRelatedLink(mhpUser, LinkTitles.PortalFavorites, UriBuilder.fromUri(portalServiceBaseUri).path("favorites-service").path("site").path(mhpUser.getVistaLocation()).path("id").path(mhpUser.getUserIdentifier().getUniqueId()).path("items").build());
	}

	private String getUserContextBaseUri(UriInfo haUriInfo) {
        return UriBuilder.fromUri(haUriInfo.getRequestUri().getScheme() + "://" + haUriInfo.getRequestUri().getAuthority()).path("UserContext").path("rest").build().toString();
    }

    private URI createPatientLink(MhpUser mhpUser, UriInfo uriInfo) {
		PatientLinkBuilder linkBuilder = new PatientLinkBuilder(uriInfo.getBaseUri());
		return linkBuilder.createPatientSelfUri(mhpUser.getPatient().getPatientIdentifier());
    }

    private void addRelatedLink(MhpUser user, String title, URI href) {
        AtomLink link = new AtomLink();
        link.setHref(href);
        link.setRel(rel.related);
        link.setTitle(title);

        user.getLink().setLinkByTitle(link);
    }

	public MhpUser mergeWithRightOfAccessInfo(MhpUser mhpUser) {
		mhpUser.setRightOfAccessAccepted(true);

		return mhpUser;
	}

    private String calculateDisplayName(String firstName, String lastName) {
        String displayName = getDisplayName(lastName, firstName);
        return (displayName.equals("")) ? "?" : displayName;

    }

    private String getDisplayName(String firstName, String lastName) {
        Joiner joiner = Joiner.on(", ").skipNulls();
        return joiner.join(lastName, firstName);
    }
}
