package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.admission.AdmissionDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.Admission;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNotes;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.AdmissionLinkBuilder;

/**
 * REST Services for working with patient admissions.
 */
@Path("/patient/{assigning-authority}/{patient-id}/admissions")
@Component
@Scope("request")
public class PatientAdmissionsResource {
	// admissions.sortByDate();

	/**
	 * Retrieves {@link Admissions} from the configured data source based upon the scope given {longitudinal, operational}<br/>
	 * Calls the MDWS service EmrSvc.getAdmissions - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param scope {longitudinal, operational}
	 * @return JSON containing the {@link Admissions}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("scope/{scope}")
	public Admissions fetchAdmissionsByScope(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("scope") String scope, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());
        Admissions results = fetchAdmissions(patientIdentifier, dateFilter, ScopeFilter.getInstanceForScope(scope), u);
        return results;

    }

	/**
	 * Retrieves {@link Admissions} by system<br/>
	 * Calls the MDWS service EmrSvc.getAdmissions - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param systemIdentifier The systemId for the system
	 * @return JSON containing the {@link Admissions}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}")
	public Admissions fetchAdmissionsBySystem(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String systemIdentifier, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());
        Admissions results = fetchAdmissions(patientIdentifier, dateFilter, ScopeFilter.getInstanceForSystem(systemIdentifier), u);
        return results;

    }

	/**
	 * Retrieves a single {@link Admission}.<br/>
	 * Calls the MDWS service EmrSvc.getAdmissions - if configured to use mdws (operational) instead of CDW (longitudinal)<br/><br/>
	 * 
	 * This is the self-link of an admission record
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param systemId The systemId for the system
	 * @param id The id for the system
	 * @return JSON containing the {@link Admission}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}/id/{id}")
	public Admission fetchAdmission(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String systemId, @PathParam("id") String id, @Context UriInfo u,
			@Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		final DataIdentifier dataIdentifier = new DataIdentifier(systemId, id);
		AdmissionDataService dataService = new AdmissionDataService();
		Admission admission = dataService.getPatientAdmission(patientIdentifier, dataIdentifier);

		AdmissionLinkBuilder linkbuilder = new AdmissionLinkBuilder(u.getBaseUri());
		linkbuilder.fillLinks(admission, u.getRequestUri());
		
		if (admission == null)
			throw ExceptionFactory.generateNoDataFoundException();
		
        return admission;
    }
	
	/**
	 * This resource returns a Discharge Summary for a patient from VistA.<br/>
	 * Calls the MDWS service EmrSvc.getDischargesReports for the given patient with a hard-coded nrpts of 10 and the given startDate and endDate.<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param systemId The systemId for the system
	 * @param id The id for the system
	 * @return JSON containing the {@link ProviderNote}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}/id/{id}/note")
	public ProviderNote fetchDischargeNote(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String systemId, @PathParam("id") String id, @Context UriInfo uriInfo, @Context HttpHeaders headers) {
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DataIdentifier dataIdentifier = new DataIdentifier(systemId, id);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);

		AdmissionDataService admissionDataService = new AdmissionDataService();
		ProviderNote dischargeNote = admissionDataService.fetchDischargeNoteForInpatientDateRange(patientIdentifier, dataIdentifier, dateFilter);

		return dischargeNote;
	}

    /**
     * This resource returns Discharge Summaries for a patient from VistA.<br/>
     * Calls the MDWS service EmrSvc.getDischargeSummaries for the given patient with a hard-coded nNotes of 1000 and the given startDate and endDate.<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
     * @param siteCode The siteId for the facility (called facilityCode in other services).
     * @param assigningAuthority The assigning authority that gave the patient their id.
     * @param patientId The id of the patient.
     * @return JSON containing the {@link ProviderNotes}
     */
    @GET
    @Produces({ "application/xml", "application/json" })
    @Path("/site/{site-id}/discharge-summaries")
    public ProviderNotes fetchPatientDischargeSummaries(@PathParam("site-id") String siteCode, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @Context UriInfo uriInfo){
        AdmissionDataService admissionDataService = new AdmissionDataService();
        ProviderNotes providerNotes = admissionDataService.getPatientDischargeSummaries(new PatientIdentifier(assigningAuthority, patientId), DateFilterFactory.createFilterFromUri(uriInfo));
        return providerNotes;
    }

	private Admissions fetchAdmissions(final PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter, UriInfo u) {
		AdmissionDataService dataService = new AdmissionDataService();
		Admissions admissions = dataService.getPatientAdmissions(patientIdentifier, dateFilter, scopeFilter);

		admissions.sortByDate();

		AdmissionLinkBuilder linkbuilder = new AdmissionLinkBuilder(u.getBaseUri());
		linkbuilder.fillLinks(admissions, u.getRequestUri());

		return admissions;
	}
}
