package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.allergy.AllergyDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.Allergies;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

/**
 * REST Services for working with patient allergies. 
 */
@Path("/patient/{assigning-authority}/{patient-id}/allergies")
@Component
@Scope("request")
public class PatientAllergiesResource {

	/**
	 * Retrieves patient {@link Allergies} from the configured data source based upon the scope given {longitudinal, operational}<br/>
	 * Calls the MDWS service EmrSvc.getAllergiesBySite for the given patient - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 *  
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param scope {longitudinal, operational}
	 * @return JSON containing the {@link Allergies}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("scope/{scope}")
	public Allergies fetchAllergiesByScope(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("scope") String scope, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		return fetchAllergies(patientIdentifier, ScopeFilter.getInstanceForScope(scope), u);
	}

	/**
	 * Retrieves {@link Allergies} by system<br/>
	 * Calls the MDWS service EmrSvc.getAllergiesBySite for the given patient - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param systemIdentifier The systemId for the system
	 * @return JSON containing the {@link Allergies}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}")
	public Allergies fetchAllergiesBySystem(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String systemIdentifier, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		return fetchAllergies(patientIdentifier, ScopeFilter.getInstanceForSystem(systemIdentifier), u);
	}

	private Allergies fetchAllergies(final PatientIdentifier patientIdentifier, ScopeFilter scopeFilter, UriInfo uriInfo) {
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);
		
		AllergyDataService dataService = new AllergyDataService();
		Allergies allergies = dataService.fetchPatientAllergies(patientIdentifier, dateFilter, scopeFilter);

		return allergies;
	}

}
