package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.appointment.AppointmentDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.DataIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderNote;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameters;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.PatientAppointmentLinkBuilder;
import com.agilex.healthcare.utility.NullChecker;

@Path("/patient/{assigning-authority}/{patient-id}/appointments")
@Component
@Scope("request")
public class PatientAppointmentsResource {

	/**
	 * Retrieves {@link Appointments} from the configured data source based upon the scope given {longitudinal, operational}<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param scope {longitudinal, operational}
	 * @return JSON containing the {@link Appointments}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("scope/{scope}")
	public Appointments fetchAppointmentsByScope(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("scope") String scope, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());
		Appointments results = fetchAppointments(u, patientIdentifier, dateFilter, ScopeFilter.getInstanceForScope(scope));
        return results;

    }

	/**
	 * Retrieves {@link Appointments} for the system provided.<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param system The systemId for the system
	 * @return JSON containing the {@link Appointments}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}")
	public Appointments fetchAppointmentsBySystem(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String system, @Context UriInfo u, @Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());
        Appointments results = fetchAppointments(u, patientIdentifier, dateFilter, ScopeFilter.getInstanceForSystem(system));
        
        return results;
    }
	
	/**
	 * Retrieves the {@link ProviderNote} for the patient for the appointment given.<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param system The systemId for the system
	 * @param visitId The id for the visit
	 * @return JSON containing the {@link ProviderNote}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("system/{system-id}/id/{appointment-id}/visit/{visit-id}/note")
	public ProviderNote fetchClinicNoteForAppointment(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String system, @PathParam("visit-id") String visitId,@Context UriInfo u, @Context HttpHeaders headers) {
		
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DataIdentifier dataIdentifier = new DataIdentifier(system, visitId);
		ScopeFilter scopeFilter = ScopeFilter.getInstanceForSystem(system);
		
		AppointmentDataService dataService = new AppointmentDataService();
		ProviderNote clinicNote = dataService.fetchClinicNotes(patientIdentifier, dataIdentifier, scopeFilter);
		
		return clinicNote;
	}
	
	/**
	 * Retrieves {@link Appointments} where the stop code is equivalent to a Mental Health Appointment from the
	 * configured data source based upon the scope given {longitudinal, operational}.<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param scope {longitudinal, operational}
	 * @return JSON containing the {@link Appointments}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("mental-health/scope/{scope}")
	public Appointments getMentalHealthAppointments(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("scope") String scope, @Context UriInfo uriInfo, @Context HttpHeaders headers) {
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo);
		ScopeFilter scopeFilter = ScopeFilter.getInstanceForScope(scope);
		
		AppointmentDataService dataService = new AppointmentDataService();
		Appointments mentalHealthAppointments = dataService.fetchMentalHealthAppointments(patientIdentifier, dateFilter, scopeFilter);
		
		return mentalHealthAppointments;
	}

	/**
	 * Retrieves the Appointments that this patient has in the next 120 days.<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>referenceDate</td>
	 * 		<td>Filters from this date to 120 days from it.</td>
	 * 	</tr>
	 * </table>
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @return JSON containing the {@link Appointments} 
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("future")
	public Appointments fetchFutureAppointments(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @Context UriInfo u, @Context HttpHeaders headers) {
        
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFutureFilterNext120DaysFromUri(u.getRequestUri());
        Appointments results = fetchAppointments(u, patientIdentifier, dateFilter, ScopeFilter.getInstanceForOperationalScope());
        
        return results;
    }
	
	/**
	 * Retrieves a specific {@link Appointment}. This URI represents the self-link of an Appointment.<br/>
	 * Either calls CDW or HDR (depending on configuration)<br/>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param systemId The systemId for the system
	 * @param id The id for the system
	 * @return JSON containing the {@link Appointment}
	 */
	@GET
	@Path("system/{system-id}/id/{id}")
	@Produces({ "application/xml", "application/json" })
	public Appointment fetchAppointment(@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("system-id") String systemId, @PathParam("id") String id, @Context UriInfo u,
			@Context HttpHeaders headers) {
		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		final DataIdentifier appointmentIdentifier = new DataIdentifier(systemId, id);

		AppointmentDataService dataservice = new AppointmentDataService();
		Appointment appointment = dataservice.getPatientAppointment(patientIdentifier, appointmentIdentifier);

		PatientAppointmentLinkBuilder linkbuilder = new PatientAppointmentLinkBuilder(u.getBaseUri());
		linkbuilder.fillLinks(appointment, u.getRequestUri());

        return appointment;
    }

	private Appointments fetchAppointments(UriInfo u, final PatientIdentifier patientIdentifier, DateFilter dateFilter, ScopeFilter scopeFilter) {
		AppointmentDataService dataservice = new AppointmentDataService();

		if (dateFilter == null)
			dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());

		Appointments appointments = dataservice.getPatientAppointments(patientIdentifier, dateFilter, scopeFilter);
		appointments = sort(appointments, u);

		PatientAppointmentLinkBuilder linkbuilder = new PatientAppointmentLinkBuilder(u.getBaseUri());
		linkbuilder.fillLinks(appointments, u.getRequestUri());

		return appointments;
	}

	// TODO: I would like to move this to the retriever if possible
	private Appointments sort(Appointments appointments, UriInfo u) {
		AppointmentDataService dataservice = new AppointmentDataService();

		QueryParameters queryParams = new QueryParameters(u.getRequestUri().getQuery());
		String sortRequest = queryParams.getValue("sort");
		if (NullChecker.isNullish(sortRequest)) {
			sortRequest = "date.desc";
		}

		if (sortRequest.contentEquals("date") || sortRequest.contentEquals("date.asc"))
			appointments = dataservice.sortByDate(appointments, false);
		else if (sortRequest.contentEquals("date.desc"))
			appointments = dataservice.sortByDate(appointments, true);

		return appointments;
	}

}
