package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import agilex.commons.NullChecker;

import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataService;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.Consult;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ConsultLinkBuilder;

@Path("/patient/{assigning-authority}/{patient-id}/consults")
@Component
@Scope("request")
public class PatientConsultsResource {
    @Autowired
    MessageSource messageSource;
    //private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientConsultsResource.class);


    /**
	 * Retrieves patient {@link Consults} from the configured data source based upon the scope given {longitudinal, operational}<br/>
	 * Calls the MDWS service EmrSvc.getConsultsForPatient for the given patient - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
	 * <br/>
	 *
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>The date you want consults to be after.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>The date you want consults to be before.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>statuses</td>
	 * 		<td>A comma separated list of statuses (See {@link com.agilex.healthcare.mobilehealthplatform.enumeration.ConsultStatus}).</td>
	 * 	</tr>
	 * </table>
	 *
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param scope {longitudinal, operational}
	 * @return JSON containing the {@link Consults}
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("scope/{scope}")
	public Consults fetchPatientConsultsByScope(@PathParam("assigning-authority") String assigningAuthority,
												 @PathParam("patient-id") String patientId,
												 @PathParam("scope") String scope,
												 @Context UriInfo uriInfo,
												 @Context HttpHeaders headers) {

		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		ConsultFilter filter = ConsultFilterFactory.createFilterFromUri(uriInfo.getRequestUri());
		Consults results = fetchPatientConsults(patientIdentifier, filter, ScopeFilter.getInstanceForScope(scope));

		ConsultLinkBuilder linkBuilder = new ConsultLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(results,uriInfo.getRequestUri());

		return results;
	}

	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("{consultIEN}")
	public Consult fetchPatientConsultByIEN(@PathParam("assigning-authority") String assigningAuthority,
												 @PathParam("patient-id") String patientId,
												 @PathParam("consultIEN") String consultIEN,
												 @Context UriInfo uriInfo,
												 @Context HttpHeaders headers) {

		final PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		ConsultFilter filter = ConsultFilterFactory.createFilterFromUri(uriInfo.getRequestUri());
		Consults results = fetchPatientConsults(patientIdentifier, filter, ScopeFilter.getInstanceForOperationalScope());
		Consult matchingConsult = null;
		if (NullChecker.isNotNullish(results)) {
			for (Consult consult : results) {
				if (consult.getConsultIEN().equalsIgnoreCase(consultIEN)) {
					matchingConsult = consult;
					break;
				}
			}
		}
		return matchingConsult;
	}

	private Consults fetchPatientConsults(PatientIdentifier patientIdentifier, ConsultFilter filter, ScopeFilter scopeFilter) {
		ConsultDataService dataservice = new ConsultDataService();
		Consults consults = dataservice.fetchPatientConsults(patientIdentifier, filter, scopeFilter);
		return consults;
	}
}
