package com.agilex.healthcare.mobilehealthplatform.restservice;

import java.net.URI;
import java.util.LinkedList;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import org.codehaus.jettison.json.JSONArray;
import org.springframework.context.annotation.Scope;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.lab.ChemistryLabDataService;
import com.agilex.healthcare.mobilehealthplatform.datalayer.order.OrderDataService;
import com.agilex.healthcare.mobilehealthplatform.domain.Admissions;
import com.agilex.healthcare.mobilehealthplatform.domain.AtomLink;
import com.agilex.healthcare.mobilehealthplatform.domain.HospitalLocations;
import com.agilex.healthcare.mobilehealthplatform.domain.LabResult;
import com.agilex.healthcare.mobilehealthplatform.domain.Order;
import com.agilex.healthcare.mobilehealthplatform.domain.OrderPickListItemEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.Orders;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RadiologyOrder;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;

@Path("/patient/{assigning-authority}/{patient-id}/orders")
@Component
@Scope("request")
public class PatientOrdersResource {

	/**
	 * Retrieves all problems for patients based upon the scope provided.<br/>
	 * Calls the MDWS service OrderMgmtSvc.getOrdersByCustom<br/><br/>
	 * 
	 * This method also takes {startDate, endDate, entryDate} query parameters
	 * 
	 * @param assigningAuthority Assigning Authority associated with the patient's login
	 * @param patientId Identifier of the Patient
	 * @param site
	 * @return
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{site}")
	public Orders fetchOrdersBySite(@PathParam("assigning-authority") String assigningAuthority, 
											   @PathParam("patient-id") String patientId, 
											   @PathParam("site") String site,
											   @QueryParam("startDate") String startDate,
											   @QueryParam("endDate") String endDate,
											   @QueryParam("status-ien") String statusIEN,
											   @QueryParam("service-ien") String serviceIEN,
											   @Context UriInfo uriInfo, 
											   @Context HttpHeaders headers) {
		OrderDataService orderDataService = new OrderDataService();
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);

		Orders orders =  orderDataService.fetchOrdersByCustom(patientIdentifier, null == startDate ? "" : startDate, null == endDate ? "" : endDate, statusIEN, serviceIEN);

		return buildOrderResultHistoryLinks(uriInfo.getBaseUri(), patientIdentifier, site, orders);
	}

	private Orders buildOrderResultHistoryLinks(URI baseUri, PatientIdentifier patientIdentifier, String site, Orders orders) {
		for (Order order : orders) {
			URI orderHistoryResultUri = UriBuilder
					.fromUri(baseUri)
					.path("patient")
					.path(patientIdentifier.getAssigningAuthority())
					.path(patientIdentifier.getUniqueId())
					.path("order-results-history")
					.path("site")
					.path(site)
					.queryParam("order-ien", order.getOrderId())
					.build();

			AtomLink link = new AtomLink();
			link.setHref(orderHistoryResultUri);
			link.setType(MediaType.APPLICATION_JSON.toString());
			link.setTitle("order-results-history");
			order.getLink().add(link);
		}

		return orders;
	}

	/**
	 * Calls the MDWS service OrderMgmtSvc.getOrdersByCustom
	 */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/custom")
    public JSONArray fetchOrdersByCustom(@PathParam("assigning-authority") String assigningAuthority,
                        @PathParam("patient-id") String patientId,
                        @PathParam("site") String site,
                        @QueryParam("startDate") String startDate,
                        @QueryParam("endDate") String endDate,
                        @QueryParam("status") String status,
                        @Context UriInfo uriInfo,
                        @Context HttpHeaders headers) {

        OrderDataService orderDataService = new OrderDataService();

		List<String> services = new LinkedList<String>();
		services.add("2");
		services.add("5");
		services.add("34");

        return orderDataService.fetchOrdersJSONByCustom(new PatientIdentifier(assigningAuthority, patientId),
				null == startDate ? "" : startDate, null == endDate ? "" : endDate, status, services);

    }

    /**
     * Unable to determine if this calls mdws.
     */
    @PUT
    @Path("site/{site}/save-patient")
    @Consumes({ "application/xml", "application/json" })
    @Produces({ "application/xml", "application/json" })
    public Order saveRadiologyOrder(RadiologyOrder radiologyOrder,
                                             @PathParam("assigning-authority") String assigningAuthority,
                                             @PathParam("patient-id") String patientId,
                                             @PathParam("site") String site,
                                             @Context UriInfo uriInfo,
                                             @Context HttpHeaders headers) {
        OrderDataService orderDataService = new OrderDataService();
        return orderDataService.saveRadiologyOrder(new PatientIdentifier(assigningAuthority, patientId), radiologyOrder);

    }

    /**
     * Unable to determine if this calls mdws.
     */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/category-supplements")
    public OrderPickListItemEntries fetchCategorySupplements(@PathParam("assigning-authority") String assigningAuthority,
                                                             @PathParam("patient-id") String patientId,
                                                             @PathParam("site") String site,
                                                             @QueryParam("displayGroup") String displayGroup,
                                                             @QueryParam("category") String category,
                                                             @Context UriInfo uriInfo,
                                                             @Context HttpHeaders headers) {
        OrderDataService orderDataService = new OrderDataService();
        return orderDataService.fetchCategorySupplements(new PatientIdentifier(assigningAuthority, patientId),
                displayGroup, category);
    }

    /**
     * Calls the MDWS service EmrSvc.getAdmissions for the given patient - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
     */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/admissions")
    public Admissions fetchAdmissions(@PathParam("assigning-authority") String assigningAuthority,
                                      @PathParam("patient-id") String patientId,
                                      @PathParam("site") String site,
                                      @Context UriInfo uriInfo,
                                      @Context HttpHeaders headers) {
        OrderDataService orderDataService = new OrderDataService();
        return orderDataService.getAdmissions(new PatientIdentifier(assigningAuthority, patientId));
    }

    /**
     * Calls the MDWS service EmrSvc.getClinics - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
     */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/clinics")
    public HospitalLocations fetchClinics(@PathParam("assigning-authority") String assigningAuthority,
                                      @PathParam("patient-id") String patientId,
                                      @PathParam("site") String site,
                                      @Context UriInfo uriInfo,
                                      @Context HttpHeaders headers) {
        OrderDataService orderDataService = new OrderDataService();
        return orderDataService.getClinics(site);
    }

    /**
     * Calls the MDWS service EmrSvc.getLocations - if configured to use mdws (operational) instead of CDW (longitudinal)<br/>
     */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/locations")
    public HospitalLocations fetchLocations(@PathParam("assigning-authority") String assigningAuthority,
                                      @PathParam("patient-id") String patientId,
                                      @PathParam("site") String site,
                                      @Context UriInfo uriInfo,
                                      @Context HttpHeaders headers) {
        OrderDataService orderDataService = new OrderDataService();
        return orderDataService.getLocations(site);
    }
    
    /**
     * Calls Mongo
     */
    @GET
    @Produces({"application/xml", "application/json"})
    @Path("site/{site}/creatininetest")
    public LabResult fetchCreatenineTestResults(@PathParam("assigning-authority") String assigningAuthority,
    											@PathParam("patient-id") String patientId,
    											@PathParam("site") String site,
    											@Context UriInfo uriInfo,
    											@Context HttpHeaders headers){
    	PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(uriInfo.getRequestUri());
		ScopeFilter scopeFilter = ScopeFilter.getInstanceForScope("operational");
    	ChemistryLabDataService dataservice = new ChemistryLabDataService();
		LabResult labResult = dataservice.getCreatineLabTestInfo(patientIdentifier, dateFilter, scopeFilter,site);
		return labResult;
    }
}
