package com.agilex.healthcare.mobilehealthplatform.restservice;

import java.util.Map;
import java.util.Map.Entry;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientClinicDataServiceMDWS;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientDataService;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientSearchCriteria;
import com.agilex.healthcare.mobilehealthplatform.datalayer.patient.PatientSearchPaging;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientSearchResults;
import com.agilex.healthcare.mobilehealthplatform.restservice.exception.PatientSearchResultsException;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.serviceregistry.ScopeFilter;
import com.agilex.healthcare.mobilehealthplatform.utils.QueryParameterHelper;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.PatientLinkBuilder;
import com.agilex.healthcare.utility.NullChecker;

/**
 * REST Services for searching for patients.<br/>
 * <br/>
 * 
 * There are 3 types of search: clinic, ward, and patient.
 * <br/>
 * <br/>
 * <b>Clinic:</b> visitdate and clinic must be populated.  The other filter criteria will be applied to remove entries
 * from the list returned.<br/>
 * <font color="red">NOTE: It should be noted that even though MDWS supports pagination, we are not using it here at
 * this time.  That means that the potential exists to search for a clinic, receive more than 44 of them, and not
 * show the user that there are more clinics to choose from.  Also, the pagination that is being done here
 * (for patients) as part of the REST service is not coming from MDWS. It is part of the filtering logic (by this REST
 * service) for patients in the clinic.</font>
 * <br/>
 * Clinics search is returning data in the mdws format (44 records at a time), the searchString passed in is the index
 * closest to where you want to start returning records from (like pointing to a name in a phone book).  The filtering
 * and other stuff they are doing here is not guaranteed to be accurate as you are only looking at a very small subset
 * of the data.
 * <br/>
 * <br/>
 * <b>Ward:</b> ward must be populated.  The other filter criteria will be applied to remove entries
 * from the list returned.<br/>
 * <font color="red">NOTE: The pagination that is being done here (for patients) as part of the REST service is not
 * coming from MDWS. It is part of the filtering logic (by this REST service) for patients in the ward.</font>
 * <br/>
 * Wards coming from MDWS are not filtered.  You receive all of them.  We should change the UI to receive all of the
 * wards and perform filtering on their end if needed - the backend should not be the one filtering.
 * <br/>
 * <br/>
 * <b>Patient:</b> There are 5 types of search that can be performed here.  "lastNameOnly" or "lastName,firstName" or
 * "lastName,firstName middleName" or "ssn" or finally "lastInitialLast4Ssn"<br/>
 * "lastNameOnly" - returns a list of Patients whose last name match<br/>
 * "lastName,firstName" - returns a list of Patients whose last name and first name match<br/>
 * "lastName,firstName middleName" - returns a list of Patients whose last name, first name, and middle name match<br/>
 * "ssn" - returns a list of Patients whose ssn match<br/>
 * "lastInitialLast4Ssn" - returns a list of Patients whose last initial and last 4 digits of their ssn match<br/>
 * <font color="red">NOTE: There is currently a 400 error that will be thrown if more than 40 records are returned
 * for Patients matching the filter criteria.  It should also be noted that even though MDWS supports pagination, we
 * are not using it here at this time.</font>
 * <br/>
 * <br/>
 * 
 * <font color="red">NOTE: The search functionality provided by these REST services differs from that of CPRS. These
 * differences may be removed in the future to make this behave just like CPRS (so you should not rely on them staying
 * this way).</font><br/>
 * Following is an explanation of the known differences at this time.<br/>
 * 1.	There is a special filter of the patients that are returned from mdws to ensure that the criteria passed in
 * matches exactly what is returned.<br/>
 * &nbsp;&nbsp;&nbsp;a.	This is the comment in the code: "MDWS returns more patients than we expect due to it's loose
 * search criteria ensure first and last names contain the appropriate search criteria"<br/>
 * When that comment was written, it appears they didn't understand that CPRS behaves more like a telephone book.
 * When you pass in a filter such as "lastName" or "lastName,firstName" to CPRS, it is the same as opening up the phone
 * book to that specific page and then listing all the rest of the names in the book.  When you pass in
 * lastInitialLast4Ssn, it returns you data that is filtered to only include those that match.<br/>
 * 2.	The data is reformatted from how it is received by MDWS<br/>
 * &nbsp;&nbsp;&nbsp;a.	Examples include adding spaces between lastName,FirstName,MiddleName for the displayName, adding
 * hyphens between digits for ssn, changing gender from a single character (m or f) to MALE or FEMALE, populating a
 * middle initial, populating their age, etc.<br/>
 * 3.	The collection of Patients returned from MDWS is sorted.<br/>
 * &nbsp;&nbsp;&nbsp;a.	The results are sorted by lastName, firstName, dateOfBirth, and ssn.<br/>
 * 4.	We remove any results past our max results (which is currently 40).<br/>
 * <font color="red">NOTE: Because these cause search functionality different from that of CPRS, some or all of these
 * differences may be changed at some time in the future.<br/>
 * </font>
 * 
 * <h3>Additional Path Parameters</h3>
 * <table border="1">
 * 	<tr><th>name</th><th>description</th></tr>
 * 	<tr>
 * 		<td>ssn</td>
 * 		<td>filters search results based on ssn</td>
 * 	</tr>
 * 	<tr>
 * 		<td>searchstring</td>
 * 		<td>the name (or partial name) of the results that you want returned.<br/>
 * 			If searchstring is not supplied, then all results will be returned.
 * 		</td>
 * 	</tr>
 * <tr>
 * 		<td>firstname</td>
 * 		<td>the first name of the patient</td>
 * 	</tr>
 * <tr>
 * 		<td>lastname</td>
 * 		<td>the last name of the patient</td>
 * 	</tr>
 * <tr>
 * 		<td>patientid</td>
 * 		<td>the patient id for the patient</td>
 * 	</tr>
 * <tr>
 * 		<td>ward</td>
 * 		<td>the id of the ward you want to retrieve patients for</td>
 * 	</tr>
 * <tr>
 * 		<td>clinic</td>
 * 		<td>If clinic is populated, then we perform the search by Clinic Visit Date.  This will then assume that
 * 			visitdate is also populated.  To filter the patients returned for that clinic on that date,
 * 			populate any of the filter parameters.</td>
 * 	</tr>
 * <tr>
 * 		<td>maxrecords</td>
 * 		<td>the maximum number of records you want returned by the search after filtering. Internally, we retrieve the
 * 			full list of all patients that match the criteria.  Then, we return the subset of them that is a combination
 * 			of maxrecords and page.  This is done only for ward or clinic search.
 * 		</td>
 * 	</tr>
 * <tr>
 * 		<td>page</td>
 * 		<td>when it comes to the results returned from a ward or a clinic search, this can be used to filter
 * 			the number of patients shown in the resultset.  Internally, we retrieve the full list of all patients that
 * 			match the criteria.  Then, we return the subset of them that is a combination of maxrecords and page.</td>
 * 	</tr>
 * <tr>
 * 		<td>visitdate</td>
 * 		<td>If visitdate is populated, then we perform the search by Clinic Visit Date.  This will then assume that
 * 			clinic is also populated.  To filter the patients returned for that clinic on that date,
 * 			populate any of the filter parameters.
 * 		</td>
 * 	</tr>
 * </table>
 */
@Component
@Path("/patients")
@Scope("request")
public class PatientsResource {
	private static final org.apache.commons.logging.Log logger = org.apache.commons.logging.LogFactory.getLog(PatientsResource.class);

	private static final String SEARCHCRITERIA_SSN = "ssn";
	private static final String SEARCHCRITERIA_SEARCHSTRING = "searchstring";
	private static final String SEARCHCRITERIA_FIRSTNAME = "firstname";
	private static final String SEARCHCRITERIA_LASTNAME = "lastname";
	private static final String SEARCHCRITERIA_PATIENTID = "patientid";
	private static final String SEARCHCRITERIA_WARD = "ward";
	private static final String SEARCHCRITERIA_CLINIC = "clinic";
	private static final String PAGINGCRITERIA_MAXRECORDS = "maxrecords";
	private static final String PAGINGCRITERIA_PAGENUMBER = "page";
	private static final String SEARCHCRITERIA_VISIT_DATE = "visitdate";
	
	private static final int MAX_SEARCH_PATIENTS = 40;


	/**
	 * See the documentation of this class (above) for how this service works.<br/>
	 * This service is identical to the other one except it retrieves the siteCode from where the user is currently
	 * logged in rather than passing it in as a parameter.
	 * 
	 * @return JSON containing a collection of Patients.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	public PatientSearchResults patientSearch(@Context UriInfo uriInfo, @Context HttpHeaders headers) {
		return handlePatientSearchRequest(uriInfo);
	}

	/**
	 * See the documentation of this class (above) for how this service works.
	 * 
	 * @param siteCode The siteId for the facility (called facilityCode in other services).
	 * @return JSON containing a collection of Patients.
	 */
	@GET
	@Path("site/{site-code}")
	@Produces({ "application/xml", "application/json" })
	public PatientSearchResults patientSearch(@Context UriInfo uriInfo, @Context HttpHeaders headers, @PathParam("site-code") String siteCode) {
		return handlePatientSearchRequest(uriInfo, siteCode);
	}
	
	private PatientSearchResults handlePatientSearchRequest(UriInfo uriInfo){
		String siteCode = retrieveSiteCode();
		return handlePatientSearchRequest(uriInfo, siteCode);
	}
	
	private PatientSearchResults handlePatientSearchRequest(UriInfo uriInfo, String siteCode){
		PatientSearchResults patientSearchResults = null;
		logger.info(String.format("received patient search request [qs=%s]", uriInfo.getRequestUri().getQuery()));
		patientSearchResults = performPatientSearch(uriInfo, siteCode);
		logger.info(patientSearchResults.size() + " patient(s) found");

		return patientSearchResults;
	}

	private String retrieveSiteCode() {
		MhpUser mhpUser = MhpUserFactory.createFromSecurityContext();
		String siteCode = mhpUser.getVistaLocation();
		if(siteCode == null || siteCode.equals("")){
			logger.error("User vista location is not available.");
			throw new RuntimeException("User vista location is not available.");
		}
		return siteCode;
	}
	

	private PatientSearchResults performPatientSearch(UriInfo uriInfo, String siteCode) {
		/**
		 * NOTE: All 4 differences in behavior compared to CPRS (see class JavaDoc) guarantee that the results returned
		 * by mdws will not work identical to cprs. For example, when pagination
		 * is implemented and we pass in "SMITH, JOSEPH" (with a space in it) to mdws for the next record, it will return
		 * records starting with "SMITH,AARON" (just as if we only passed in "SMITH") because it needs there to be no
		 * space between last and first name in the displayName given to it.
		 */		
		ScopeFilter scopeFilter = ScopeFilter.getInstanceForOperationalScope();
		PatientSearchResults patientSearchResult = null;
		String query = null;
		if (uriInfo != null) {
			query = uriInfo.getRequestUri().getQuery();
		}

		PatientSearchCriteria patientSearchCriteria = extractPatientSearchCriteria(query, siteCode);
		PatientSearchPaging pagingInfo = extractPagingInfo(query);
		
		/**
		 * Implementation of a clinic search by date must use a different MDWS library (SchedulingSvc)
		 * so if there is a date present (new for a patient search by clinic and date) then the 
		 * dataservice for MDWS must be different than the other patient searches. We wish to not 
		 * have any ill effects on existing patient search behavior.  
		 */	
		if (NullChecker.isNotNullish(patientSearchCriteria.getVisitDate())) {
			patientSearchResult = performPatientSearchByClinicVisitDate(uriInfo, siteCode, patientSearchCriteria, pagingInfo);
		}
		else {
			patientSearchResult = performPatientSearch(uriInfo, siteCode, patientSearchCriteria, scopeFilter, pagingInfo);
		}

		return patientSearchResult;
	}
	
	private boolean isClinicOrWard(PatientSearchCriteria patientSearchCriteria) {
		if (patientSearchCriteria.getClinic() != null && patientSearchCriteria.getClinic().isEmpty() == false)
			return true;
		if (patientSearchCriteria.getWard() != null && patientSearchCriteria.getWard().isEmpty() == false)
			return true;
		return false;
	}
	
	private int pagingTotalResults(PatientSearchResults patientSearchResult) {
		if (patientSearchResult.getPaging() != null	&& patientSearchResult.getPaging().getTotalResults() != null)
			return patientSearchResult.getPaging().getTotalResults();
		else
			return -1;
	}
	
	private PatientSearchResults performPatientSearch(UriInfo uriInfo, String siteCode, PatientSearchCriteria patientSearchCriteria, ScopeFilter scopeFilter, PatientSearchPaging pagingInfo) {
		
		PatientSearchResults patientSearchResult = null;
		
		PatientDataService dataservice = new PatientDataService();
		
		if (isClinicOrWard(patientSearchCriteria) == false && pagingInfo.getMaxRecords() == 0)
			pagingInfo.setMaxRecords(MAX_SEARCH_PATIENTS);

		try {
			if (patientSearchCriteria.getPrototype() != null && NullChecker.isNotNullish(patientSearchCriteria.getPrototype().getPatientIdentifier().getUniqueId())) {
				logger.debug("performing search by patient id");
				Patient patient = dataservice.getPatient(patientSearchCriteria.getPrototype().getPatientIdentifier(), scopeFilter);

				patientSearchResult = new PatientSearchResults();
				patientSearchResult.add(patient);
			} else {
				logger.debug("performing search by searchstring, prototype, ward, or clinic");
				patientSearchResult = dataservice.patientSearch(patientSearchCriteria, pagingInfo, scopeFilter);
			}
		} catch (IllegalArgumentException e) {
			logger.error("IllegalArgumentException occurred, possibly due to insufficent criteria, returning empty collection ", e);
			patientSearchResult = new PatientSearchResults();
		}
		
		if (isClinicOrWard(patientSearchCriteria) == false && (patientSearchResult.size() > MAX_SEARCH_PATIENTS || pagingTotalResults(patientSearchResult) > MAX_SEARCH_PATIENTS))
			throw new PatientSearchResultsException("Please refine your search criteria to decrease the size of the result set");

		PatientLinkBuilder linkBuilder = new PatientLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(patientSearchResult, uriInfo.getRequestUri(),siteCode);

		return patientSearchResult;
	}

	
	private PatientSearchResults performPatientSearchByClinicVisitDate(UriInfo uriInfo, String siteCode, PatientSearchCriteria patientSearchCriteria, PatientSearchPaging pagingInfo) {
		
		PatientSearchResults patientSearchResult = null;
		
		//PatientDataService dataservice = new PatientDataService();
		PatientClinicDataServiceMDWS dataservice = new PatientClinicDataServiceMDWS();
		try {
			if (NullChecker.isNotNullish(patientSearchCriteria.getClinic()) && (NullChecker.isNotNullish(patientSearchCriteria.getVisitDate()))) {
				logger.debug("performing search by clinic/visit-date");
				patientSearchResult = dataservice.patientSearchByClinic(patientSearchCriteria, siteCode, pagingInfo);
			}
		} catch (IllegalArgumentException e) {
			logger.error("IllegalArgumentException occurred, possibly due to insufficent criteria, returning empty collection ", e);
			patientSearchResult = new PatientSearchResults();
		}

		PatientLinkBuilder linkBuilder = new PatientLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(patientSearchResult, uriInfo.getRequestUri(),siteCode);
		return patientSearchResult;
	}
	

	private PatientSearchCriteria extractPatientSearchCriteria(String query, String siteCode) {
		PatientSearchCriteria patientSearchCriteria = new PatientSearchCriteria();
		Patient prototype = new Patient();
		patientSearchCriteria.setPrototype(prototype);
		patientSearchCriteria.setVistaSiteCode(siteCode);
		
		if (NullChecker.isNotNullish(query)) {
			Map<String, String> queryParams = QueryParameterHelper.getQueryMap(query);
			for (Entry<String, String> queryParam : queryParams.entrySet()) {
				if (NullChecker.isNotNullish(queryParam.getKey()) && NullChecker.isNotNullish(queryParam.getValue())) {
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_LASTNAME)) {
						prototype.setLastName(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_FIRSTNAME)) {
						prototype.setFirstName(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_SEARCHSTRING)) {
						patientSearchCriteria.setSearchString(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_SSN)) {
						prototype.setSsn(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_PATIENTID)) {
						logger.debug("got patient id value = " + queryParam.getValue());
						prototype.getPatientIdentifier().setUniqueId(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_WARD)) {
						patientSearchCriteria.setWard(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_CLINIC)) {
						patientSearchCriteria.setClinic(queryParam.getValue());
					}
					if (queryParam.getKey().toLowerCase().contentEquals(SEARCHCRITERIA_VISIT_DATE)) {
						patientSearchCriteria.setVisitDate(queryParam.getValue());
					}
				}
			}
		}
		return patientSearchCriteria;
	}

	private PatientSearchPaging extractPagingInfo(String query) {
		PatientSearchPaging pagingInfo = new PatientSearchPaging();

		if (NullChecker.isNotNullish(query)) {
			Map<String, String> queryParams = QueryParameterHelper.getQueryMap(query);
			for (Entry<String, String> queryParam : queryParams.entrySet()) {
				if (NullChecker.isNotNullish(queryParam.getKey()) && NullChecker.isNotNullish(queryParam.getValue())) {
					if (queryParam.getKey().equalsIgnoreCase(PAGINGCRITERIA_MAXRECORDS)) {
						pagingInfo.setMaxRecords(Integer.parseInt(queryParam.getValue()));
					}
					if (queryParam.getKey().equalsIgnoreCase(PAGINGCRITERIA_PAGENUMBER)) {
						pagingInfo.setPageNumber(Integer.parseInt(queryParam.getValue()));
					}
				}
			}
		}
		return pagingInfo;
	}
}
