package com.agilex.healthcare.mobilehealthplatform.restservice;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.security.oauth2.common.exceptions.UnauthorizedClientException;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultDataService;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilter;
import com.agilex.healthcare.mobilehealthplatform.datalayer.consult.ConsultFilterFactory;
import com.agilex.healthcare.mobilehealthplatform.domain.Consults;
import com.agilex.healthcare.mobilehealthplatform.domain.MhpUser;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.mobilehealthplatform.security.MhpUserFactory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ProviderConsultLinkBuilder;

@Path("/provider/{facility-code}/{provider-id}")
@Component
@Scope("request")
public class ProviderConsultsResource {

    /**
	 * Retrieves {@link Consults} written by a provider from CDW.
	 * @param facilityCode facility code (sta3n) such as 688 for DCVAMC.
	 * @param providerId provider unique identifier from Vista, provider IEN.
	 * @return {@link Consults}
	 */

	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("/consults/system/cdw")
	public Consults fetchConsultsWrittenByProvider(@PathParam("facility-code") String facilityCode,
												   @PathParam("provider-id") String providerId,
												   @Context UriInfo uriInfo,
												   @Context HttpHeaders headers){

		MhpUser mhpUser = getAuthenticatedUserFromProvider(providerId, facilityCode);

		ProviderIdentifier providerIdentifier = new ProviderIdentifier(mhpUser.getVistaLocation(), mhpUser.getId());
		ConsultFilter filter = ConsultFilterFactory.createFilterFromUri(uriInfo.getRequestUri());
		ConsultDataService dataservice = new ConsultDataService();
		Consults consults = dataservice.fetchConsultsByProvider(providerIdentifier, filter);
		if (consults != null) {
			ProviderConsultLinkBuilder linkBuilder = new ProviderConsultLinkBuilder(uriInfo.getBaseUri());
			linkBuilder.fillLinks(consults, uriInfo.getRequestUri(), mhpUser);
		}
		return consults;
	}

	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("/consults-by-specialties/system/cdw")
	public Consults fetchConsultsBySpecialties(@PathParam("facility-code") String facilityCode,
											   @PathParam("provider-id") String providerId,
											   @Context UriInfo uriInfo,
											   @Context HttpHeaders headers){

		MhpUser mhpUser = getAuthenticatedUserFromProvider(providerId, facilityCode);
		ConsultFilter filter = ConsultFilterFactory.createFilterFromUri(uriInfo.getRequestUri());
		if (filter.getSpecialties().size() == 0)
			throw new IllegalArgumentException("Expected one or more specialties as input.");

		ConsultDataService dataservice = new ConsultDataService();
		Consults consults = dataservice.fetchConsultsBySpecialties(filter, facilityCode);
		if (consults != null) {
			ProviderConsultLinkBuilder linkBuilder = new ProviderConsultLinkBuilder(uriInfo.getBaseUri());
			linkBuilder.fillLinks(consults, uriInfo.getRequestUri(), mhpUser);
		}
		return consults;
	}

	private MhpUser getAuthenticatedUserFromProvider(String providerId, String facilityCode) throws UnauthorizedClientException{
		MhpUser mhpUser = MhpUserFactory.createFromSecurityContext();
		if ((!providerId.equals(mhpUser.getId())) || (!facilityCode.equals(mhpUser.getVistaLocation()))){
			throw new UnauthorizedClientException(String.format("Provider with id=%s at facility=%s is not logged on.", providerId, facilityCode));
		}
		return mhpUser;
	}

}
