package com.agilex.healthcare.notewriter.restservice;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalNoteSignature;
import com.agilex.healthcare.mobilehealthplatform.domain.Count;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitles;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteWriterResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndexFactory;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.ClinicalEventResourceDirectoryBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.ClinicalEventLinkBuilder;
import com.agilex.healthcare.notewriter.datalayer.ClinicalEventDataService;
import com.agilex.healthcare.notewriter.datalayer.SearchFilter;
import com.agilex.healthcare.notewriter.datalayer.SearchFilterFactory;
import com.agilex.healthcare.utility.NullChecker;

/**
 * REST Services for saving, retrieving, updating, and signing Clinical Event's.  Also included are services for
 * retrieving NoteTitles and the last selected NoteTitle for a provider.
 */
@Path("/note-writer/clinical-event-service/")
@Component
@Scope("request")
public class ClinicalEventResource {

	@Resource
	ClinicalEventResourceDirectoryBuilder clinicalEventResourceDirectoryBuilder;
	
	@Resource(name = "clinicalEventDataService")
	ClinicalEventDataService clinicalEventDataService;
	/**
	 * Resource directory for Clinical Events (Progress Note and Encounters).
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @return JSON containing the various resources that NoteWriter can use.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/resource-directory")
	public NoteWriterResourceDirectory fetchResourceDirectory(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @QueryParam("assigningAuthority") String assigningAuthority,  @QueryParam("patientId") String patientId) {
		NoteWriterResourceDirectory resourceDirectory = clinicalEventResourceDirectoryBuilder.getResourceDirectory(uriInfo.getBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		return resourceDirectory;
	}
	
	/**
	 * Retrieves the latest clinical event for the patient that is in a draft state (i.e. one that is not signed).<br/>
	 * <font color="red">Currently we're only supporting 1 clinical event per patient.</font>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @return JSON containing the latest clinical event for this patient.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events/latest")
	public ClinicalEvent getLatestClinicalEvent(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId) {
		//Currently we're only supporting 1 clinical event per patient.
		ClinicalEvent latestClinicalEvent = clinicalEventDataService.fetchLatestClinicalEvent(facilityCode, staffId, assigningAuthority, patientId);

		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(latestClinicalEvent);
		
		return latestClinicalEvent;
	}

	/**
	 * Saves the clinical event to the database.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param clinicalEvent JSON containing a valid clinical event.
	 * @return JSON containing the clinical event that was saved (including the new id for that clinical event)
	 */
	@POST
	@Consumes({ "application/xml", "application/json"  })
	@Produces({ "application/xml", "application/json"  })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events")
	public ClinicalEvent saveClinicalEvent(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, ClinicalEvent clinicalEvent) {
		ClinicalEvent savedClinicalEvent = clinicalEventDataService.saveClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEvent);

		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(savedClinicalEvent);
		
		return savedClinicalEvent;
	}
	
	/**
	 * Retrieves a Count of how many clinical events exist for the patient by this staff member.<br/>
	 * <font color="red">NOTE: Since we are currently only supporting 1 clinical event per patient this will always return one.</font>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @return JSON containing a Count of how many clinical events exist for the patient by this staff member.<br/>
	 * <font color="red">NOTE: Since we are currently only supporting 1 clinical event per patient this will always return one.</font>
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events/count")
	public Count getClinicalEventCount(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId) {
		return clinicalEventDataService.getClinicalEventCount(facilityCode, staffId, assigningAuthority, patientId);
	}
	
	/**
	 * Retrieves the one clinical event for the patient by this provider with the given clinicalEventId that is in a
	 * state of DRAFT in the database.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param clinicalEventId The id (in the database) of the clinical event for this patient that is owned by this provider.
	 * @return JSON containing the clinical event.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events/id/{clinical-event-id}")
	public ClinicalEvent getClinicalEvent(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId, @PathParam("clinical-event-id") String clinicalEventId) {
		ClinicalEvent fetchedClinicalEvent = clinicalEventDataService.fetchClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEventId);

		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(fetchedClinicalEvent);
		
		return fetchedClinicalEvent;
	}
	
	/**
	 * Updates the clinical event (with the clinical event id that matches the one passed in) in the database with the
	 * clinical event that is passed in.  The id is updated in the clinical event before saving it (though it should
	 * should already exist in the clinicalEvent passed in).
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param clinicalEventId The id (in the database) of the clinical event for this patient that is owned by this provider.
	 * @param clinicalEvent The clinicalEvent that you want to replace in the database
	 * @return JSON containing The clinical event that was saved in the database (including any changes that may be made
	 * to it when saving such as modifications to the last modified date.
	 */
	@PUT
	@Consumes({ "application/xml", "application/json" })
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events/id/{clinical-event-id}")
	public ClinicalEvent updateClinicalEvent(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
											@PathParam("facility-code") String facilityCode,
											@PathParam("staff-id") String staffId,
											@PathParam("assigning-authority") String assigningAuthority,
											@PathParam("patient-id") String patientId,
											@PathParam("clinical-event-id") String clinicalEventId,
											ClinicalEvent clinicalEvent) {
		clinicalEvent.setId(clinicalEventId);
		ClinicalEvent updatedClinicalEvent = clinicalEventDataService.updateClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEvent);

		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(updatedClinicalEvent);
		
		return updatedClinicalEvent;
	}
	
	/**
	 * Retrieves all of the NoteTitles for the facility that contain (or partially contain) the parameter searchstring.<br/>
	 * <br/>
	 * 
	 * <font color="red">NOTE: This currently accesses the mongo database to retrieve note titles and performs
	 * filtering based on that.  The functionality differs from the way CPRS behaves.  It also means the TableSync
	 * must be run to populate the NoteTitles in the mongo database.  Because you are accessing a real database, you can
	 * perform filtering and other database operations.  You are not limited to the interface or the way that MDWS
	 * returns data (indexed like a phone book).  However, it also means you don't have the most current data.</font>
	 * <br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>the name (or partial name) of the results that you want returned.<br/>
	 * 			If searchstring is not supplied, then all results will be returned.
	 * 		</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>maxrecords</td>
	 * 		<td>the maximum number of records you want returned by the search after filtering.
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @return JSON containing a collection of all of the NoteTitles which match or partially match your searchstring as
	 * contained in the Mongo database.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-titles/progress-notes")
	public NoteTitles getNoteTitles(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId) {
		SearchFilterFactory factory = new SearchFilterFactory();
		SearchFilter filter = factory.createFromUriDecoded(uriInfo.getRequestUri());
		
		NoteTitles noteTitles = clinicalEventDataService.getMongoNoteTitles(facilityCode, staffId, filter);
		
		return noteTitles;
	}

	/**
	 * Returns the NoteTitles starting at searchIndex using MDWS services and returning data following the same behavior as CPRS.
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>
	 * The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * If searchIndex.getSearchString() is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * <font color="red"><b>Example</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * what you searched for.<br/><br/>
	 * 		</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>direction</td>
	 * 		<td>
	 * <b>Optional Direction</b><br/>
	 * If you decide to include the optional direction, you can pass in &quot;-1&quot;, or &quot;1&quot;.
	 * This tells the system whether to return the previous (&quot;-1&quot;) or the next (&quot;1&quot;) set of data.
	 * If you don't pass anything in, it's similar to passing in &quot;1&quot;.  When returning the previous set of
	 * data, it does not include the index you searched for and the order of elements in the returned list is in a
	 * reversed order.<br/><br/>
	 * 
	 * <font color="red"><b>Example of using the (optional) direction</b></font><br/>
	 * Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * If you searched for 'bo' with a direction of 1, you'd get back {'bones', 'brain'}.<br/>
	 * If you searched for 'bp' with a direction of 1, you'd get back {'brain'}.<br/>
	 * If you searched for 'bo' with a direction of -1, you'd get back {'bladder', 'benefit'}.<br/>
	 * If you searched for 'bp' with a direction of -1, you'd get back {'bones', 'bladder', 'benefit'}<br/>
	 * Notice that the list elements are reversed when direction is -1.<br/><br/>
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.  <font color="red">NOTE: This is no longer used as we can get the information
	 * from the logged on user.</font>
	 * @param staffId The id of the staff member.  This value is currently ignored.
	 * @return JSON containing the NoteTitles (up to 44 at a time)
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-titles/mdws-progress-notes")
	public NoteTitles getMdwsNoteTitles(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId) {
		SearchIndexFactory factory = new SearchIndexFactory();
		SearchIndex searchIndex = factory.createFromUriDecoded(uriInfo.getRequestUri());
		
		NoteTitles noteTitles = clinicalEventDataService.getMdwsNoteTitles(facilityCode, searchIndex);
		
		return noteTitles;
	}
	
	/**
	 * This will query MDWS to see if the note-title with the given ien is of type Consult.
	 * 
	 * @param facilityCode The siteId for the facility.  This value is currently ignored (as we get it from the logged in user).
	 * @param staffId The id of the staff member.  This value is currently ignored.
	 * @param ien the ien of the note-title that you want to see if it is a consult note-title.
	 * @return JSON containing a &quot;result&quot; with a value of either &quot;true&quot; or &quot;false&quot;.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-title/is-consult/ien/{ien}")
	public Response isNoteTitleConsult(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("ien") String ien) {
		boolean result = clinicalEventDataService.isConsultNote(ien);
		
		return Response.status(Status.OK).entity("{\"result\":" + result + "}").type(MediaType.APPLICATION_JSON).build();
	}
	
	/**
	 * This will query MDWS to see if the note-title with the given ien is of type One Visit.
	 * 
	 * @param facilityCode The siteId for the facility.  This value is currently ignored (as we get it from the logged in user).
	 * @param staffId The id of the staff member.  This value is currently ignored.
	 * @param patientId the id of the patient
	 * @param noteDefinitionIEN the ien of the note-title.
	 * @param noteTitle the title of the note (there could be multiple with the same ien so we need to specify which one)
	 * @param locationIen the location
	 * @param vistaTimestamp the timestamp
	 * @param serviceCategory 'A' (ambulatory), 'H' (hospitalization), or 'E' (event historical)
	 * @return JSON containing a &quot;result&quot; with a value of either &quot;true&quot; or &quot;false&quot;.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/note-title/is-one-visit-note/ien/{ien}/noteTitle/{noteTitle}/locationIen/{locationIen}/vistaTimestamp/{vistaTimestamp}/serviceCategory/{serviceCategory}")
	public Response isNoteTitleOneVisitNote(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
			@PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId,
			@PathParam("patient-id") String patientId, @PathParam("ien") String noteDefinitionIEN,
			@PathParam("noteTitle") String noteTitle, @PathParam("locationIen") String locationIen,
			@PathParam("vistaTimestamp") String vistaTimestamp, @PathParam("serviceCategory") String serviceCategory) {
		boolean result = clinicalEventDataService.isOneVisitNote(patientId, noteDefinitionIEN, noteTitle, locationIen, vistaTimestamp, serviceCategory);
		
		return Response.status(Status.OK).entity("{\"result\":" + result + "}").type(MediaType.APPLICATION_JSON).build();
	}
	
	/**
	 * This will query MDWS to see if the note-title with the given ien is of type Surgery Note.
	 * 
	 * @param facilityCode The siteId for the facility.  This value is currently ignored (as we get it from the logged in user).
	 * @param staffId The id of the staff member.  This value is currently ignored.
	 * @param ien the ien of the note-title that you want to see if it is a Surgery Note.
	 * @return JSON containing a &quot;result&quot; with a value of either &quot;true&quot; or &quot;false&quot;.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-title/is-surgery-note/ien/{ien}")
	public Response isNoteTitleSurgeryNote(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("ien") String ien) {
		boolean result = clinicalEventDataService.isPrfNote(ien);
		
		return Response.status(Status.OK).entity("{\"result\":" + result + "}").type(MediaType.APPLICATION_JSON).build();
	}
	
	/**
	 * This will query MDWS to see if the note-title with the given ien is of type Patient Record Flag.
	 * 
	 * @param facilityCode The siteId for the facility.  This value is currently ignored (as we get it from the logged in user).
	 * @param staffId The id of the staff member.  This value is currently ignored.
	 * @param ien the ien of the note-title that you want to see if it is a patient record flag note-title.
	 * @return JSON containing a &quot;result&quot; with a value of either &quot;true&quot; or &quot;false&quot;.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-title/is-patient-record-flag/ien/{ien}")
	public Response isNoteTitlePatientRecordFlag(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, @PathParam("ien") String ien) {
		boolean result = clinicalEventDataService.isPrfNote(ien);
		
		return Response.status(Status.OK).entity("{\"result\":" + result + "}").type(MediaType.APPLICATION_JSON).build();
	}
	
	/**
	 * Retrieves the last NoteTitle that was selected by this staff member at this facility.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @return JSON containing the NoteTitle that is the last note title selected by this staff member at this facility.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/note-titles/last-selected-progress-note-title")
	public NoteTitle getLastSelectedNoteTitle(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId) {
		NoteTitle noteTitle = clinicalEventDataService.getLastSelectedNoteTitle(facilityCode, staffId);
		
		return noteTitle;
	}

	/**
	 * This sends the ClinicalEvent to VistA through MDWS.  First, we save or update the clinical event in mongo, then
	 * the clinicalEvent is sent to MDWS.<br/>
	 * At any point, we may receive a number of errors (400, 403, 500).  In most cases, these should contain JSON with a
	 * message, uuid, and detailedDescription.  The message is user friendly while the detailedDescription is more
	 * technical.  The uuid is there so you can look it up in the log files (so you can easily get to the stack trace
	 * of an error).
	 * 
	 * @throws javax.ws.rs.WebApplicationException.WebApplicationException Bad Request (400 error) if it is not a
	 * valid signable ClinicalEvent.
	 * @throws com.agilex.healthcare.mdws.datalayer.MdwsResponseException (400, 403, 500 error) could be returned from
	 * our communication with MDWS.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param noteSignature This is a ClinicalEvent with the eSig of the provider. 
	 * @return JSON containing the signed clinical event.  This will include the Date it was signed as well as having
	 * the status changed to SIGNED.
	 */
	@POST
	@Consumes({ "application/xml", "application/json"  })
	@Produces({ "application/xml", "application/json"  })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/clinical-events/sign")
	public ClinicalEvent signClinicalEvent(@Context UriInfo uriInfo, 
												    @Context HttpServletRequest request, 
												    @Context HttpHeaders headers, 
												    @PathParam("facility-code") String facilityCode, 
												    @PathParam("staff-id") String staffId, 
												    @PathParam("assigning-authority") String assigningAuthority, 
												    @PathParam("patient-id") String patientId, 
												    ClinicalNoteSignature noteSignature) {
		ClinicalEvent savedClinicalEvent = null;
		if (NullChecker.isNullish(noteSignature.getClinicalEvent().getId()))
			savedClinicalEvent = clinicalEventDataService.saveClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, noteSignature.getClinicalEvent());
		else
			savedClinicalEvent = clinicalEventDataService.updateClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, noteSignature.getClinicalEvent());

		ClinicalEvent signedClinicalEvent = clinicalEventDataService.signClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, savedClinicalEvent.getId(), noteSignature);
		ClinicalEventLinkBuilder linkBuilder = new ClinicalEventLinkBuilder(uriInfo.getBaseUri());
		linkBuilder.fillLinks(signedClinicalEvent);
		
		return signedClinicalEvent;
	}
}
