package com.agilex.healthcare.notewriter.restservice;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.Modifiers;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitRelatedTos;
import com.agilex.healthcare.mobilehealthplatform.domain.VisitTypes;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndex;
import com.agilex.healthcare.mobilehealthplatform.mdws.SearchIndexFactory;
import com.agilex.healthcare.notewriter.datalayer.SearchFilter;
import com.agilex.healthcare.notewriter.datalayer.SearchFilterFactory;
import com.agilex.healthcare.notewriter.datalayer.diagnosis.DiagnosisDataService;
import com.agilex.healthcare.notewriter.datalayer.facilityprovider.FacilityProviderDataService;
import com.agilex.healthcare.notewriter.datalayer.procedure.ProcedureDataService;
import com.agilex.healthcare.notewriter.datalayer.visitrelatedto.VisitRelatedToDataService;
import com.agilex.healthcare.notewriter.datalayer.visittype.VisitTypesDataService;
import com.agilex.healthcare.notewriter.domain.EncounterCodes;
import com.agilex.healthcare.notewriter.domain.EncounterDiagnosisICDCodes;

/**
 * REST Services for working with Encounters such as retrieving diagnoses, procedures, procedure modifiers,
 * the visit types, and providers at the facility.  Also includes patient specific REST services such as retrieving
 * the active medical diagnoses (patient problem list) and the Service Connected Related To Conditions (visitRelatedTo).
 */
@Path("/note-writer/encounter-form-service/")
@Component
@Scope("request")
public class EncounterFormResource {

	/**
	 * Retrieves Diagnoses ICD-9 Codes from MDWS.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param searchString The name (or partial name) of the Diagnoses that you want returned.
	 * @param maxRecords The maximum number of records you want returned by the search after filtering.
	 * @return The EncounterCodes that match the searchString passed in.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/diagnoses")
	public EncounterCodes findDiagnosisCodes(@PathParam("facility-code") String facilityCode, 
									 @QueryParam("searchstring") String searchString, 
									 @QueryParam("maxrecords") Integer maxRecords, 
									 @Context UriInfo uriInfo, 
									 @Context HttpHeaders headers) {
		
		SearchFilterFactory filterFactory = new SearchFilterFactory();
		SearchFilter filter = filterFactory.createFromUri(uriInfo.getRequestUri());
		
		EncounterCodes codes = findDiagnosisCodes(facilityCode, filter);
		return codes;
	}
	private EncounterCodes findDiagnosisCodes(String facilityCode, SearchFilter filter){
		DiagnosisDataService dataService = new DiagnosisDataService();
		return dataService.findCodes(facilityCode, filter);
	}
	
	/**
	 * Retrieves the active medical diagnoses for the patient.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param visitDate (optional) If this is not passed in, then all active medical diagnoses are returned.
	 * @return the active medical diagnoses for the patient.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/problem-list-diagnoses")
	public EncounterDiagnosisICDCodes getPatientProblemListDiagnoses(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
														@PathParam("facility-code") String facilityCode,
														@PathParam("staff-id") String staffId,
														@PathParam("assigning-authority") String assigningAuthority,
														@PathParam("patient-id") String patientId,
														@QueryParam("visitDate") String visitDate) {
		DiagnosisDataService dataService = new DiagnosisDataService();
		EncounterDiagnosisICDCodes codes = dataService.getPatientProblemListDiagnoses(facilityCode, patientId, visitDate);
		return codes;
	}
		
	/**
	 * Retrieves Procedures (CPT-4) Codes from MDWS.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param searchString The name (or partial name) of the Procedures that you want returned.
	 * @param maxRecords The maximum number of records you want returned by the search after filtering.
	 * @return The EncounterCodes that match the searchString passed in.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/procedures")
	public EncounterCodes findProcedureCodes(@PathParam("facility-code") String facilityCode, 
									 @QueryParam("searchstring") String searchString, 
									 @QueryParam("maxrecords") Integer maxRecords, 
									 @Context UriInfo uriInfo, 
									 @Context HttpHeaders headers) {
		
		SearchFilterFactory filterFactory = new SearchFilterFactory();
		SearchFilter filter = filterFactory.createFromUri(uriInfo.getRequestUri());
		
		EncounterCodes codes = findProcedureCodes(facilityCode, filter);
		
		return codes;
	}
	
	private EncounterCodes findProcedureCodes(String facilityCode, SearchFilter filter){
		ProcedureDataService dataService = new ProcedureDataService();
		return dataService.findCPTCodes(facilityCode, filter);
	}

	/**
	 * Retrieves CPT-4 Code Modifiers for the given cptCode at this facility.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param cptCode The cptCode that you want to retrieve the Modifiers of.
	 * @return CPT-4 Code Modifiers for the given cptCode at this facility.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/modifiers")
	public Modifiers getModifiersByCptCode(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @QueryParam("cpt-code") String cptCode) {
		ProcedureDataService dataService = new ProcedureDataService();
		return dataService.getModifiers(facilityCode, cptCode);
	}
	
	/**
	 * Get VisitTypes for this facility at the given location
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param locationIen The IEN of the Hospital Location file (#44) for the selected Visit.
	 * @param timestamp (optional)
	 * @return VisitTypes for this facility at the given location.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/visit-types")
	public VisitTypes getVisitTypes(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
									@PathParam("facility-code") String facilityCode,
									@QueryParam("location-ien") String locationIen,
									@QueryParam("timestamp") String timestamp) {
		VisitTypesDataService dataService = new VisitTypesDataService();
		return dataService.getVisitTypes(facilityCode, locationIen, timestamp);
	}
	
	/**
	 * Retrieves all of the FacilityProviders for the facility that contain (or partially contain) the parameter searchstring.<br/>
	 * <br/>
	 * 
	 * <font color="red">NOTE: This currently accesses the mongo database to retrieve facility providers and performs
	 * filtering based on that.  The functionality differs from the way CPRS behaves.  It also means the TableSync
	 * must be run to populate the Providers in the mongo database.  Because you are accessing a real database, you can
	 * perform filtering and other database operations.  You are not limited to the interface or the way that MDWS
	 * returns data (indexed like a phone book).  However, it also means you don't have the most current data.</font>
	 * <br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>the name (or partial name) of the results that you want returned.<br/>
	 * 			If searchstring is not supplied, then all results will be returned.
	 * 		</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>maxrecords</td>
	 * 		<td>the maximum number of records you want returned by the search after filtering.
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @return JSON containing a collection of all of the NoteTitles which match or partially match your searchstring as
	 * contained in the Mongo database.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/providers")
	public FacilityProviders getFacilityProviders(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		SearchFilterFactory filterFactory = new SearchFilterFactory();
		SearchFilter filter = filterFactory.createFromUri(uriInfo.getRequestUri());
		FacilityProviders providers = findFacilityProviders(facilityCode, filter);		 
		return providers;
	}
	
	private FacilityProviders findFacilityProviders(String facilityCode, SearchFilter filter){
		FacilityProviderDataService dataService = new FacilityProviderDataService();
		return dataService.findFacilityProviders(facilityCode, filter);
	}

	/**
	 * Returns the FacilityProviders starting at searchIndex using MDWS services and returning data following the same behavior as CPRS.
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>searchstring</td>
	 * 		<td>
	 * 			The searchIndex should be thought of as being identical to looking up a name in a telephone book.
	 * 			When you do a lookup in a phone book you pick someone's name and then start scanning down the list of names
	 * 			from there.  When you perform a lookup here and you receive results, it is like opening the phone book and picking
	 * 			a name in it.  It just selects the first item that matches closest to the one you supplied and then it returns
	 * 			you the rest of the list of all items from that point to the end of the list.  Thus, this is not filtering
	 * 			anything but is providing you a way to start your list at a point somewhere within the entire list.<br/><br/>
	 * 
	 * 			If searchIndex.getSearchString() is null or the empty string, this will return you ONLY the first 44 records.<br/><br/>
	 * 
	 * 			When it comes to anything that involves pagination in MDWS, if there are more than 44 results, we get back
	 * 			exactly 44 records. At that point, you should call the exact same MDWS call again passing in the value from the
	 * 			last record (the 44th record). This would continue until we receive less than 44 records.  Remember the
	 * 			telephone book analogy - this is like going down 44 records in that list and returning 44 more.<br/><br/>
	 * 
	 * 			<font color="red"><b>Example</b></font><br/>
	 * 			Suppose you had the following set of data: {'benefit', 'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bl', you'd get back {'bladder', 'bones', 'brain'}.<br/>
	 * 			If you searched for 'bm', you'd get back {'bones', 'brain'}.  This is the first index of data that occurs after
	 * 			what you searched for.<br/><br/>
	 * 		</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param facilityCode The siteId for the facility.  <font color="red">NOTE: This is no longer used as we can get the information
	 * from the logged on user.</font>
	 * @return JSON containing the FacilityProviders (up to 44 at a time)
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/mdws-providers")
	public FacilityProviders getMdwsFacilityProviders(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode) {
		SearchIndexFactory filterFactory = new SearchIndexFactory();
		SearchIndex searchIndex = filterFactory.createFromUriDecoded(uriInfo.getRequestUri());
		FacilityProviderDataService dataService = new FacilityProviderDataService();
		FacilityProviders providers = dataService.findFacilityProviders(facilityCode, searchIndex);
		return providers;
	}
	
	/**
	 * Get the Service Connected Related To Conditions for a selected Patient
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @param locationIen The IEN of the Hospital Location file (#44) for the selected Visit.
	 * @param visitDate (optional) if passed in, it should be in the format "MM/dd/yyyy".
	 * @return the Service Connected Related To Conditions for a selected Patient
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/patient/{assigning-authority}/{patient-id}/visit-related-to")
	public VisitRelatedTos getVisitRelatedTos(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers,
			@PathParam("facility-code") String facilityCode,
			@PathParam("staff-id") String staffId,
			@PathParam("assigning-authority") String assigningAuthority,
			@PathParam("patient-id") String patientId,
			@QueryParam("location-ien") String locationIen,
			@QueryParam("visitdate") String visitDate) {
		VisitRelatedToDataService dataService = new VisitRelatedToDataService();
		
		VisitRelatedTos visitRelatedTos = dataService.getVisitRelatedTos(facilityCode, patientId, visitDate, locationIen);
		return visitRelatedTos;
	}
}
