package com.agilex.healthcare.notewriter.restservice;

import java.util.Calendar;
import java.util.Date;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilter;
import com.agilex.healthcare.mobilehealthplatform.domain.filter.datefilter.DateFilterFactory;
import com.agilex.healthcare.notewriter.datalayer.PatientAppointmentsDataService;

/**
 * REST Service to retrieve the appointments for a given patient.
 */
@Path("/note-writer/patient/{assigning-authority}/{patient-id}/appointments")
@Component
@Scope("request")
public class NoteWriterAppointmentsResource {
	
	/**
	 * Retrieves the appointments for the given patient.  If startDate and endDate are not specified the default
	 * behavior is to retrieve all appointments that are one year prior to today up to the end of today.
	 * All appointments returned are sorted in a descending order based on their appointment start date.<br/>
	 * <br/>
	 * 
	 * <h3>Additional Path Parameters</h3>
	 * <table border="1">
	 * 	<tr><th>name</th><th>description</th></tr>
	 * 	<tr>
	 * 		<td>startDate</td>
	 * 		<td>Filters before this date.</td>
	 * 	</tr>
	 * 	<tr>
	 * 		<td>endDate</td>
	 * 		<td>Filters after this date.</td>
	 * 	</tr>
	 * </table>
	 * 
	 * @param assigningAuthority The assigning authority that gave the patient their id.
	 * @param patientId The id of the patient.
	 * @return JSON containing the appointments for this patient.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("visits")
	public Appointments fetchAppointments(@Context UriInfo u, @Context HttpHeaders headers,
			@PathParam("assigning-authority") String assigningAuthority, @PathParam("patient-id") String patientId) {
		//http://192.168.25.149/mdws2/EmrSvc.asmx?op=getVisits
		DateFilter dateFilter = DateFilterFactory.createFilterFromUri(u.getRequestUri());
		Date startDate = dateFilter.getStartDate();
		Date endDate = dateFilter.getEndDate();
		if (startDate == null || endDate == null) {
			//Choose today at 1 second before midnight for the endDate.
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR_OF_DAY, 23);
			cal.set(Calendar.MINUTE, 59);
			cal.set(Calendar.SECOND, 59);
			endDate = cal.getTime();
			
			//Go back one year from today for the start date.
			cal.add(Calendar.YEAR, -1);
			cal.set(Calendar.HOUR_OF_DAY, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			startDate = cal.getTime();
			
			//Now, set the new dateFilter to that
			dateFilter = DateFilterFactory.createFilterFromDate(startDate, endDate);
		}
		PatientIdentifier patientIdentifier = new PatientIdentifier(assigningAuthority, patientId);
		PatientAppointmentsDataService dataService = new PatientAppointmentsDataService();
		Appointments retValue = dataService.getVisits(patientIdentifier, dateFilter);
		retValue.sortDescending("appointmentStartDate");
		return retValue;
	}
}
