package com.agilex.healthcare.patientsearch.restservice;

import java.net.URI;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriInfo;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.ResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.mobilehealthplatform.restservice.PatientsResource;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.PatientSearchResourceDirectoryBuilder;
import com.agilex.healthcare.mobilehealthplatform.utils.uriformaters.linkbuilder.PatientLinkBuilder;
import com.agilex.healthcare.patientsearch.datalayer.PatientSearchDataService;

/**
 * REST Services for searching for recently selected patients.<br/>
 * <font color="blue">For additional REST services for searching for patients, see {@link PatientsResource}.</font>
 * @see PatientsResource
 */
@Path("/patient-search/")
@Component
@Scope("request")
public class PatientSearchResource {

	@Resource
	PatientSearchResourceDirectoryBuilder patientSearchResourceDirectoryBuilder;
	
	/**
	 * Resource directory for searching for recently selected patients
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @return JSON containing the various resources that can be used.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/resource-directory")
	public ResourceDirectory fetchResourceDirectory(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId) {
		ResourceDirectory resourceDirectory = patientSearchResourceDirectoryBuilder.getResourceDirectory(uriInfo.getBaseUri(), uriInfo.getRequestUri(), facilityCode, staffId);
		return resourceDirectory;
	}
	
	/**
	 * Up to 20 recently selected patients can be returned for this staff member at this facility.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @return JSON a collection of recently selected patients by this staff member at this facility.
	 */
	@GET
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/recently-selected-patients")
	public RecentlySelectedPatients fetchRecentlySelectedPatients(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId) {
		RecentlySelectedPatients results = new RecentlySelectedPatients();
		
		PatientSearchDataService dataService = new PatientSearchDataService();
		results = dataService.fetchRecentlySelectedPatients(facilityCode, staffId);
		
		buildLinks(uriInfo.getBaseUri(), results);
		
		return results;
	}
	
	/**
	 * Saves this patient to the list of 20 patients that the staff member has last viewed at this facility.
	 * If the list of last viewed patients is already 20, then the oldest record will be removed and this one added.
	 * 
	 * @param facilityCode The siteId for the facility.
	 * @param staffId The id of the staff member.
	 * @param patient JSON for a Patient object.
	 * @return JSON for a SelectedPatient (the time it was searched for and the Patient)
	 */
	@POST
	@Consumes({ "application/xml", "application/json" })
	@Produces({ "application/xml", "application/json" })
	@Path("site/{facility-code}/id/{staff-id}/recently-selected-patients")
	public SelectedPatient addSelectedPatient(@Context UriInfo uriInfo, @Context HttpServletRequest request, @Context HttpHeaders headers, @PathParam("facility-code") String facilityCode, @PathParam("staff-id") String staffId, Patient patient) {
		PatientSearchDataService dataService = new PatientSearchDataService();
		SelectedPatient result = dataService.addSelectedPatient(facilityCode, staffId, patient);
		return result;
	}
	
	private void buildLinks(URI baseURI, RecentlySelectedPatients selectedPatients) {
		PatientLinkBuilder linkBuilder = new PatientLinkBuilder(baseURI);
		for (SelectedPatient searchedPatient : selectedPatients) {
			searchedPatient.getPatient().setSelfUri(linkBuilder.createPatientSelfUri(searchedPatient.getPatient().getPatientIdentifier()));
		}
	}
	
	
	
	
}
