package com.agilex.healthcare.mobilehealthplatform.restservice.patient;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.GraphData;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntries;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalEntry;
import com.agilex.healthcare.mobilehealthplatform.domain.VitalTableData;
import com.agilex.healthcare.mobilehealthplatform.domain.code.VitalSectionCode;
import com.agilex.healthcare.testutility.IdentifierUtil;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientVitalResourceTest {
	private static final String OPERATIONAL = "operational";
	private static final String PATIENT_ID = "1";
	private static final String ASSIGNING_AUTHORITY = "EDIPI";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void fetchVitalEntriesByScope() {
		try {
			URI uri = getVitalEntriesByScopeUri("dfn-777", "774", OPERATIONAL);
			VitalEntries obj = client.getJerseyClient().resource(uri).get(VitalEntries.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchVitalEntry() {
		try {
			//Grab all of the vital entries for this patient.
			URI uri = getVitalEntriesByScopeUri("dfn-777", "774", OPERATIONAL);
			VitalEntries vitalEntries = client.getJerseyClient().resource(uri).get(VitalEntries.class);
			assertNotNull(vitalEntries);
			assertTrue(vitalEntries.size() > 0);
			
			//Now, take the first entry and grab just that entry - validate that it matches.
			VitalEntry expected = vitalEntries.get(0);
			uri = getVitalEntryUri("dfn-777", "774", "vista", expected.getId());
			VitalEntry actual = client.getJerseyClient().resource(uri).get(VitalEntry.class);
			assertNotNull(actual);
			assertEquals(expected.getId(), actual.getId());
			assertEquals(expected.getAssigningAuthority(), actual.getAssigningAuthority());
			IdentifierUtil.assertDataIdentifier(expected.getDataIdentifier(), actual.getDataIdentifier());
			assertEquals(expected.getDate(), actual.getDate());
			assertEquals(expected.getEntryDate(), actual.getEntryDate());
			assertEquals(expected.getNotes(), actual.getNotes());
			assertEquals(expected.getPatientId(), actual.getPatientId());
			assertEquals(expected.getQualifier(), actual.getQualifier());
			assertEquals(expected.getSection(), actual.getSection());
			assertEquals(expected.getSourceSystem(), actual.getSourceSystem());
			assertEquals(expected.getSystemId(), actual.getSystemId());
			assertEquals(expected.getUniqueId(), actual.getUniqueId());
			IdentifierUtil.assertPatientIdentifier(expected.getPatientIdentifier(), actual.getPatientIdentifier());
			assertNotNull(expected.getVitalObservations());
			assertNotNull(actual.getVitalObservations());
			assertEquals(expected.getVitalObservations().size(), actual.getVitalObservations().size());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getVitalEntriesGraphData() {
		try {
			URI uri = getVitalEntriesGraphDataUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL, VitalSectionCode.BloodPressure);
			GraphData obj = client.getJerseyClient().resource(uri).get(GraphData.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getVitalEntriesAsTableData() {
		try {
			URI uri = getVitalEntriesAsTableDataUri(ASSIGNING_AUTHORITY, PATIENT_ID, OPERATIONAL);
			VitalTableData obj = client.getJerseyClient().resource(uri).get(VitalTableData.class);
			assertNotNull(obj);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getVitalEntriesByScopeUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/vitals/scope/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getVitalEntryUri(String assigningAuthority, String patientId, String systemId, String vitalId) {
		String uri = String.format("%s/patient/%s/%s/vitals/system/%s/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, systemId, vitalId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getVitalEntriesGraphDataUri(String assigningAuthority, String patientId, String scope, String graphType) {
		String uri = String.format("%s/patient/%s/%s/vitals/scope/%s/graphdata?graphType=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope, graphType);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getVitalEntriesAsTableDataUri(String assigningAuthority, String patientId, String scope) {
		String uri = String.format("%s/patient/%s/%s/vitals/scope/%s/tabledata", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), assigningAuthority, patientId, scope);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
