package com.agilex.healthcare.mobilehealthplatform.restservice.patients;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientSearchResults;
import com.agilex.healthcare.mobilehealthplatform.domain.Patients;
import com.agilex.healthcare.mobilehealthplatform.restservice.GeneralResourceTestBase;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;

/**
 * THE FUNCTIONALITY TESTED HERE IS NOT THE SAME FUNCTIONALITY CPRS HAS - FOR EXAMPLE, THE FILTERING THEY ARE DOING
 * FOR NAME SEARCHES FILTER AGAINST THE 44 RESULTS RETURNED FROM MDWS (NOT THE ENTIRE COLLECTION OF NAMES).
 * THE FUNCTIONALITY FOR HOW THEY ARE PERFORMING SEARCH IS FLAWED AND THESE TESTS WILL NEED TO BE ALTERED COMPLETELY
 * WHEN THE SEARCH FUNCTIONALITY IS CHANGED TO WORK LIKE CPRS.<br/><br/>
 * 
 * <font color="red">NOTE: The search functionality provided by these REST services differs from that of CPRS. These
 * differences may be removed in the future to make this behave just like CPRS (so you should not rely on them staying
 * this way).</font><br/>
 * Following is an explanation of the known differences at this time.<br/>
 * 1.	There is a special filter of the patients that are returned from mdws to ensure that the criteria passed in
 * matches exactly what is returned.<br/>
 * &nbsp;&nbsp;&nbsp;a.	This is the comment in the code: "MDWS returns more patients than we expect due to it's loose
 * search criteria ensure first and last names contain the appropriate search criteria"<br/>
 * When that comment was written, it appears they didn't understand that CPRS behaves more like a telephone book.
 * When you pass in a filter such as "lastName" or "lastName,firstName" to CPRS, it is the same as opening up the phone
 * book to that specific page and then listing all the rest of the names in the book.  When you pass in
 * lastInitialLast4Ssn, it returns you data that is filtered to only include those that match.<br/>
 * 2.	The data is reformatted from how it is received by MDWS<br/>
 * &nbsp;&nbsp;&nbsp;a.	Examples include adding spaces between lastName,FirstName,MiddleName for the displayName, adding
 * hyphens between digits for ssn, changing gender from a single character (m or f) to MALE or FEMALE, populating a
 * middle initial, populating their age, etc.<br/>
 * 3.	The collection of Patients returned from MDWS is sorted.<br/>
 * &nbsp;&nbsp;&nbsp;a.	The results are sorted by lastName, firstName, dateOfBirth, and ssn.<br/>
 * 4.	We remove any results past our max results (which is currently 40).<br/>
 * <font color="red">NOTE: Because these cause search functionality different from that of CPRS, some or all of these
 * differences may be changed at some time in the future.<br/><br/>
 * 
 * Some additional notes.  Wards coming from MDWS are not filtered.  You receive all of them.  If this changes to behave
 * like CPRS, we should change the UI to receive all of the wards and perform filtering on their end if needed - the
 * backend should not be the one filtering.<br/>
 * Clinics search is returning data in the mdws format (44 records at a time), the searchString passed in is the index
 * closest to where you want to start returning records from (like pointing to a name in a phone book).  The filtering
 * and other stuff they are doing here is not guaranteed to be accurate as you are only looking at a very small subset
 * of the data.
 * </font>
 *
 */
public class PatientsResourceTest extends GeneralResourceTestBase {
	private static final String WARD = "38"; //7a gen med
	private static final String CLINIC = "195"; //cardiology
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@Override
	protected String getResourcePath() {
		return IntegrationTestConfiguration.getMobileHealthServerBaseUri() + "/patients?lastname=younger";
	}

	@BeforeClass
	public static void setup() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@AfterClass
	public static void destroy() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
//-----------------------------------Search-----------------------------------------------------------------------------
//	NOTE: There are 5 types of search that can be performed here.  "lastNameOnly" or "lastName,firstName" or "lastName,firstName middleName" or "ssn" or finally "lastInitialLast4Ssn"
	
	@Test
	public void searchLastNameOnly() {
		try {
			URI uri = getSearchUri("ten");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertTrue("Expected to find at least one patient", patients.size() > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchLastNameFirstName() {
		try {
			URI uri = getSearchUri("ten%2Cpatient");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("TEN, PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchLastNameFirstNameMiddleName() {
		try {
			URI uri = getSearchUri("bcma%2Ceighteen%20patient");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("BCMA, EIGHTEEN-PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchSsn() {
		try {
			URI uri = getSearchUri("666-33-0018");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("BCMA, EIGHTEEN-PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchSsnNoDashes() {
		try {
			URI uri = getSearchUri("666330018");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("BCMA, EIGHTEEN-PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchLastInitialLast4Ssn() {
		try {
			URI uri = getSearchUri("B0018");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("BCMA, EIGHTEEN-PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchLastInitialLast4SsnLowercase() {
		try {
			URI uri = getSearchUri("b0018");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			
			assertNotNull(patients);
			assertEquals(1, patients.size());
			assertEquals("BCMA, EIGHTEEN-PATIENT", patients.get(0).getDisplayName());
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
//-----------------------------------Clinic-----------------------------------------------------------------------------
	
	@Test
	public void searchByClinic() {
		try {
			URI uri = getSearchByClinicUri(CLINIC, "20140203");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int patientsSize = patients.size();
			assertTrue(patientsSize > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchByClinicNoResults() {
		try {
			URI uri = getSearchByClinicUri(CLINIC, "20140303"); //March instead of February
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int patientsSize = patients.size();
			assertEquals(0, patientsSize);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchByClinicFiltered() {
		try {
			URI uri = getSearchByClinicUri(CLINIC, "20140203", "BCMA");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int patientsSize = patients.size();
			assertTrue(patientsSize > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchByClinicFilteredSmaller() {
		try {
			URI uri = getSearchByClinicUri(CLINIC, "20140203");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int all = patients.size();
			
			uri = getSearchByClinicUri(CLINIC, "20140203", "BCMA");
			patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int filtered = patients.size();
			
			assertTrue(all > filtered);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
//-----------------------------------Ward-------------------------------------------------------------------------------
	
	@Test
	public void searchByWard() {
		try {
			URI uri = getSearchByWardUri(WARD, null);
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int numPatients = patients.size();
			assertTrue(numPatients > 0);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchByWardFiltered() {
		try {
			URI uri = getSearchByWardUri(WARD, "eight");
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int numPatients = patients.size();
			assertTrue(numPatients > 0);
			for (Patient patient : patients) {
				assertTrue(patient.getLastName().toLowerCase().contains("eight"));
			}
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void searchByWardFilteredSmaller() {
		try {
			URI uri = getSearchByWardUri(WARD, null);
			Patients patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int all = patients.size();
			
			uri = getSearchByWardUri(WARD, "eight");
			patients = client.getJerseyClient().resource(uri).get(PatientSearchResults.class);
			assertNotNull(patients);
			int filtered = patients.size();
			assertTrue(all > filtered);
		} catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private URI getSearchUri(String searchString) {
		StringBuilder sb = new StringBuilder(IntegrationTestConfiguration.getMobileHealthServerBaseUri().toString());
		sb.append("/patients?searchstring=");
		sb.append(searchString);
		return UriBuilder.fromUri(sb.toString()).build();
	}
	
	private URI getSearchByClinicUri(String clinic, String visitDate) {
		URI uri = getSearchByClinicUri(clinic, visitDate, null);
		return uri;
	}
	private URI getSearchByClinicUri(String clinic, String visitDate, String searchString) {
		StringBuilder sb = new StringBuilder(IntegrationTestConfiguration.getMobileHealthServerBaseUri().toString());
		sb.append("/patients?clinic=");
		sb.append(clinic);
		if (NullChecker.isNotNullish(visitDate)) {
			sb.append("&visitDate=");
			sb.append(visitDate);
		}
		if (NullChecker.isNotNullish(searchString)) {
			sb.append("&searchstring=");
			sb.append(searchString);
		}
		return UriBuilder.fromUri(sb.toString()).build();
	}
	
	private URI getSearchByWardUri(String ward, String searchString) {
		StringBuilder sb = new StringBuilder(IntegrationTestConfiguration.getMobileHealthServerBaseUri().toString());
		sb.append("/patients?ward=");
		sb.append(ward);
		if (NullChecker.isNotNullish(searchString)) {
			sb.append("&searchstring=");
			sb.append(searchString);
		}
		return UriBuilder.fromUri(sb.toString()).build();
	}
}
