package com.agilex.healthcare.notewriter.restservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Date;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEvent;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalEventState;
import com.agilex.healthcare.mobilehealthplatform.domain.ClinicalNoteSignature;
import com.agilex.healthcare.mobilehealthplatform.domain.Count;
import com.agilex.healthcare.mobilehealthplatform.domain.Encounter;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.EncounterProviders;
import com.agilex.healthcare.mobilehealthplatform.domain.FacilityProvider;
import com.agilex.healthcare.mobilehealthplatform.domain.HealthFactor;
import com.agilex.healthcare.mobilehealthplatform.domain.HealthFactors;
import com.agilex.healthcare.mobilehealthplatform.domain.Note;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteTitle;
import com.agilex.healthcare.mobilehealthplatform.domain.NoteWriterResourceDirectory;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.ProviderIdentifier;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCode;
import com.agilex.healthcare.notewriter.domain.EncounterProcedureCPTCodes;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.api.client.WebResource;

public class ClinicalEventResourceTest {
	private static final String SITE = "777";
	private static final String STAFF_ID_FOR_MDWS = "1";
	private static final String ESIG = "CPRS1234";
	private static final String FACILITY_CODE_DFN = "dfn-777";
	private static final String STAFF_ID = "CPRS1234";
	private static final String PATIENT_ID = "1";
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void checkWhetherTestsAreRunningInCIHMode() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void tearDown() {
	    if (client != null) {
	    	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	@Test
	public void addClinicalEventThenFetch() {
		try {
			String facilityCode = SITE;
			String staffId = STAFF_ID;
			String assigningAuthority = FACILITY_CODE_DFN;
			String patientId = PATIENT_ID;
			NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
			String expectedNoteTitle = "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>";
			String noteBody = "RESOURCE TEST BODY";

			Note note = createNote(noteTitle, noteBody);
			Encounter encounter = new Encounter();
			ClinicalEvent clinicalEvent = createClinicalEvent(facilityCode, new ProviderIdentifier(facilityCode, staffId), new PatientIdentifier(assigningAuthority, patientId), encounter, note);
			
			URI uri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalEvent savedClinicalEvent = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, clinicalEvent);
			
			URI clinicalEventUri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId, savedClinicalEvent.getId());
			ClinicalEvent fetchedClinicalEvent = client.getJerseyClient().resource(clinicalEventUri).get(ClinicalEvent.class);
			
			assertEquals(assigningAuthority, fetchedClinicalEvent.getPatientIdentifier().getAssigningAuthority());
			assertEquals(patientId, fetchedClinicalEvent.getPatientIdentifier().getUniqueId());
			
			assertEquals(facilityCode, fetchedClinicalEvent.getProviderIdentifier().getFacilityCode());
			assertEquals(staffId, fetchedClinicalEvent.getProviderIdentifier().getStaffId());
			
			assertEquals(noteTitle.getIen(), fetchedClinicalEvent.getNote().getTitle().getIen());
			assertEquals(expectedNoteTitle, fetchedClinicalEvent.getNote().getTitle().getName());
			assertEquals(noteTitle.isConsult(), fetchedClinicalEvent.getNote().getTitle().isConsult());
			assertEquals(noteTitle.isPrf(), fetchedClinicalEvent.getNote().getTitle().isPrf());
			assertEquals(noteBody, fetchedClinicalEvent.getNote().getBody());
			assertEquals(savedClinicalEvent.getId(), fetchedClinicalEvent.getId());
			
			//Validate that the object that we fetch back matches the original save with a new id and modified date.
			clinicalEvent.setId(fetchedClinicalEvent.getId()); //Make the id's match.
			clinicalEvent.setLastModifiedDate(fetchedClinicalEvent.getLastModifiedDate()); //Make the last modified date match.
			clinicalEvent.getNote().getTitle().setName(expectedNoteTitle);
			assertEquals(clinicalEvent, fetchedClinicalEvent);
			
			testLastSelectedTitle(facilityCode, staffId, noteTitle);
			testLatestClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEvent);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void addClinicalEventThenSaveThenSign() {
		try {
			String facilityCode = SITE;
			String staffId = STAFF_ID;
			String assigningAuthority = FACILITY_CODE_DFN;
			String patientId = PATIENT_ID;
			NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
			String noteBody = "RESOURCE TEST BODY";
			String electronicSignatureCode = ESIG;
			
			Note note = createNote(noteTitle, noteBody);
			Encounter encounter = new Encounter();
			ClinicalEvent clinicalEvent = createClinicalEvent(facilityCode, new ProviderIdentifier(facilityCode, staffId), new PatientIdentifier(assigningAuthority, patientId), encounter, note);
			clinicalEvent.getEncounter().setLocationId("285");
			clinicalEvent.getEncounter().setServiceCategory("H");
			
			EncounterProviders providers = new EncounterProviders();
			providers.add(createProvider("5", false));
			providers.add(createProvider("7", false));
			providers.add(createProvider("9", false));
			//In order for doc123 to sign, they must be the primary provider (first provider in the list sent to mdws).
			//This tests the ClinicalNoteTranslator.translateToEncounterNote ability to put doc123 first in the list sent to mdws.
			providers.add(createProvider(STAFF_ID_FOR_MDWS, true));
			clinicalEvent.getEncounter().setProviders(providers);
			
			EncounterProcedureCPTCodes procedureCodes = new EncounterProcedureCPTCodes();
			procedureCodes.add(createProcedureCode("23420", "Reconstruction of Complete Shoulder (Rotator) Cuff Avulsion, Chronic (includes Acromioplasty)"));
			clinicalEvent.getEncounter().setProcedureCodes(procedureCodes);
			
			URI uri = getNoteWriterSignClinicalEventUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalNoteSignature cns = new ClinicalNoteSignature();
			cns.setClinicalEvent(clinicalEvent);
			cns.setElectronicSignature(electronicSignatureCode);
			ClinicalEvent signedClinicalEvent = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, cns);
			
			assertNotNull(signedClinicalEvent.getNote().getSignedDate());
			
			assertEquals(ClinicalEventState.SIGNED, signedClinicalEvent.getClinicalEventState());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void addClinicalEventThenSign() {
		testClinicalEventThenSign(false);
	}

	@Test
	public void addClinicalEventWithHealthFactorsThenSign() {
		testClinicalEventThenSign(true);
	}
	
	private void testClinicalEventThenSign(boolean includeHealthFactors) {
		try {
			String facilityCode = SITE;
			String staffId = STAFF_ID;
			String assigningAuthority = FACILITY_CODE_DFN;
			String patientId = PATIENT_ID;
			NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
			String noteBody = "RESOURCE TEST BODY";
			String electronicSignatureCode = ESIG;

			Note note = createNote(noteTitle, noteBody);
			Encounter encounter = new Encounter();
			ClinicalEvent clinicalEvent = createClinicalEvent(facilityCode, new ProviderIdentifier(facilityCode, staffId), new PatientIdentifier(assigningAuthority, patientId), encounter, note);
			clinicalEvent.getEncounter().setLocationId("285");
			clinicalEvent.getEncounter().setServiceCategory("H");
			if (includeHealthFactors)
				clinicalEvent.setHealthFactors(getListOfHealthFactors());

			EncounterProviders providers = new EncounterProviders();
			providers.add(createProvider("5", false));
			providers.add(createProvider("7", false));
			providers.add(createProvider("9", false));
			//In order for doc123 to sign, they must be the primary provider (first provider in the list sent to mdws).
			//This tests the ClinicalNoteTranslator.translateToEncounterNote ability to put doc123 first in the list sent to mdws.
			providers.add(createProvider(STAFF_ID_FOR_MDWS, true));
			clinicalEvent.getEncounter().setProviders(providers);

			EncounterProcedureCPTCodes procedureCodes = new EncounterProcedureCPTCodes();
			procedureCodes.add(createProcedureCode("23420", "Reconstruction of Complete Shoulder (Rotator) Cuff Avulsion, Chronic (includes Acromioplasty)"));
			clinicalEvent.getEncounter().setProcedureCodes(procedureCodes);

			URI uri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalEvent savedClinicalEvent = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, clinicalEvent);

			clinicalEvent.setId(savedClinicalEvent.getId());
			URI clinicalEventUri = getNoteWriterSignClinicalEventUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalNoteSignature cns = new ClinicalNoteSignature();
			cns.setClinicalEvent(clinicalEvent);
			cns.setElectronicSignature(electronicSignatureCode);

			ClinicalEvent signedClinicalEvent = client.getJerseyClient().resource(clinicalEventUri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, cns);

			assertNotNull(signedClinicalEvent);
			assertNotNull(signedClinicalEvent.getNote().getSignedDate());
			if (includeHealthFactors) {
				assertNotNull(savedClinicalEvent.getHealthFactors());
				assertTrue(savedClinicalEvent.getHealthFactors().size()>0);
			}

			assertEquals(ClinicalEventState.SIGNED, signedClinicalEvent.getClinicalEventState());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void addClinicalEventThenUpdate() {
		try {
			String facilityCode = SITE;
			String staffId = STAFF_ID;
			String assigningAuthority = FACILITY_CODE_DFN;
			String patientId = PATIENT_ID;
			NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
			String expectedNoteTitle = "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>";
			
			String noteBody = "RESOURCE TEST BODY";
			String newNoteBody = "NEW RESOURCE TEST BODY";

			Note note = createNote(noteTitle, noteBody);
			Encounter encounter = new Encounter();
			ClinicalEvent clinicalEvent = createClinicalEvent(facilityCode, new ProviderIdentifier(facilityCode, staffId), new PatientIdentifier(assigningAuthority, patientId), encounter, note);
			
			URI uri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalEvent savedClinicalEvent = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, clinicalEvent);
			
			//Validate that the object that we fetch back matches the original save with a new id and modified date.
			String backupId = clinicalEvent.getId();
			Date backupLastModifiedDate = clinicalEvent.getLastModifiedDate();
			clinicalEvent.setId(savedClinicalEvent.getId()); //Make the id's match.
			clinicalEvent.setLastModifiedDate(savedClinicalEvent.getLastModifiedDate()); //Make the last modified date match.
			clinicalEvent.getNote().getTitle().setName(expectedNoteTitle);
			assertEquals(clinicalEvent, savedClinicalEvent);
			clinicalEvent.getNote().getTitle().setName(noteTitle.getName());
			clinicalEvent.setId(backupId);
			clinicalEvent.setLastModifiedDate(backupLastModifiedDate);
			
			note.setBody(newNoteBody);
			clinicalEvent.setNote(note);
			URI noteUri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId, savedClinicalEvent.getId());
			ClinicalEvent updatedClinicalEvent = client.getJerseyClient().resource(noteUri).type("application/xml").accept("application/xml").put(ClinicalEvent.class, clinicalEvent);
			
			assertEquals(assigningAuthority, updatedClinicalEvent.getPatientIdentifier().getAssigningAuthority());
			assertEquals(patientId, updatedClinicalEvent.getPatientIdentifier().getUniqueId());
			
			assertEquals(facilityCode, updatedClinicalEvent.getProviderIdentifier().getFacilityCode());
			assertEquals(staffId, updatedClinicalEvent.getProviderIdentifier().getStaffId());
			
			assertEquals(noteTitle.getIen(), updatedClinicalEvent.getNote().getTitle().getIen());
			assertEquals(expectedNoteTitle, updatedClinicalEvent.getNote().getTitle().getName());
			assertEquals(noteTitle.isConsult(), updatedClinicalEvent.getNote().getTitle().isConsult());
			assertEquals(noteTitle.isPrf(), updatedClinicalEvent.getNote().getTitle().isPrf());
			
			assertEquals(newNoteBody, updatedClinicalEvent.getNote().getBody());
			assertEquals(savedClinicalEvent.getId(), updatedClinicalEvent.getId());

			//Validate that the object that we fetch back matches the original save with a new id and modified date.
			clinicalEvent.setId(updatedClinicalEvent.getId()); //Make the id's match.
			clinicalEvent.setLastModifiedDate(updatedClinicalEvent.getLastModifiedDate()); //Make the last modified date match.
			clinicalEvent.getNote().getTitle().setName(expectedNoteTitle);
			assertEquals(clinicalEvent, updatedClinicalEvent);
			
			testLastSelectedTitle(facilityCode, staffId, noteTitle);
			testLatestClinicalEvent(facilityCode, staffId, assigningAuthority, patientId, clinicalEvent);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void fetchResourceDirectory() {
		try {
			URI uri = getNoteWriterResourceDirectoryUri(SITE, "1", "EDIPI", "1");
			NoteWriterResourceDirectory dir = client.getJerseyClient().resource(uri).get(NoteWriterResourceDirectory.class);
			
			assertNotNull(dir);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void getClinicalEventsCount() {
		try {
			String facilityCode = SITE;
			String staffId = STAFF_ID;
			String assigningAuthority = FACILITY_CODE_DFN;
			String patientId = PATIENT_ID;
			NoteTitle noteTitle = new NoteTitle("1267", "PHARMACY  <TRANSITIONAL PHARMACY BENEFIT NOTE>", false, false);
			String noteBody = "RESOURCE TEST BODY";
			
			Note note = createNote(noteTitle, noteBody);
			Encounter encounter = new Encounter();
			ClinicalEvent clinicalEvent = createClinicalEvent(facilityCode, new ProviderIdentifier(facilityCode, staffId), new PatientIdentifier(assigningAuthority, patientId), encounter, note);
			
			URI uri = getNoteWriterClinicalEventsUri(facilityCode, staffId, assigningAuthority, patientId);
			ClinicalEvent savedClinicalEvent = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(ClinicalEvent.class, clinicalEvent);

			assertNotNull(savedClinicalEvent);
			
			uri = getNoteWriterClinicalEventsCountUri(facilityCode, staffId, assigningAuthority, patientId);
			Count count = client.getJerseyClient().resource(uri).get(Count.class);
			assertTrue(count.getValue() > 0);
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

//----------------------------------------------------------------------------------------------------------------------

	private HealthFactors getListOfHealthFactors(){
		HealthFactors healthFactors = new HealthFactors();
		
		HealthFactor healthFactor = new HealthFactor();
		healthFactor.setIen("660154");
		healthFactor.setHealthFactorCode("660154");
		healthFactor.setHealthFactorDescription("VANOD SKIN INITIAL");
		healthFactor.setParentCategory("660153");
		healthFactor.setCategory(false);
		healthFactors.add(healthFactor);
		
		healthFactor = new HealthFactor();
		healthFactor.setIen("612903");
		healthFactor.setHealthFactorCode("612903");
		healthFactor.setHealthFactorDescription("BRADEN SCALE 9 OR LOWER");
		healthFactor.setParentCategory("612715");
		healthFactor.setCategory(false);
		
		healthFactor = new HealthFactor();
		healthFactor.setIen("612747");
		healthFactor.setHealthFactorCode("612747");
		healthFactor.setHealthFactorDescription("SKIN COLOR");
		healthFactor.setParentCategory("612742");
		healthFactor.setCategory(false);
		healthFactor.setHealthFactorComment("Pale");
		
		return healthFactors;
	}

	private EncounterProcedureCPTCode createProcedureCode(String code, String description) {
		EncounterProcedureCPTCode encounterProcedureCPTCode = new EncounterProcedureCPTCode();
		encounterProcedureCPTCode.setCode(code);
		encounterProcedureCPTCode.setDescription(description + " (CPT-4" + code + ")");
		encounterProcedureCPTCode.setShortDescription(description);
		return encounterProcedureCPTCode;
	}

	private EncounterProvider createProvider(String staffId, boolean isPrimary) {
		EncounterProvider encounterProvider = new EncounterProvider();
		encounterProvider.setPrimary(isPrimary);
		encounterProvider.setSelected(true);
		
		FacilityProvider facilityProvider = new FacilityProvider();
		facilityProvider.setProviderIen(staffId);
		
		encounterProvider.setProvider(facilityProvider);
		return encounterProvider;
	}
		
	private Note createNote(NoteTitle title, String body) {
		Note note = new Note();
		note.setTitle(title);
		note.setBody(body);
		return note;
	}
	
	private ClinicalEvent createClinicalEvent(String facilityCode, ProviderIdentifier providerIdentifier, PatientIdentifier patientIdentifier, Encounter encounter, Note note) {
		ClinicalEvent clinicalEvent = new ClinicalEvent();
		
		clinicalEvent.setFacilityCode(facilityCode);
		clinicalEvent.setProviderIdentifier(providerIdentifier);
		clinicalEvent.setPatientIdentifier(patientIdentifier);
		clinicalEvent.setEncounter(encounter);
		clinicalEvent.setNote(note);
		
		return clinicalEvent;
	}
	
	private void testLastSelectedTitle(String facilityCode, String staffId, NoteTitle expectedTitle) {
		URI uri = ClinicalEventResourceNoteTitlesTest.getNoteWriterLastSelectedNoteTitleUri(facilityCode, staffId);
		WebResource resource = client.getJerseyClient().resource(uri);
		ClientResponse response = resource.get(ClientResponse.class);
		assertEquals(200, response.getStatus());
		NoteTitle title = resource.get(NoteTitle.class);
		assertNotNull(title);
		assertEquals(expectedTitle.getIen(), title.getIen());
		assertEquals(expectedTitle.getName(), title.getName());
		assertEquals(expectedTitle.isConsult(), title.isConsult());
		assertEquals(expectedTitle.isPrf(), title.isPrf());
		
	}
	
	private void testLatestClinicalEvent(String facilityCode, String staffId, String assigningAuthority, String patientId, ClinicalEvent expectedClinicalEvent) {
		URI uri = getNoteWriterLatestClinicalEventUri(facilityCode, staffId, assigningAuthority, patientId);
		ClinicalEvent latestClinicalEvent = client.getJerseyClient().resource(uri).get(ClinicalEvent.class);
		
		if (latestClinicalEvent.getLastModifiedDate() != null) {
			assertEquals(expectedClinicalEvent.getProviderIdentifier().getFacilityCode(), latestClinicalEvent.getProviderIdentifier().getFacilityCode());
			assertEquals(expectedClinicalEvent.getProviderIdentifier().getStaffId(), latestClinicalEvent.getProviderIdentifier().getStaffId());
			assertEquals(expectedClinicalEvent.getPatientIdentifier().getAssigningAuthority(), latestClinicalEvent.getPatientIdentifier().getAssigningAuthority());
			assertEquals(expectedClinicalEvent.getPatientIdentifier().getUniqueId(), latestClinicalEvent.getPatientIdentifier().getUniqueId());
			assertEquals(expectedClinicalEvent.getNote().getBody(), latestClinicalEvent.getNote().getBody());
			assertTrue(expectedClinicalEvent.getClinicalEventState() != ClinicalEventState.SIGNED);
		}
	}
	
//----------------------------------------------------------------------------------------------------------------------
	
	private URI getNoteWriterResourceDirectoryUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/resource-directory?assigningAuthority=%s&patientId=%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterLatestClinicalEventUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/clinical-events/latest", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterClinicalEventsUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/clinical-events", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterClinicalEventsCountUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/clinical-events/count", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterClinicalEventsUri(String facilityCode, String staffId, String assigningAuthority, String patientId, String clinicalEventId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/clinical-events/id/%s", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId, clinicalEventId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
	
	private URI getNoteWriterSignClinicalEventUri(String facilityCode, String staffId, String assigningAuthority, String patientId) {
		String uri = String.format("%s/note-writer/clinical-event-service/site/%s/id/%s/patient/%s/%s/clinical-events/sign", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId, assigningAuthority, patientId);
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
