package com.agilex.healthcare.notewriter.restservice;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Calendar;
import java.util.Date;

import javax.ws.rs.core.UriBuilder;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointment;
import com.agilex.healthcare.mobilehealthplatform.domain.Appointments;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;

public class NoteWriterAppointmentsResourceTest {
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void checkWhetherTestsAreRunningInCIHMode() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}
	
	@AfterClass
	public static void tearDown() {
		if (client != null) {
			try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}
	
	/**
	 * Retrieves the appointments for the given patient.  If startDate and endDate are not specified the default
	 * behavior is to retrieve all appointments that are one year prior to today up to the end of today.
	 */
	@Test
	public void fetchPatientAppointments() {
		try {
			String assigningAuthority = "EDIPI";
			String patientId = "1";
			URI uri = getNoteWriterAppointmentsUri(assigningAuthority, patientId);
			Appointments appts = client.getJerseyClient().resource(uri).get(Appointments.class);
			assertNotNull(appts);
			assertTrue(appts.size() > 0);
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR_OF_DAY, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.set(Calendar.MILLISECOND, 0);
			cal.add(Calendar.YEAR, -1);
			Date lastYear = cal.getTime();
			cal.add(Calendar.YEAR, +1);
			cal.add(Calendar.DATE, 1);
			Date tomorrow = cal.getTime();
			
			for (Appointment appt : appts) {
				Date apptDate = appt.getDate();
				assertTrue(lastYear.before(apptDate));
				assertTrue(tomorrow.after(apptDate));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	/**
	 * Validate that the filters for stop and start date work.
	 */
	@Test
	public void fetchPatientAppointmentsFiltered() {
		try {
			String assigningAuthority = "EDIPI";
			String patientId = "1";
			URI uri = getNoteWriterAppointmentsUri(assigningAuthority, patientId, "01/01/2014", "02/25/2014");
			Appointments appts = client.getJerseyClient().resource(uri).get(Appointments.class);
			assertNotNull(appts);
			assertTrue(appts.size() > 0);
			
			Calendar cal = Calendar.getInstance();
			cal.set(Calendar.HOUR_OF_DAY, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			cal.set(Calendar.MILLISECOND, 0);
			cal.set(Calendar.YEAR, 2014);
			cal.set(Calendar.MONTH, 0);
			cal.set(Calendar.DATE, 1);
			Date jan1st = cal.getTime();
			cal.set(Calendar.MONTH, 1);
			cal.set(Calendar.DATE, 26);
			Date feb26 = cal.getTime(); //One day more
			
			for (Appointment appt : appts) {
				Date apptDate = appt.getDate();
				assertTrue(jan1st.before(apptDate));
				assertTrue(feb26.after(apptDate));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	private URI getNoteWriterAppointmentsUri(String assigningAuthority, String patientId) {
		URI uri = getNoteWriterAppointmentsUri(assigningAuthority, patientId, null, null);
		return uri;
	}
	private URI getNoteWriterAppointmentsUri(String assigningAuthority, String patientId, String startDate, String endDate) {
		String uri = String.format("%s/note-writer/patient/%s/%s/appointments/visits",
				IntegrationTestConfiguration.getMobileHealthServerBaseUri(), 
				assigningAuthority,
				patientId);
		if (NullChecker.isNotNullish(startDate)) {
			uri = uri.concat("?startDate=" + startDate);
		}
		if (NullChecker.isNotNullish(endDate)) {
			uri = uri.concat("&endDate=" + endDate);
		}
		URI notesUri = UriBuilder.fromUri(uri).build();
		return notesUri;
	}
}
