package com.agilex.healthcare.patientsearch.restservice;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.net.URI;
import java.util.Date;

import javax.ws.rs.core.UriBuilder;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.agilex.healthcare.mobilehealthplatform.clientapi.AssertJersey;
import com.agilex.healthcare.mobilehealthplatform.clientapi.MobileHealthClientTestVersion;
import com.agilex.healthcare.mobilehealthplatform.domain.DOBDate;
import com.agilex.healthcare.mobilehealthplatform.domain.Patient;
import com.agilex.healthcare.mobilehealthplatform.domain.PatientIdentifier;
import com.agilex.healthcare.mobilehealthplatform.domain.RecentlySelectedPatients;
import com.agilex.healthcare.mobilehealthplatform.domain.SelectedPatient;
import com.agilex.healthcare.testutility.IntegrationTestConfiguration;
import com.agilex.healthcare.testutility.TestHelper;
import com.agilex.healthcare.utility.DateHelper;
import com.agilex.healthcare.utility.NullChecker;
import com.sun.jersey.api.client.UniformInterfaceException;

public class PatientSearchResourceTest {
	private static final org.apache.commons.logging.Log LOGGER = org.apache.commons.logging.LogFactory.getLog(PatientSearchResourceTest.class);
	
	private static MobileHealthClientTestVersion client;
	private static String deleteTokenURI = null;

	@BeforeClass
	public static void checkWhetherTestsAreRunningInCIHMode() {
		client = TestHelper.createApprovedMdwsProviderMobileHealthClient();
		deleteTokenURI = TestHelper.getDeleteTokenURI();
	}

	@Before
	public void setup() {
		PatientSearchUtilities.removeTestDataFromMongo(LOGGER);
	}

	@After
	public void teardown() {
		PatientSearchUtilities.removeTestDataFromMongo(LOGGER);
	}
	
	@AfterClass
	public static void tearDown() {
	    if (client != null) {
	    	try {
				client.getJerseyClient().resource(deleteTokenURI).delete();
			}
			catch (UniformInterfaceException e) {
				AssertJersey.fail(e);
			}
			client.getJerseyClient().destroy();
			client = null;
		}
	}

	@Test
	public void testFetchSelectedPatientsFromInvalidFacility() {
		try {
			URI uri = getPatientSearchRecentUri("invalidFacility", "staff001");
			RecentlySelectedPatients selectedPatients = client.getJerseyClient().resource(uri).get(RecentlySelectedPatients.class);
			assertEquals(0, selectedPatients.size());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	@Test
	public void testFetchSelectedPatients() {
		try {
			Date before = new Date();
			String facilityCode = "TESTDATA";
			String staffId = "staff046";
			addPatient(staffId, "EDIPI", "DTESTDATA1", "MAY 05, 1900", "TEST1", "DATA01", "TESTDATA1", "MALE");
			addPatient(staffId, "EDIPI", "DTESTDATA2", "MAY 05, 1901", "TEST2", "DATA01", "TESTDATA2", "MALE");
			addPatient(staffId, "EDIPI", "DTESTDATA3", "MAY 05, 1902", "TEST3", "DATA01", "TESTDATA3", "MALE");
			URI uri = getPatientSearchRecentUri(facilityCode, staffId);
			RecentlySelectedPatients selectedPatients = client.getJerseyClient().resource(uri).get(RecentlySelectedPatients.class);
			assertEquals(3, selectedPatients.size());
			assertEquals(facilityCode, selectedPatients.getProvider().getFacilityCode());
			assertEquals(staffId, selectedPatients.getProvider().getStaffId());
			
			Date after = new Date();
			for (SelectedPatient selectedPatient : selectedPatients) {
				assertTrue("Date on record is not before when this test started (is after or equals to)", selectedPatient.getSearchedTime().after(before));

				assertTrue("searchedTime should always be before now", after.after(selectedPatient.getSearchedTime()));
				assertNotNull(selectedPatient.getPatient());
				assertNotNull(selectedPatient.getPatient().getPatientIdentifier());
				assertTrue(NullChecker.isNotNullish(selectedPatient.getPatient().getPatientIdentifier().getAssigningAuthority()));
				assertTrue(NullChecker.isNotNullish(selectedPatient.getPatient().getPatientIdentifier().getUniqueId()));
				assertTrue(NullChecker.isNotNullish(selectedPatient.getPatient().getDisplayName()));
			}
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}
	
	@Test
	public void testAddSelectedPatient() {
		try {
			Date beforeSavingDate = new Date();
			SelectedPatient selectedPatient = addPatient("staff046", "EDIPI", "DTESTDATA1", "MAY 05, 1900", "TEST1", "DATA01", "TESTDATA1", "MALE");
			
			assertNotNull(selectedPatient);
			assertTrue(!selectedPatient.getSearchedTime().before(beforeSavingDate));
			DOBDate dob = new DOBDate(null, "MAY 05, 1900");
			assertNotNull(selectedPatient.getPatient());
			assertEquals(dob, selectedPatient.getPatient().getDateOfBirth());
			assertEquals(String.valueOf(DateHelper.calculateDeltaInYears(dob.getDate(), new Date())), selectedPatient.getPatient().getAge());
			assertEquals("TESTDATA1", selectedPatient.getPatient().getSsn());
			assertEquals("MALE", selectedPatient.getPatient().getGender().toUpperCase());
			assertEquals("DATA01, TEST1", selectedPatient.getPatient().getDisplayName());
		}
		catch (UniformInterfaceException e) {
			AssertJersey.fail(e);
		}
	}

	private SelectedPatient addPatient(String staffId, String assigningAuthority, String uniqueId, String dateOfBirth,
			String firstName, String lastName, String ssn, String gender) {
		DOBDate dob = new DOBDate(null, dateOfBirth);
		Patient patient = new Patient();
		patient.setPatientIdentifier(new PatientIdentifier(assigningAuthority, uniqueId));
		patient.setDateOfBirth(dob);
		patient.setFirstName(firstName);
		patient.setLastName(lastName);
		patient.setSsn(ssn);
		patient.setGender(gender);
		
		URI uri = getPatientSearchRecentUri("TESTDATA", staffId);
		SelectedPatient selectedPatient = client.getJerseyClient().resource(uri).type("application/xml").accept("application/xml").post(SelectedPatient.class, patient);
		return selectedPatient;
	}
        
	private URI getPatientSearchRecentUri(String facilityCode, String staffId) {
		String uri = String.format("%s/patient-search/site/%s/id/%s/recently-selected-patients", IntegrationTestConfiguration.getMobileHealthServerBaseUri(), facilityCode, staffId);
		URI recentUri = UriBuilder.fromUri(uri).build();
		return recentUri;
	}
}
